/************************************************************************/
/*BEGINPROLOGUE************************************************************************
 * @copyright(disclaimer)                                                             *
 * Licensed Materials - Property of IBM                                               *
 * IBM Enterprise Information Portal for Multiplatforms V8.1 (program number 5724-B43)*
 * (c ) Copyright IBM Corp. 1998,  2002. All Rights Reserved.                         *
 *                                                                                    *
 * US Government Users Restricted Rights                                              *
 * Use, duplication or disclosure restricted by GSA ADP Schedule                      *
 * Contract with IBM Corp.                                                            *
 *                                                                                    *
 * DISCLAIMER OF WARRANTIES :                                                         *
 *                                                                                    *
 * Permission is granted to copy and modify this Sample code, and to                  *
 * distribute modified versions provided that both the copyright                      *
 * notice,- and this permission notice and warranty disclaimer appear in all copies   * 
 * and modified versions.                                                             *
 *                                                                                    *
 * THIS SAMPLE CODE IS LICENSED TO YOU AS-IS.                                         *
 * IBM AND ITS SUPPLIERS AND LICENSORS DISCLAIM ALL WARRANTIES, EITHER EXPRESS OR     *
 * IMPLIED, IN SUCH SAMPLE CODE, INCLUDING THE WARRANTY OF NON-INFRINGEMENT AND THE   *
 * IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE. IN NO   *
 * EVENT WILL IBM OR ITS LICENSORS OR SUPPLIERS BE LIABLE FOR ANY DAMAGES ARISING OUT *
 * OF THE USE OF OR INABILITY TO USE THE SAMPLE CODE, DISTRIBUTION OF THE SAMPLE CODE,*
 * OR COMBINATION OF THE SAMPLE CODE WITH ANY OTHER CODE. IN NO EVENT SHALL IBM OR ITS*
 * LICENSORS AND SUPPLIERS BE LIABLE FOR ANY LOST REVENUE, LOST PROFITS OR DATA, OR   *
 * FOR DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL,INCIDENTAL OR PUNITIVE DAMAGES,       *
 * HOWEVER CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY,-, EVEN IF IBM OR ITS     *
 * LICENSORS OR SUPPLIERS HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.       *
 *                                                                                    *
 * @endCopyright                                                                      *
 **************************************************************************************/
/************************************************************************/

/*
 * Filename       : TCallbackExecuteDL.cpp
 * Description    : Connect to DL Datastore.
 *                  Create query string.
 *                  Specify to retrieve folder and documents.
 *                  Specify to retrieve a max result of 20 items.
 *                  Get the result in a resultset cursor.
 *                  Loop thru results.
 *                  Display documents found and their parts and attributes,
 *                   if folders found display them and their documents/
 *                   folders.
 *                  Disconnect from DL Datastore.
 */

#include <iostream.h>
#include <stdio.h>                
#include <DKConstant.h>
#include <TCallbackDL.hpp>
#include <DKDatastoreDL.hpp>
#include <DKException.hpp>
#include <DKString.hpp>
#include <dkResultSetCursor.hpp>
#include <DKDDO.hpp>
#include <DKAny.hpp>
#include <DKPid.hpp>
#include <dkCollection.hpp>
#include <DKNVPair.hpp>
#include <DKBlobDL.hpp>
#include <DKPidXDODL.hpp>
#include <DKEnvironment.hpp>
#include <stdio.h>                

void main(int argc, char *argv[])
{
    cout << "begin..." << endl;

    // Indicate this is a Console Application on Windows NT/95 for error
    // message code page conversion to OEM code page. If this was a Windows
    // GUI (Graphic User Interface) Application this would not be necessary
    // because DK_SS_WINDOWS is the default.
    DKEnvironment::setSubSystem(DK_SS_CONSOLE);

    //-------------------------------------------------------------- 
    // Checking for input parameters
    //--------------------------------------------------------------   
    if (argc < 4)
    {
     cout<<"Usage: "<<endl;
     cout<<"  TCallbackExecuteDL <libsrv> <userid> <pw>" << endl;
     return;
    }
    // ### get input parameters
    DKString libsrv = DKString(argv[1]);
    DKString userid = DKString(argv[2]);
    DKString pw = DKString(argv[3]);

    DKDatastoreDL dsDL; // must be outside try/catch block

    try {
    TCallbackDL cbkDL;
    DKNVPair* parms = 0;
    dkResultSetCursor* pCur = 0;
    DKException* exc1 = 0;
    cout << "Datastore DL created" << endl;
    cout << "connecting to datastore" << endl;
    dsDL.connect(libsrv,userid,pw);
    cout << "datastore connected " << libsrv << " userid - " << userid << endl;
    DKString cmd = "SEARCH=(INDEX_CLASS=DLSAMPLE,";
    cmd += "MAX_RESULTS=20,";
    cmd += "COND=(DLSEARCH_DocType <> NULL));";
    cmd += "OPTION=(CONTENT=YES;";
    cmd += "TYPE_QUERY=STATIC;TYPE_FILTER=FOLDERDOC)";
    cout << "query string " << cmd << endl;
    cout << "executing query" << endl;
    dsDL.executeWithCallback(cmd,DK_CM_PARAMETRIC_QL_TYPE,parms,
                        (dkCallback*)&cbkDL);
    cout << "query executed" << endl;
    DKBoolean isDone = FALSE;
    while (isDone == FALSE)
    {
     cout << "press <enter> to continue... " << isDone << endl;
     getchar();
     isDone = cbkDL.isDone();
     cout << "isDone ??? " << isDone << endl;
    }

    exc1 = cbkDL.getException();
    if (exc1 != 0)
    {
       cout << "Error found " << exc1->text(0) << endl;
       delete exc1;
       cbkDL.resetException();
    }

    dsDL.disconnect();
    cout << "datastore disconnected" << endl;
    }
    catch(DKException &exc)
    {

      cout << "Error id " << exc.errorId() << endl;
      cout << "Exception id " << exc.exceptionId() << endl;
      for(unsigned long i = 0; i< exc.textCount();i++)
      {
       cout << "Error text: " << exc.text(i) << endl;
      }
      for (unsigned long g=0; g< exc.locationCount();g++)
      {
       const DKExceptionLocation* p = exc.locationAtIndex(g);
       cout << "Filename: " << p->fileName() << endl;
       cout << "Function: " << p->functionName() << endl;
       cout << "LineNumber: " << p->lineNumber() << endl;
      }
      cout << "Exception Class Name: " << exc.name() << endl;
    }
    cout << "done ..." << endl;

}

