/************************************************************************/
/*BEGINPROLOGUE************************************************************************
 * @copyright(disclaimer)                                                             *
 * Licensed Materials - Property of IBM                                               *
 * IBM Enterprise Information Portal for Multiplatforms V8.1 (program number 5724-B43)*
 * (c ) Copyright IBM Corp. 1998,  2002. All Rights Reserved.                         *
 *                                                                                    *
 * US Government Users Restricted Rights                                              *
 * Use, duplication or disclosure restricted by GSA ADP Schedule                      *
 * Contract with IBM Corp.                                                            *
 *                                                                                    *
 * DISCLAIMER OF WARRANTIES :                                                         *
 *                                                                                    *
 * Permission is granted to copy and modify this Sample code, and to                  *
 * distribute modified versions provided that both the copyright                      *
 * notice,- and this permission notice and warranty disclaimer appear in all copies   * 
 * and modified versions.                                                             *
 *                                                                                    *
 * THIS SAMPLE CODE IS LICENSED TO YOU AS-IS.                                         *
 * IBM AND ITS SUPPLIERS AND LICENSORS DISCLAIM ALL WARRANTIES, EITHER EXPRESS OR     *
 * IMPLIED, IN SUCH SAMPLE CODE, INCLUDING THE WARRANTY OF NON-INFRINGEMENT AND THE   *
 * IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE. IN NO   *
 * EVENT WILL IBM OR ITS LICENSORS OR SUPPLIERS BE LIABLE FOR ANY DAMAGES ARISING OUT *
 * OF THE USE OF OR INABILITY TO USE THE SAMPLE CODE, DISTRIBUTION OF THE SAMPLE CODE,*
 * OR COMBINATION OF THE SAMPLE CODE WITH ANY OTHER CODE. IN NO EVENT SHALL IBM OR ITS*
 * LICENSORS AND SUPPLIERS BE LIABLE FOR ANY LOST REVENUE, LOST PROFITS OR DATA, OR   *
 * FOR DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL,INCIDENTAL OR PUNITIVE DAMAGES,       *
 * HOWEVER CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY,-, EVEN IF IBM OR ITS     *
 * LICENSORS OR SUPPLIERS HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.       *
 *                                                                                    *
 * @endCopyright                                                                      *
 **************************************************************************************/
/************************************************************************/

/*
 * Filename       : TCreateStructDocIndexTS.cpp
 * Description    : Perform doc model creation functions and
 *                  text search index creation for an index
 *                  that supports structured documents.
 *                  Note: You will need to modify the document
 *                  model name everytime you run this. Since
 *                  you cannot create 2 models with the same
 *                  name.
 *                  
 */


#include <iostream.h>
#include <stdio.h>                
#include <DKConstantDL.h>
#include <DKDatastoreTS.hpp>
#include <DKDatastoreDefTS.hpp>
#include <DKDatastoreAdminTS.hpp>
#include <DKIndexingRulesTS.hpp>
#include <DKSearchIndexDefTS.hpp>
#include <DKDocModelTS.hpp>
#include <DKDocSectionTS.hpp>
#include <DKException.hpp>
#include <DKString.hpp>
#include <DKEnvironment.hpp>

void main(int argc, char *argv[])
{
   cout << "begin..." << endl;
   // Indicate this is a Console Application on Windows NT/95 for error
   // message code page conversion to OEM code page. If this was a Windows
   // GUI (Graphic User Interface) Application this would not be necessary
   // because DK_SS_WINDOWS is the default.
   DKEnvironment::setSubSystem(DK_SS_CONSOLE);

    //-------------------------------------------------------------- 
    // Checking for input parameters
    //--------------------------------------------------------------   
    if (argc < 3)
    {
     cout<<"Usage: "<<endl;
     cout<<"  TCreateStructDocIndexTS <srchSrv> <srchIndex>" << endl;
     return;
    }
    // ### get input parameters
    DKString srchSrv = DKString(argv[1]);
    DKString strIndex = DKString(argv[2]);

    DKDatastoreTS dsTS; //must be outside try/catch block

    try {
      DKDatastoreDefTS* dsDef = 0;
      DKDatastoreAdminTS* dsAdmin = 0;
      DKSearchIndexDefTS* pEnt = 0;
      DKIndexingRulesTS* indexRules = new DKIndexingRulesTS();
      DKDocModelTS* docModel = new DKDocModelTS();
      DKDocModelTS* docModel2 = new DKDocModelTS();
      DKDocSectionTS* docSection = new DKDocSectionTS();
      DKDocSectionTS* docSection2 = new DKDocSectionTS();
      cout << "Datastore TS created" << endl;
      indexRules->setIndexName(strIndex);
      docModel->setCCSID(DK_TS_CCSID_00850);
      docModel->setName("SAMPCORPMOD");
      docModel2->setCCSID(DK_TS_CCSID_00850);
      docModel2->setName("SAMPCORPMOD");
      docSection->setName("SAMPCORPTITLE");
      docSection->setTag("TITLE");
      docModel->addDocSection(docSection);
      docSection2->setName("SAMPCORPBODY");
      docSection2->setTag("BODY");
      docModel->addDocSection(docSection2);
      indexRules->setDefaultDocFormat(DK_TS_DOCFMT_HTML);
      indexRules->setDefaultDocModel(docModel2);
      cout << "connecting to datastore" << endl;
      dsTS.connect(srchSrv,"","");
      cout << "datastore connected " << srchSrv << endl;
      dsDef = (DKDatastoreDefTS*)dsTS.datastoreDef();
      dsAdmin = (DKDatastoreAdminTS*)dsDef->datastoreAdmin();
      cout << "create doc model for instance" << endl;
      dsAdmin->createDocModel("",docModel);
      cout << "doc model created for instance" << endl;

      pEnt = new DKSearchIndexDefTS(&dsTS);
      cout << "create index " << strIndex << endl;
      pEnt->setName(strIndex);
      //    pEnt->setIndexType(DK_TS_INDEX_TYPE_GTR);
      pEnt->setIndexType(DK_TS_INDEX_TYPE_PRECISE);
      //    pEnt->setIndexFeature(DK_TS_INDEX_FEATURE_CASE_ENABLED);
      // pEnt->setIndexCCSID(DK_TS_CCSID_00850);
      pEnt->setIndexProperty(DK_TS_PROPERTY_SECTIONS_ENABLED);
      pEnt->setLibraryId("LIBSUM");
      //**** for AIX ***************************************
      //pEnt->setLibraryId("LSMUFASA");
      pEnt->setLibraryClientServices("IMLLSCDL");
      pEnt->setLibraryServerServices("IMLLSSDL");
      DKString strDefaultDir = "e:\\tsindex\\";
      DKString strIndexFileDir = strDefaultDir;
      strIndexFileDir = strIndexFileDir + "index\\";
      strIndexFileDir = strIndexFileDir + strIndex;
      //**** for AIX ***************************************
      //DKString strDefaultDir = "/frnhome/cltadmin/tsindex/";
      //DKString strIndexFileDir = strDefaultDir;
      //strIndexFileDir = strIndexFileDir + "index/";
      //strIndexFileDir = strIndexFileDir + strIndex;
      pEnt->setIndexDataArea(strIndexFileDir);
      DKString strWorkFileDir = strDefaultDir + "work\\";
      strWorkFileDir = strWorkFileDir + strIndex;
      //**** for AIX ***************************************
      //DKString strWorkFileDir = strDefaultDir + "work/";
      //strWorkFileDir = strWorkFileDir + strIndex;
      pEnt->setIndexWorkArea(strWorkFileDir);
      pEnt->addDocModel(docModel);
      dsDef->add(pEnt);
      cout << "created index " << strIndex << endl;
 
      cout << "set indexing rules for " << strIndex << endl;
      dsAdmin->setIndexingRules(indexRules);    
      cout << "indexing rules set for " << strIndex << endl;
      delete indexRules; 
      delete pEnt;

      dsTS.disconnect();
      cout << "datastore disconnected" << endl;
   }
   catch(DKException &exc)
   {
      cout << "Error id" << exc.errorId() << endl;
      for(unsigned long i = 0; i< exc.textCount();i++)
      {
         cout << "Error text:" << exc.text(i) << endl;
      }
      for (unsigned long g=0; g< exc.locationCount();g++)
      {
         const DKExceptionLocation* p = exc.locationAtIndex(g);
         cout << "Filename: " << p->fileName() << endl;
         cout << "Function: " << p->functionName() << endl;
         cout << "LineNumber: " << p->lineNumber() << endl;
      }
      cout << "Exception Class Name: " << exc.name() << endl;
   }
   
   cout << "done ..." << endl;
}

