/************************************************************************/
/*BEGINPROLOGUE************************************************************************
 * @copyright(disclaimer)                                                             *
 * Licensed Materials - Property of IBM                                               *
 * IBM Enterprise Information Portal for Multiplatforms V8.1 (program number 5724-B43)*
 * (c ) Copyright IBM Corp. 1998,  2002. All Rights Reserved.                         *
 *                                                                                    *
 * US Government Users Restricted Rights                                              *
 * Use, duplication or disclosure restricted by GSA ADP Schedule                      *
 * Contract with IBM Corp.                                                            *
 *                                                                                    *
 * DISCLAIMER OF WARRANTIES :                                                         *
 *                                                                                    *
 * Permission is granted to copy and modify this Sample code, and to                  *
 * distribute modified versions provided that both the copyright                      *
 * notice,- and this permission notice and warranty disclaimer appear in all copies   * 
 * and modified versions.                                                             *
 *                                                                                    *
 * THIS SAMPLE CODE IS LICENSED TO YOU AS-IS.                                         *
 * IBM AND ITS SUPPLIERS AND LICENSORS DISCLAIM ALL WARRANTIES, EITHER EXPRESS OR     *
 * IMPLIED, IN SUCH SAMPLE CODE, INCLUDING THE WARRANTY OF NON-INFRINGEMENT AND THE   *
 * IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE. IN NO   *
 * EVENT WILL IBM OR ITS LICENSORS OR SUPPLIERS BE LIABLE FOR ANY DAMAGES ARISING OUT *
 * OF THE USE OF OR INABILITY TO USE THE SAMPLE CODE, DISTRIBUTION OF THE SAMPLE CODE,*
 * OR COMBINATION OF THE SAMPLE CODE WITH ANY OTHER CODE. IN NO EVENT SHALL IBM OR ITS*
 * LICENSORS AND SUPPLIERS BE LIABLE FOR ANY LOST REVENUE, LOST PROFITS OR DATA, OR   *
 * FOR DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL,INCIDENTAL OR PUNITIVE DAMAGES,       *
 * HOWEVER CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY,-, EVEN IF IBM OR ITS     *
 * LICENSORS OR SUPPLIERS HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.       *
 *                                                                                    *
 * @endCopyright                                                                      *
 **************************************************************************************/
/************************************************************************/

/*
 * Filename       : TADLsmSample.cpp
 * Description    : test add, del, list schema mapping 
 *
 */

#include <iostream.h>
#include <stdio.h>                
#include <DKConstant.h>
#include <DKString.hpp>
#include <DKAny.hpp>
#include <DKDatastoreFed.hpp>
#include <DKDatastoreDefFed.hpp>
#include <DKException.hpp>
#include <DKEnvironment.hpp>
#include <DKAttrDefFed.hpp>
#include <DKEntityDefFed.hpp>
#include <DKAttrMappingFed.hpp>
#include <DKAttrMapDef.hpp>
#include <dkResultSetCursor.hpp>
#include <DKDatastoreAdminFed.hpp>
#include <DKServerMapDefFed.hpp>

void main(int argc, char *argv[])
{
   cout << "begin..." << endl;

   // Indicate this is a Console Application on Windows NT/95 for error
   // message code page conversion to OEM code page. If this was a Windows
   // GUI (Graphic User Interface) Application this would not be necessary
   // because DK_SS_WINDOWS is the default.
   DKEnvironment::setSubSystem(DK_SS_CONSOLE);

   //-------------------------------------------------------------- 
   // Checking for input parameters
   //--------------------------------------------------------------   
   if (argc < 5)  {
      cout <<"Usage syntax: "<<endl;
      cout <<"\t TADLsmSample <feddb> <userid> <pw> <add/del/list>" << endl;
      return;
   }
   // ### get input parameters
   DKString libsrv(argv[1]);
   DKString userid(argv[2]);
   DKString pw    (argv[3]);
   DKString cmd   (argv[4]);
   DKDatastoreFed dsFed; // must be outside try block to avoid exception trap error.
   
   try  {
   
      // All set; Connect and execute the compound query expression
      cout << " *** connecting to " << libsrv << " *** " << endl;
      dsFed.connect(libsrv, userid, pw);
      cout << " \t -----> connected " << endl;
      cout << "     user " << dsFed.userName() << ", dsName " << dsFed.datastoreName() << endl;
   
      // get the Def
      dkDatastoreDef*  defFed = dsFed.datastoreDef();
     
      // create a federated entity : FedSample
      DKEntityDefFed* entityObj = new DKEntityDefFed(&dsFed);        
      entityObj->setName("FedSample");
      entityObj->setDescription("FedSample to map to DLSAMPLE at LSMUFASA and LSCHIUN");
     
     // create a federated attributes :
      dkAttrDef* fAuthor = new DKAttrDefFed();
      fAuthor->setEntityName("FedSample");
      fAuthor->setName("FedAuthor");
      fAuthor->setType(DK_CM_VSTRING);
      fAuthor->setId((short)95);
      fAuthor->setSize(50);
      fAuthor->setScale(1);
      fAuthor->setPrecision(50);
      fAuthor->setNullable(TRUE);
      fAuthor->setStringType(DK_ATTR_ALPHA);
     
      dkAttrDef* fTitle = new DKAttrDefFed();
      fTitle->setEntityName("FedSample");
      fTitle->setName("FedTitle");
      fTitle->setType(DK_CM_VSTRING);
      fTitle->setId((short)99);
      fTitle->setSize(100);
      fTitle->setScale(1);
      fTitle->setPrecision(100);
      fTitle->setNullable(TRUE);
      fTitle->setStringType(DK_ATTR_ALPHA);
     
      dkAttrDef* fSubject = new DKAttrDefFed();
      fSubject->setEntityName("FedSample");
      fSubject->setName("FedSubject");
      fSubject->setType(DK_CM_VSTRING);
      fSubject->setId((short)100);
      fSubject->setSize(100);
      fSubject->setScale(1);
      fSubject->setPrecision(100);
      fSubject->setNullable(TRUE);
      fSubject->setStringType(DK_ATTR_ALPHA);


      dkAttrDef* fAuthor1 = new DKAttrDefFed();
      fAuthor1->setEntityName("FedSample");
      fAuthor1->setName("FedAuthor");
      fAuthor1->setType(DK_CM_VSTRING);
      fAuthor1->setId((short)95);
      fAuthor1->setSize(50);
      fAuthor1->setScale(1);
      fAuthor1->setPrecision(50);
      fAuthor1->setNullable(TRUE);
      fAuthor1->setStringType(DK_ATTR_ALPHA);
     
      dkAttrDef* fTitle1 = new DKAttrDefFed();
      fTitle1->setEntityName("FedSample");
      fTitle1->setName("FedTitle");
      fTitle1->setType(DK_CM_VSTRING);
      fTitle1->setId((short)99);
      fTitle1->setSize(100);
      fTitle1->setScale(1);
      fTitle1->setPrecision(100);
      fTitle1->setNullable(TRUE);
      fTitle1->setStringType(DK_ATTR_ALPHA);
     
      dkAttrDef* fSubject1 = new DKAttrDefFed();
      fSubject1->setEntityName("FedSample");
      fSubject1->setName("FedSubject");
      fSubject1->setType(DK_CM_VSTRING);
      fSubject1->setId((short)100);
      fSubject1->setSize(100);
      fSubject1->setScale(1);
      fSubject1->setPrecision(100);
      fSubject1->setNullable(TRUE);
      fSubject1->setStringType(DK_ATTR_ALPHA);
      
      // associate data sources to this entity:
      // get the Fed DS Def and Admin
      DKDatastoreAdminFed* adm = (DKDatastoreAdminFed*) defFed->datastoreAdmin();

       // Now, interpret the command: add, del, or list
      if (cmd == "add") {
         entityObj->add();        // adding an entity to the federated database
         entityObj->add(fAuthor);
         entityObj->add(fTitle);
         entityObj->add(fSubject);
         cout << " *** federated entity and attributes created " << endl;
      }
     
      // list federated entity:

      cout << " *** federated entities :" << endl;
      DKSequentialCollection* pCol = (DKSequentialCollection*) dsFed.listEntities();
      DKSequentialCollection* pCol2;
      dkIterator* pIter = pCol->createIterator();
      dkIterator* pIter2;
      dkEntityDef* enDef;
      dkAttrDef* attrDef;
      DKString entName;
      int k = 0, j = 0, i = 0;
      while (pIter->more()) {
         k++;
         enDef = (dkEntityDef*) ((void *) (*pIter->next())); 
         entName = enDef->getName();
         cout << " Entity Def name [" << k << "] - " << entName << endl;
         cout << "  list attributes for " << entName << " entity" << endl; 
         pCol2 = (DKSequentialCollection*) dsFed.listEntityAttrs(entName);
         pIter2 = pCol2->createIterator();
         j = 0;
         while (pIter2->more()) {
            j++;
            attrDef = (dkAttrDef*)  ((void *) (*pIter2->next()));
            cout << "  Attribute name [" << j << "] - " << attrDef->getName()          << endl;
            cout << "    datastoreType : "              << attrDef->datastoreType()    << endl;
            cout << "    attributeOf   : "              << attrDef->getEntityName()    << endl;
            cout << "    type          : "              << attrDef->getType()          << endl;
            cout << "    size          : "              << attrDef->getSize()          << endl;
            cout << "    id            : "              << attrDef->getId()            << endl;
            cout << "    nullable      : "              << attrDef->isNullable()       << endl;
            cout << "    precision     : "              << attrDef->getPrecision()     << endl;
            cout << "    scale         : "              << attrDef->getScale()         << endl;
            cout << "    stringType    : "              << attrDef->getStringType()    << endl;
            delete attrDef;
         }  
         delete pIter2;
         delete pCol2; 
         delete enDef;
         cout << "  " << j << " attr listed for " << entName << " entity name"  << endl;
      }
      delete pIter;
      delete pCol;
     
      // create native attributes in-memory : DL index_class DLSAMPLE at mufasa
      dkAttrDef* nmAuthor = new DKAttrMapDef();
      nmAuthor->setEntityName("DLSAMPLE");
      nmAuthor->setName("DLSEARCH_Author");
      nmAuthor->setType(DK_CM_VSTRING);
      nmAuthor->setId((short)95);
      nmAuthor->setSize(50);
      nmAuthor->setScale(1);
      nmAuthor->setPrecision(50);
      nmAuthor->setNullable(TRUE);
      nmAuthor->setStringType(DK_ATTR_ALPHA);
     
      dkAttrDef* nmTitle = new DKAttrMapDef();
      nmTitle->setEntityName("DLSAMPLE");
      nmTitle->setName("DLSEARCH_Title");
      nmTitle->setType(DK_CM_VSTRING);
      nmTitle->setId((short)99);
      nmTitle->setSize(100);
      nmTitle->setScale(1);
      nmTitle->setPrecision(100);
      nmTitle->setNullable(TRUE);
      nmTitle->setStringType(DK_ATTR_ALPHA);
     
      dkAttrDef* nmSubject = new DKAttrMapDef();
      nmSubject->setEntityName("DLSAMPLE");
      nmSubject->setName("DLSEARCH_Subject");
      nmSubject->setType(DK_CM_VSTRING);
      nmSubject->setId((short)100);
      nmSubject->setSize(100);
      nmSubject->setScale(1);
      nmSubject->setPrecision(100);
      nmSubject->setNullable(TRUE);
      nmSubject->setStringType(DK_ATTR_ALPHA);
     
      cout << " *** native attributes defined at LSMUFASA, DLSAMPLE " << endl;
     
      // create native attributes in-memory : DL index_class DLSAMPLE at talisman2
      dkAttrDef* ncAuthor = new DKAttrMapDef();
      ncAuthor->setEntityName("DLSAMPLE");
      ncAuthor->setName("DLSEARCH_Author");
      ncAuthor->setType(DK_CM_VSTRING);
      ncAuthor->setId((short)95);
      ncAuthor->setSize(50);
      ncAuthor->setScale(1);
      ncAuthor->setPrecision(50);
      ncAuthor->setNullable(TRUE);
      ncAuthor->setStringType(DK_ATTR_ALPHA);
     
      dkAttrDef* ncTitle = new DKAttrMapDef();
      ncTitle->setEntityName("DLSAMPLE");
      ncTitle->setName("DLSEARCH_Title");
      ncTitle->setType(DK_CM_VSTRING);
      ncTitle->setId((short)99);
      ncTitle->setSize(100);
      ncTitle->setScale(1);
      ncTitle->setPrecision(100);
      ncTitle->setNullable(TRUE);
      ncTitle->setStringType(DK_ATTR_ALPHA);
     
      dkAttrDef* ncSubject = new DKAttrMapDef();
      ncSubject->setEntityName("DLSAMPLE");
      ncSubject->setName("DLSEARCH_Subject");
      ncSubject->setType(DK_CM_VSTRING);
      ncSubject->setId((short)100);
      ncSubject->setSize(100);
      ncSubject->setScale(1);
      ncSubject->setPrecision(100);
      ncSubject->setNullable(TRUE);
      ncSubject->setStringType(DK_ATTR_ALPHA);
     
      cout << " *** native attributes defined at LSCHIUN, DLSAMPLE " << endl;
     
      //  create schema mapping : entity and attr mappings 
      //   from FedSample to DLSAMPLE in LSMUFASA
      dkAttrMapping* mapAuthor = new DKAttrMappingFed();
      mapAuthor->addMappedAttr(fAuthor);
      mapAuthor->addMapToAttr(nmAuthor);
     
      dkAttrMapping* mapTitle = new DKAttrMappingFed();
      mapTitle->addMappedAttr(fTitle);
      mapTitle->addMapToAttr(nmTitle);
     
      dkAttrMapping* mapSubject = new DKAttrMappingFed();
      mapSubject->addMappedAttr(fSubject);
      mapSubject->addMapToAttr(nmSubject);
     
      cout << " ** attribute mapping to LSMUFASA defined " << endl;
     
      dkSchemaMapping* msm = new DKSchemaMappingFed(&dsFed);
      msm->setServerName("LSMUFASA");
      msm->setServerType(DK_DSTYPE_DL);
      msm->setEntityName("DLSAMPLE");
      msm->setMappedEntityName("FedSample");
     
      // add attribute mapping to this schema
      msm->addAttrMapping(mapAuthor);
      msm->addAttrMapping(mapTitle);
      msm->addAttrMapping(mapSubject);
     
      if (cmd == "add") {
         msm->add();
         cout << " ** schema mapping for LSMUFASA added " << endl;
      }
      else if (cmd == "del") {
         try {
            msm->del();
            cout << " ** schema mapping for LSMUFASA deleted " << endl;
         }
         catch(DKException exc) { 
            cout << "Error id " << exc.errorId() << endl;
            for(unsigned long i = 0; i< exc.textCount();i++)
               cout << "Error text: " << exc.text(i) << endl;
      
            for (unsigned long g=0; g< exc.locationCount();g++) {
               const DKExceptionLocation* p = exc.locationAtIndex(g);
               cout << "Filename: " << p->fileName() << endl;
               cout << "Function: " << p->functionName() << endl;
               cout << "LineNumber: " << p->lineNumber() << endl;
               }
      
            cout << "Exception Class Name: " << exc.name() << endl;
            
         }
      }
      //  create schema mapping : entity and attr mappings 
      //   from FedSample to DLSAMPLE in LSCHIUN
      dkAttrMapping* cmapAuthor = new DKAttrMappingFed();
      cmapAuthor->addMappedAttr(fAuthor1);
      cmapAuthor->addMapToAttr(ncAuthor);
     
      dkAttrMapping* cmapTitle = new DKAttrMappingFed();
      cmapTitle->addMappedAttr(fTitle1);
      cmapTitle->addMapToAttr(ncTitle);
     
      dkAttrMapping* cmapSubject = new DKAttrMappingFed();
      cmapSubject->addMappedAttr(fSubject1);
      cmapSubject->addMapToAttr(ncSubject);
     
      cout << " ** attribute mapping to LSCHIUN defined " << endl;
     
      dkSchemaMapping* csm = new DKSchemaMappingFed(&dsFed);
      csm->setServerName("LSCHIUN");
      csm->setServerType(DK_DSTYPE_DL);
      csm->setEntityName("DLSAMPLE");
      csm->setMappedEntityName("FedSample");
     
      // add attribute mapping to this schema
      csm->addAttrMapping(cmapAuthor);
      csm->addAttrMapping(cmapTitle);
      csm->addAttrMapping(cmapSubject);
     
      if (cmd == "add") {
         csm->add();
         cout << " ** schema mapping for LSCHIUN added " << endl;
      }
      else if (cmd == "del") {
         try {
            csm->del();
            cout << " ** schema mapping for LSCHIUN deleted " << endl;
         }
         catch(DKException exc) { 
            cout << "Error id " << exc.errorId() << endl;
            for(unsigned long i = 0; i< exc.textCount();i++)
               cout << "Error text: " << exc.text(i) << endl;
      
            for (unsigned long g=0; g< exc.locationCount();g++) {
               const DKExceptionLocation* p = exc.locationAtIndex(g);
               cout << "Filename: " << p->fileName() << endl;
               cout << "Function: " << p->functionName() << endl;
               cout << "LineNumber: " << p->lineNumber() << endl;
            }
      
            cout << "Exception Class Name: " << exc.name() << endl;
              
         }
         // Delete Schema mapping before delete entity object,
         entityObj->del(fSubject);
         entityObj->del(fTitle);
         entityObj->del(fAuthor);
         entityObj->del();        // deleting an entity from the federated database
         cout << " *** federated entity and attributes deleted " << endl;
      } 


      // list schema mappings in this entity:
      cout << " list schema mappings in this entity : " << entityObj->getName() << endl;
      long count;
      DKString* smnames = entityObj->listSchemaMappingNames(count);
      if (count != 0 && smnames != NULL) {
         for (i = 0; i < count; i++) {
            cout << " SM name : " << smnames[i] << endl;
         }
         delete [] smnames;
      } 
      else cout << " no sm defined in this entity " << endl;

    
      // retrieve all schema mapping objects for this entity
      dkCollection* coll = entityObj->listSchemaMappings();
      dkIterator* iter;
      if (coll != NULL && coll->cardinality() > 0) {
         iter = coll->createIterator();
         cout << " " << endl;
         cout << "-- list AttrMapping -- " << endl;
         dkAttrMapping*   _am;
         dkAttrDef*       _ad;
         dkSchemaMapping* _sm;
         DKAny *elm, *elm1;
         while (iter->more()) {
            elm = iter->next();
            _sm = (dkSchemaMapping*) elm->value();

            cout << "\n *** Schema Mapping Name : " << _sm->getName() << endl;
            dkCollection* amCol = _sm->listAttrMappings();
            dkIterator* amIter = amCol->createIterator();
            while (amIter->more()) {
               elm1= amIter->next();
               _am = (dkAttrMapping*) elm1->value();
               cout << " " << endl;
               cout << "  ++ AttrMapping Name : " << _am->getName()        << endl;
               cout << "    -- To Function : "    << _am->getToNative()    << endl;
               cout << "    -- From Function : "  << _am->getFromNative()  << endl;
               cout << " -- list Mapped Attr : "  << endl;
               int num = _am->numberOfMappedAttr();
               cout << "\t Mapped attr size = " << num  << endl;
               for (i = 0; i < num; i++) {
                  _ad = (dkAttrDef*)_am->getMappedAttr((short)(i+1));
                  cout << "  ++ Mapped Attr Name   : " << _ad->getName()      << endl;
                  cout << "    -- Mapped Atrr Type : " << _ad->getType()      << endl;
                  cout << "    -- Size             : " << _ad->getSize()      << endl;
                  cout << "    -- Precision        : " << _ad->getPrecision() << endl;
                  cout << "    -- Scale            : " << _ad->getScale()     << endl;
                  cout << "    -- Max              : " << _ad->getMax()       << endl;
                  cout << "    -- Min              : " << _ad->getMin()       << endl;
                  cout << "    -- Nullable         : " << _ad->isNullable()   << endl;
               }
               cout << " -- list MapTo Attr -- " << endl;
               num = _am->numberOfMapToAttr();
               cout << "\t MapTo attr size = " << num << endl;
               for (i = 0; i < num; i++) {
                  _ad = (dkAttrDef*)_am->getMapToAttr((short)(i+1));
                  cout << "  ++ MapTo Attr Name   : " << _ad->getName()      << endl;
                  cout << "    -- MapTo Atrr Type : " << _ad->getType()      << endl;
                  cout << "    -- Size            : " << _ad->getSize()      << endl;
                  cout << "    -- Precision       : " << _ad->getPrecision() << endl;
                  cout << "    -- Scale           : " << _ad->getScale()     << endl;
                  cout << "    -- Max             : " << _ad->getMax()       << endl;
                  cout << "    -- Min             : " << _ad->getMin()       << endl;
                  cout << "    -- Searchable      : " << _ad->isQueryable()  << endl;
               }  
            }
            delete _sm;
            delete amIter;
            delete amCol;
         }  
         delete iter;

      } 
      else 
         cout << " schema mapping is null " << endl;
      delete coll;

      delete msm;
      delete csm;
      delete entityObj;

      // disconnects
       cout << "\n\n *** disconnect from the datastore Fed. ***" << endl;
      dsFed.disconnect();
      cout << " *** datastore Fed disconnected ***" << endl;
   }
   catch(DKException &exc)
      {
      //   exc.printTrace();
      cout << "Error id " << exc.errorId() << endl;
      for(unsigned long i = 0; i< exc.textCount();i++)
         cout << "Error text: " << exc.text(i) << endl;
      
      for (unsigned long g=0; g< exc.locationCount();g++) {
         const DKExceptionLocation* p = exc.locationAtIndex(g);
         cout << "Filename: " << p->fileName() << endl;
         cout << "Function: " << p->functionName() << endl;
         cout << "LineNumber: " << p->lineNumber() << endl;
      }
     
      cout << "Exception Class Name: " << exc.name() << endl;
   }
 
   cout << "done ..." << endl;   
}
