/*BEGINPROLOGUE************************************************************************
 * @copyright(disclaimer)                                                             *
 * Licensed Materials - Property of IBM                                               *
 * IBM Information Integrator for Content V8.2  (program number 5724-B43)             *
 * (c ) Copyright IBM Corp. 1998, 2002, 2003. All Rights Reserved.                    *
 *                                                                                    *
 * US Government Users Restricted Rights                                              *
 * Use, duplication or disclosure restricted by GSA ADP Schedule                      *
 * Contract with IBM Corp.                                                            *
 *                                                                                    *
 * DISCLAIMER OF WARRANTIES :                                                         *
 *                                                                                    *
 * Permission is granted to copy and modify this Sample code, and to                  *
 * distribute modified versions provided that both the copyright                      *
 * notice,- and this permission notice and warranty disclaimer appear in all copies   * 
 * and modified versions.                                                             *
 *                                                                                    *
 * THIS SAMPLE CODE IS LICENSED TO YOU AS-IS.                                         *
 * IBM AND ITS SUPPLIERS AND LICENSORS DISCLAIM ALL WARRANTIES, EITHER EXPRESS OR     *
 * IMPLIED, IN SUCH SAMPLE CODE, INCLUDING THE WARRANTY OF NON-INFRINGEMENT AND THE   *
 * IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE. IN NO   *
 * EVENT WILL IBM OR ITS LICENSORS OR SUPPLIERS BE LIABLE FOR ANY DAMAGES ARISING OUT *
 * OF THE USE OF OR INABILITY TO USE THE SAMPLE CODE, DISTRIBUTION OF THE SAMPLE CODE,*
 * OR COMBINATION OF THE SAMPLE CODE WITH ANY OTHER CODE. IN NO EVENT SHALL IBM OR ITS*
 * LICENSORS AND SUPPLIERS BE LIABLE FOR ANY LOST REVENUE, LOST PROFITS OR DATA, OR   *
 * FOR DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL,INCIDENTAL OR PUNITIVE DAMAGES,       *
 * HOWEVER CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY,-, EVEN IF IBM OR ITS     *
 * LICENSORS OR SUPPLIERS HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.       *
 *                                                                                    *
 * @endCopyright                                                                      *
 **************************************************************************************

Deleting a Work Node
    In order to delete a Work Node, any Work Lists & any Processes using the Work Node
    must first be deleted before the Work Node is deleted.

********************************************************************/

// Includes / Imports
#include <iostream.h>
#include <DKConstant.h>
#include <DKDatastoreICM.hpp>
#include <DKDocRoutingServiceMgmtICM.hpp>
#include "SConnectDisconnectICM.hpp"
#include "SDocRoutingDefinitionDeletionICM.hpp"

/************************************************************************************************
 *          FILENAME: SDocRoutingDefinitionDeletionICM.cpp
 *                    ---------------------------------------------------------------------------
 *       DESCRIPTION: Delete a Routing Process Definition, Work Node Definitions, & Work List
 *                    Definitions.  Deletes All Created by SDocRoutingDefinitionCreationICM.
 *                    ---------------------------------------------------------------------------
 *     DEMONSTRATION: Delete a Routing Process Definition
 *                    Delete a Work Node Definition
 *                    Delete a Work List Definition
 *                    Deletes All Created by SDocRoutingDefinitionCreationICM
 *                    ---------------------------------------------------------------------------
 * COMMANDLINE USAGE: SDocRoutingDefinitionDeletionICM <database> <userName> <password>
 *                    ---------------------------------------------------------------------------
 *      PREREQUISITS: The Doc Routing Definition should already exist.  If not, please run:
 *                      - SDocRoutingDefinitionCreationICM
 *                    ---------------------------------------------------------------------------
 *    FILES REQUIRED: SConnectDisconnectICM.hpp/cpp
 *                    SDocRoutingDefinitionDeletionICM_launch.cpp
 *                    SDocRoutingDefinitionDeletionICM.hpp
 ************************************************************************************************/

    //-------------------------------------------------------------
    // Main
    //-------------------------------------------------------------
    /**
     * Run the Sample.
     * @param argc   Argument Count / Number of Arguments Specified
     * @param argv[] String Array containing arguments.  Optional arguments are <databse> <userName> <password>.
     * @return  Returns SConnectDisconnectICM_RC_SAMPLE_PASSED (0) if no sample program completed, SConnectDisconnectICM_RC_SAMPLE_FAILED (-1) if any errors / exceptions were caught.
     **/
    int SDocRoutingDefinitionDeletionICM::main(int argc, char *argv[]){

        // Defaults for connecting to the database.
        char * database = SConnectDisconnectICM_DEFAULT_DATABASE;
        char * userName = SConnectDisconnectICM_DEFAULT_USERNAME;
        char * password = SConnectDisconnectICM_DEFAULT_PASSWORD;

        //------------------------------------------------------------
        // Checking for input parameters
        //--------------------------------------------------------------
        if (argc < 4) { // if not all 3 arguments were specified, use defaults and report correct usage.
            cout << "Usage: "                                                               << endl;
            cout << "  SDocRoutingDefinitionDeletionICM <database> <userName> <password>"   << endl;
            cout << "  *** Some parameters not specified, using defaults..."                << endl;
            cout << endl;
        } else {  // otherwise enough parameters were specified, use the first 3 for the 3 parameters.
            if (argc > 1) database = argv[1];
            if (argc > 2) userName = argv[2];
            if (argc > 3) password = argv[3];
        }//end else
        char * ver = SConnectDisconnectICM_EIP_VERSION;

        cout << "================================================="     << endl;
        cout << "IBM Information Integrator for Content       v" << ver << endl;
        cout << "Sample Program:  SDocRoutingDefinitionDeletionICM"     << endl;
        cout << "-------------------------------------------------"     << endl;
        cout << " Database: " << database                               << endl;
        cout << " UserName: " << userName                               << endl;
        cout << "=================================================\n"   << endl;

        try{
            //-------------------------------------------------------------
            // Connect to datastore
            //-------------------------------------------------------------
            // See Sample SConnectDisconnectICM for more information
            cout << "Connecting to datastore (Database '" << database << "', UserName '" << userName << "')..." << endl;

                DKDatastoreICM* dsICM = new DKDatastoreICM();  // Create new datastore object.
                dsICM->connect(database,userName,password,""); // Connect to the datastore.

            cout << "Connected to datastore (Database '" << dsICM->datastoreName() << "', UserName '" << dsICM->userName() << "')." << endl;

            //---------------------------------------------------------------------
            // Routing Management / Definition Object
            //---------------------------------------------------------------------
            // DKDocRoutingServiceMgmtICM:
            //                          Provides methods to manage DKProcessICM, 
            //                          DKWorkNodeICM, and DKWorkListICM.
            cout << "Creating Routing Service & Management Objects..." << endl;

                DKDocRoutingServiceMgmtICM* routingMgmt = new DKDocRoutingServiceMgmtICM(dsICM); // Obtain the routing management object.

            cout << "Created Routing Service & Management Objects." << endl;

            //---------------------------------------------------------------------
            // Delete Work Lists
            //---------------------------------------------------------------------
            // Delete the Work Lists Defined in the Creation Sample
            cout << "Deleting Work List Definitions..." << endl;

                routingMgmt->delWorkList("S_fillClaimWL");
                routingMgmt->delWorkList("S_agentWL");
            
            cout << "Deleted Work List Definitions." << endl;

            //---------------------------------------------------------------------
            // Delete Routing Process
            //---------------------------------------------------------------------
            // Delete the Process Defined in the Creation Sample
            cout << "Deleting Routing Process Definition..." << endl;

                routingMgmt->delProcess("S_claimProcess");
            
            cout << "Deleted Routing Process Definition." << endl;

            //---------------------------------------------------------------------
            // Delete Work Nodes
            //---------------------------------------------------------------------
            // Delete the Work Nodes Defined in the Creation Sample
            // 
            // All Work Lists & Processes using this Work Node must already be
            // deleted.
            cout << "Deleting Work Node Definitions..." << endl;

                routingMgmt->delWorkNode("S_fillClaim");
                routingMgmt->delWorkNode("S_gatherAll");
                routingMgmt->delWorkNode("S_adjustment");

                delete(routingMgmt);  // Free Memory

            cout << "Deleted Work Node Definitions." << endl;

            //-------------------------------------------------------------
            // Disconnect from datastore & Destroy Reference
            //-------------------------------------------------------------
            // See Sample SConnectDisconnectICM for more information
            cout << "Disconnecting from datastore & destroying reference..." << endl;            

                dsICM->disconnect();
                delete(dsICM);

            cout << "Disconnected from datastore & destroying reference." << endl;
    
            //-------------------------------------------------------------
            // Sample program completed without exception
            //-------------------------------------------------------------
            cout << "\n==========================================" << endl;
            cout << "Sample program completed."                    << endl;
            cout << "==========================================\n" << endl;
        }
        //------------------------------------------------------------
        // Catch & Print Exceptions        
        //------------------------------------------------------------
        catch (DKException &exc){
            SConnectDisconnectICM::printException(exc);      // Print the exception to the screen
            return(SConnectDisconnectICM_RC_SAMPLE_FAILED);  // Report using return code that it failed
        }
        return(SConnectDisconnectICM_RC_SAMPLE_PASSED);      // If no exceptions were thrown, return a code to indicate passed.
    }// end main

    //=================================================================
    // Wrapper Functions
    //=================================================================
    // The following are wrapper functions for functionality covered
    // in this sample.  These functions may be used by other samples.

    // No additional wrapper functions from this sample.
