/*BEGINPROLOGUE************************************************************************
 * @copyright(disclaimer)                                                             *
 * Licensed Materials - Property of IBM                                               *
 * IBM Information Integrator for Content V8.2  (program number 5724-B43)             *
 * (c ) Copyright IBM Corp. 1998, 2002, 2003. All Rights Reserved.                    *
 *                                                                                    *
 * US Government Users Restricted Rights                                              *
 * Use, duplication or disclosure restricted by GSA ADP Schedule                      *
 * Contract with IBM Corp.                                                            *
 *                                                                                    *
 * DISCLAIMER OF WARRANTIES :                                                         *
 *                                                                                    *
 * Permission is granted to copy and modify this Sample code, and to                  *
 * distribute modified versions provided that both the copyright                      *
 * notice,- and this permission notice and warranty disclaimer appear in all copies   * 
 * and modified versions.                                                             *
 *                                                                                    *
 * THIS SAMPLE CODE IS LICENSED TO YOU AS-IS.                                         *
 * IBM AND ITS SUPPLIERS AND LICENSORS DISCLAIM ALL WARRANTIES, EITHER EXPRESS OR     *
 * IMPLIED, IN SUCH SAMPLE CODE, INCLUDING THE WARRANTY OF NON-INFRINGEMENT AND THE   *
 * IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE. IN NO   *
 * EVENT WILL IBM OR ITS LICENSORS OR SUPPLIERS BE LIABLE FOR ANY DAMAGES ARISING OUT *
 * OF THE USE OF OR INABILITY TO USE THE SAMPLE CODE, DISTRIBUTION OF THE SAMPLE CODE,*
 * OR COMBINATION OF THE SAMPLE CODE WITH ANY OTHER CODE. IN NO EVENT SHALL IBM OR ITS*
 * LICENSORS AND SUPPLIERS BE LIABLE FOR ANY LOST REVENUE, LOST PROFITS OR DATA, OR   *
 * FOR DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL,INCIDENTAL OR PUNITIVE DAMAGES,       *
 * HOWEVER CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY,-, EVEN IF IBM OR ITS     *
 * LICENSORS OR SUPPLIERS HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.       *
 *                                                                                    *
 * @endCopyright                                                                      *
 **************************************************************************************

Pre-requisit Informaiton:
    For information on Item Creation, Retrieval, Updating, & Deletion, please refer to 
    the SItemCreationICM, SItemRetrievalICM, SItemUpdateICM, & SItemDeletionICM Samples.

Folders
    A Folder is a DDO of Semantic Type "Folder", which contains a DKFolder Object.
    Specifically, the DDO has an attribute named DK_CM_DKFOLDER (DKConstant.h) which  
    contains a "DKFolder" object, a DKSequentialCollection that can contain DDOs.

    Item Types
        An Item can be created as a Folder for any Item Type.  An Item Type, such as
        the "S_withChild" Item Type used in the SItemCreationICM sample may have DDOs
        created of any Item Property Type or Semantic Type, including Document, Folder,
        Item, etc.
    
        NOTE: Please refer to SItemCreationICM for "Item Property Type" & "Semantic Type"
            information.
    
    Folders Full DDO Support
        A Folder is a complete DDO representing an Item created of an Item Type.  It
        has the full hierarchial data structure of that Item Type.  It may be used
        in all of the functions demonstrated in SItemCreationICM.  A Folder is a fully
        supported DDO with the additional functionality of a Folder.

Creating Folder Items
    The DKDatastoreICM.createDDO() method is used to create DDOs at runtime.  As shown
    in SItemCreationICM, when creating Folder Items, the "Item Property Type" or 
    "Semantic Type" simply needs to be specified as DK_CM_FOLDER (DKConstant.h).

    The DDO will automatically get a DKFolder attribute for the folder mechanism.

Updating Folder Contents
    Folder contents may be added or removed by manipulating the DKFolder collection
    within the DKFolder attribute.

    Incremental Changes Tracked
        All changes to the DKFolder collection object are tracked internally.  
        Each incremental change will be packaged up and sent to the Library Server
        in single call when the root component's DKDDO::update() function is called.
        It is important to note that the incremental changes are tracked and it is
        not the end result that is made persistent.  Optimal modifications rely on
        the fewest changes that result in the same end result.

    Adding Items to a Folder
        All contents placed in the DKFolder must be persistent in the datastore
        before an add() or update() is called on the Folder DDO.  This means that
        all Folder contents must have already had the DKDDO::add() operation called.
    
        The best and most efficient way to add Items to a Folder is to use the 
        DKFolder's addElement() function.  This allows any number of Folder 
        modifications to be grouped into a single call to the Library Server.

        REQUIREMENT:
            It is up to the User Program to use valid DDOs and to not add multiple
            copies of the same Item to the DKFolder.
    
        Immediate Results
            For Immediate results, use the DKFolder.addMember() or
            DKDatastoreExtICM's addToFolder() functions, which make the changes
            persistent immediately, but at an expensive cost of a call to the
            Library Server for every item added to the Folder.

    Removing Items from a Folder
        Removing an Item is performed simply by removing the undesired DKDDO
        object from the DKFolder collection, and then calling DKDDO::update()
        on the root item.
        
        The desired DDO must be located in the DKFolder collection by creating
        a dkIterator from the collection, and searching for it based on the ItemID
        in the PIDs of each DDO within.  If the collection is retrieved with
        attributes, the undesired DDO may be located based on User-Defined Attributes.
        DKFolder::removeElementAt(dkIterator) function may be used to remove the
        undesired DDO.

Retrieving a Folder
    Folder contents are not retrieved by default.  This allows greater control by
    the User Program to control all performance implications.  A retrieval option
    must be set to retrieve the contents.  If Folder contents are not retrieved,
    the DKFolder object & attribute will not exist until the program chooses 
    explicitly to retrieve Folder contents.  

    Retrieval Options: Outbound Links
        Folders require outbound links to be turned on.  The DKFolder is a 
        simplified interface to an Outbound Link Collection, which means that
        since it is implemented by the system using Links, Link options must
        be specified.
        
        Retrieval options that will accomplish this are either of the following:

            - DKConstant.DK_CM_CONTENT_LINKS_OUTBOUND
            - DKConstant.DK_CM_CONTENT_ITEMTREE
    
    Retrieve DKFolder Contents
        The retrieved folder contents will take the form of a DKFolder collection, 
        which contains non-retrieved DDOs.  The User Program must explicitly
        choose to retrieve each of the contents (DDOs) individually by calling 
        the retrieve() operation on each, or by using multi-item retrieve.  

Searching / Querying with Folder Contents
    Folder contents for the DKFolder are implemented as Links, but with a simplified,
    easy to use interface to make using a Folder as easy as possible.  Therefore, when
    searching / querying with Folder contents, searches must treat them as searches for
    Outbound Links using the DKFolder Link Type, DK_ICM_LINKTYPENAME_DKFOLDER (DKConstantICM.h).

    Use DKFolder Link Type for Outbound Links to search for DKFolder contents.

Obtaining Folders Containing a Particular Item
    The Datastore Extension's getFoldersContainingDDO() function returns a collection
    of all Folder Items that contain the specified DDO.  It builds, processes, and
    returns the Query / Search operation to accomplish this common task.

Links Alternative
    User Applications have the freedom to implement their own folder mechanism.  Links
    are a great tool with great versatility that can be easily adapted to implement
    behaviors such as folders.

Element vs. Member
    The DKSequentialCollection's "Element" methods are very different from the
    "Member" methods.  
    
    Element Methods
        "Element" methods are the recommended and most efficient method to use.
        Changes are not made persistent until the DKDDO.add() or DKDDO.update()
        methods are called.  All operations are grouped into one call to the 
        Library Server.
    
    Member Methods
        For Immediate results, use the "Member" methods.  Changes are made 
        persistent immediately, but at an expensive cost of a call to the 
        Library Server for every add or remove using these methods.
    
addElement() vs. addMember()
    This is an example of the "Element vs. Member" section above.

removeElement() vs. removeMember()
    This is an example of the "Element vs. Member" section above.

DKDatastoreExtICM.addToFolder() vs. addElement()
    The Datastore Extention's addToFolder() function is very much like the
    "Member" operations on the DKFolder.  This is a slightly different
    immediate, but expensive method.

DKDatastoreExtICM.addToFolder() vs. addMember()
    There is very little difference between these two methods.  The Datastore
    Extension's addToFolder() method may not affect the contents of the DKFolder
    object.

********************************************************************/

// Includes / Imports
#include <iostream.h>
#include <DKConstant.h>
#include <DKConstantICM.h>
#include <DKDatastoreICM.hpp>
#include <DKDatastoreExtICM.hpp>
#include <DKFolder.hpp>
#include "SConnectDisconnectICM.hpp"
#include "SFolderICM.hpp"

/************************************************************************************************
 *          FILENAME: SFolderICM.cpp
 *                    ---------------------------------------------------------------------------
 *       DESCRIPTION: Creating Folders, Adding Items to a Folder, Retrieving Folders, Retrieving
 *                    Folder Contents / Data, Element vs. Member, Brief on Searching Folders, &
 *                    Obtaining Folders Containing a Particular Item
 *                    ---------------------------------------------------------------------------
 *     DEMONSTRATION: Creating Folders
 *                    Adding Items to a Folder
 *                    Retrieving Folders
 *                    Retrieving Folder Contents / Data
 *                    Element vs. Member
 *                    Brief Explanation of Searching Folders
 *                    Obtaining Folders Containing a Particular Item
 *                    ---------------------------------------------------------------------------
 * COMMANDLINE USAGE: SFolderICM <database> <userName> <password>
 *                    ---------------------------------------------------------------------------
 *      PREREQUISITS: The Data Model must be defined.  If needed please run the following Samples  
 *                        - SAttributeDefinitionCreationICM
 *                        - SAttributeGroupDefCreationICM
 *                        - SReferenceAttrDefCreationICM
 *                        - SItemTypeCreationICM
 *                    ---------------------------------------------------------------------------
 *    FILES REQUIRED: SConnectDisconnectICM.hpp/cpp
 *                    SFolderICM_launch.cpp
 *                    SFolderICM.hpp
 ************************************************************************************************/

    //-------------------------------------------------------------
    // Main
    //-------------------------------------------------------------
    /**
     * Run the Sample.
     * @param argc   Argument Count / Number of Arguments Specified
     * @param argv[] String Array containing arguments.  Optional arguments are <databse> <userName> <password>.
     * @return  Returns SConnectDisconnectICM_RC_SAMPLE_PASSED (0) if no sample program completed, SConnectDisconnectICM_RC_SAMPLE_FAILED (-1) if any errors / exceptions were caught.
     **/
    int SFolderICM::main(int argc, char *argv[]){

        // Defaults for connecting to the database.
        char * database = SConnectDisconnectICM_DEFAULT_DATABASE;
        char * userName = SConnectDisconnectICM_DEFAULT_USERNAME;
        char * password = SConnectDisconnectICM_DEFAULT_PASSWORD;

        //------------------------------------------------------------
        // Checking for input parameters
        //--------------------------------------------------------------
        if (argc < 4) { // if not all 3 arguments were specified, use defaults and report correct usage.
            cout << "Usage: "                                                  << endl;
            cout << "  SFolderICM <database> <userName> <password>"            << endl;
            cout << "  *** Some parameters not specified, using defaults..."   << endl;
            cout << endl;
        } else {  // otherwise enough parameters were specified, use the first 3 for the 3 parameters.
            if (argc > 1) database = argv[1];
            if (argc > 2) userName = argv[2];
            if (argc > 3) password = argv[3];
        }//end else
        char * ver = SConnectDisconnectICM_EIP_VERSION;

        cout << "==========================================="     << endl;
        cout << "IBM Information Integrator for Content v" << ver << endl;
        cout << "Sample Program:  SFolderICM"                     << endl;
        cout << "-------------------------------------------"     << endl;
        cout << " Database: " << database                         << endl;
        cout << " UserName: " << userName                         << endl;
        cout << "===========================================\n"   << endl;

        try{
            //-------------------------------------------------------------
            // Connect to datastore
            //-------------------------------------------------------------
            // See Sample SConnectDisconnectICM for more information
            cout << "Connecting to datastore (Database '" << database << "', UserName '" << userName << "')..." << endl;

                DKDatastoreICM* dsICM = new DKDatastoreICM();  // Create new datastore object.
                dsICM->connect(database,userName,password,""); // Connect to the datastore.

            cout << "Connected to datastore (Database '" << dsICM->datastoreName() << "', UserName '" << dsICM->userName() << "')." << endl;

            //-------------------------------------------------------------
            // Creating a Folder
            //-------------------------------------------------------------
            // Simply specify "DK_CM_FOLDER" for the secont parameter of createDDO.
            cout << "Creating a Folder..." << endl;

	            DKDDO* ddoFolder  = dsICM->createDDO("S_simple", DK_CM_FOLDER);

	            ddoFolder->add(); // Save the DDO in the persistent datastore.

            cout << "Created a Folder." << endl;

            //-------------------------------------------------------------
            // Create Contents to Place in Folder
            //-------------------------------------------------------------
            // create 3 Items, one of Item Property type Document, one Folder, & one Item.
            cout << "Creating & Saving Sample Items..." << endl;

	            DKDDO* ddoDocument = dsICM->createDDO("S_simple", DK_CM_DOCUMENT);
	            DKDDO* ddoFolder2  = dsICM->createDDO("S_simple", DK_CM_FOLDER);   
	            DKDDO* ddoItem     = dsICM->createDDO("S_simple", DK_CM_ITEM);     

	            ddoDocument->add();
	            ddoItem->add();
	            ddoFolder2->add();

				DKString documentPidString = ((DKPidICM*)ddoDocument->getPidObject())->pidString(); 

            cout << "Created & Saved Sample Items." << endl;

            //-------------------------------------------------------------
            // Accessing the DKFolder Attribute
            //-------------------------------------------------------------
            // The DKFolder attribute is accessed the same way shown in 
            // SItemCreationICM & SItemRetrievalICM.
            //
            // This attribute can be found programatically as described in
            // Sample SItemRetrievalICM.
            cout << "Accessing the DKFolder Attribute..." << endl;
            
                DKFolder* dkFolder = (DKFolder*)(dkCollection*) ddoFolder->getData(ddoFolder->dataId(DK_CM_NAMESPACE_ATTR,DK_CM_DKFOLDER)); 
                
                // Important:  If the DDO is not retrieved with Folder contents,
                //             the above will fail due to a null pointer exception.
                //             The following accounts for this.
                
                short dataid = ddoFolder->dataId(DK_CM_NAMESPACE_ATTR,DK_CM_DKFOLDER);
                
                if(dataid==0)
                    throw DKException("No DKFolder Attribute Found!  DDO is either not a Folder or Folder Contents have not been explicitly retrieved.");
                    
                dkFolder = (DKFolder*)(dkCollection*) ddoFolder->getData(dataid); 

            cout << "Accessed the DKFolder Attribute." << endl;

            //-------------------------------------------------------------
            // Check out the Folder for Modification
            //-------------------------------------------------------------
            cout << "Checking Out / Locking Folder Before Update..." << endl;

                dsICM->checkOut(ddoFolder);  // Must check out / lock before updating.

            cout << "Checked Out / Locked Folder Before Update." << endl;

            //-------------------------------------------------------------
            // Adding Contents to Folder
            //-------------------------------------------------------------
            // As explained in the header, the "Element" methods below may
            // be replaced with "Member" methods, which are an Immediately
            // presistent result which does not require the following 
            // DKDDO.update(), but is very expive.  
            cout << "Adding Contents to Folder..." << endl;
            
                dkFolder->addElement(ddoDocument);
                dkFolder->addElement(ddoItem);
                dkFolder->addElement(ddoFolder2);  // Note, Folders can contain sub-folders.

            cout << "Added Contents to Folder." << endl;

            //-------------------------------------------------------------
            // Make changes persistent
            //-------------------------------------------------------------
            cout << "Making Changes Persistent..." << endl;
        
                ddoFolder->update();        // Don't forget to explicitly check in when finished.
                dsICM->checkIn(ddoFolder);  // May specify DKConstant.DK_CM_CHECKIN option or use the datastore method.

            cout << "Made Changes Persistent." << endl;

            //-------------------------------------------------------------
            // Printing Folder Contents
            //-------------------------------------------------------------
            cout << "Printing Folder Contents..." << endl;
                printFolderContents(ddoFolder);  // Use Wrapper Provided At Bottom of Sample
            cout << "Printed Folder Contents." << endl;

            //-------------------------------------------------------------
            // Check out the Folder for Modification
            //-------------------------------------------------------------
            cout << "Checking Out / Locking Folder Before Update..." << endl;

                dsICM->checkOut(ddoFolder);  // Must check out / lock before updating.

            cout << "Checked Out / Locked Folder Before Update." << endl;

            //-------------------------------------------------------------
            // Remove Contents from a Folder        
            //-------------------------------------------------------------
            // Please refer to the Application Programming Reference for 
            // a complete list of "DKFolder" Operations.
            cout << "Removing Contents from a Folder..." << endl;

                // Look for the Item to Remove.  In this case, we will look for 
                // the ddoItem.
                dkIterator* iter = dkFolder->createIterator();
                while(iter->more()){                      // while there are still items, continue searching
                    DKDDO* ddo = (DKDDO*) iter->next()->value();     // Move pointer to next element & return that object.
                    if(strcmp(((DKPidICM*)ddo->getPidObject())->pidString(),((DKPidICM*)ddoItem->getPidObject())->pidString())==0){

                        dkFolder->removeElementAt(*iter);  // Now that we found the ddoItem, remove it.
                        delete ddo;                        // must delete DDOs if taken out of a folder
                        break;                             // found the one we want, pop out
                    }
                }

                delete(iter); // Free Memory
            
            cout << "Removed Contents from a Folder." << endl;

            //-------------------------------------------------------------
            // Make Changes Persistent
            //-------------------------------------------------------------
            cout << "Making Changes Persistent..." << endl;
        
                ddoFolder->update();        // Don't forget to explicitly check in when finished.
                dsICM->checkIn(ddoFolder);  // May specify DKConstant.DK_CM_CHECKIN option or use the datastore method.

            cout << "Made Changes Persistent." << endl;

            //-------------------------------------------------------------
            // Recreate DDO to Demonstrate Retrieval
            //-------------------------------------------------------------
            cout << "Recreating Folder DDO..." << endl;
            
                DKString  ddoFolderPidString = ((DKPidICM*)ddoFolder->getPidObject())->pidString();  // Get the PID String
                delete(ddoFolder);    // Free Memory since this object is no longer needed.
                ddoFolder = dsICM->createDDO(ddoFolderPidString);                   // Recreate Blank DDO Using PID String.

            cout << "Recreated Folder DDO." << endl;

            //-------------------------------------------------------------
            // Retrieving Folder
            //-------------------------------------------------------------
            // This section covers retrieving a Folder Item with the 
            // DKFolder attribute / collection.
            cout << "Retrieving Folder..." << endl;
            
                ddoFolder->retrieve(DK_CM_CONTENT_LINKS_OUTBOUND);  // Outbound Links must be specified.

                // OR

                ddoFolder->retrieve(DK_CM_CONTENT_ITEMTREE);        // Item Tree includes Links & Folder Contents.

            cout << "Retrieved Folder." << endl;

            //-------------------------------------------------------------
            // Printing Folder Contents
            //-------------------------------------------------------------
            cout << "Printing Folder Contents..." << endl;
                printFolderContents(ddoFolder);  // Use Wrapper Provided At Bottom of Sample
            cout << "Printed Folder Contents." << endl;

            //-------------------------------------------------------------
            // Retrieving Folder Contents / Data
            //-------------------------------------------------------------
            // The DKFolder object will contain a list of non-retrieved 
            // DDOs.  Each must be retrieved if desired.  Multi-item retrieve
            // may be substituted.
            cout << "Retrieving Folder Contents / Data..." << endl;
            
                // Get the DKFolder object.        
                dataid = ddoFolder->dataId(DK_CM_NAMESPACE_ATTR,DK_CM_DKFOLDER); 
                dkFolder = (DKFolder*)(dkCollection*) ddoFolder->getData(ddoFolder->dataId(DK_CM_NAMESPACE_ATTR,DK_CM_DKFOLDER)); 
                
                // Use Multi-Item Retrieve for Maximum Performance
                dsICM->retrieveObjects(dkFolder);

            cout << "Retrieved Folder Contents / Data." << endl;
            
            //-------------------------------------------------------------
            // Obtaining All Folders Containing DDO
            //-------------------------------------------------------------
            // If you want to find out which folders contain a particular 
            // DDO, since contents can be stored in multiple places at the
            // same time, refer to the following.
            cout << "Obtaining All Folders Containing DDO..." << endl;

                DKDatastoreExtICM* dsExtICM = (DKDatastoreExtICM*) dsICM->getExtension(DK_CM_DATASTORE_EXT); // Get the Datastore Extension Object

				//Recreate the ddoDocument corresponding to the one created earlier
				ddoDocument = dsICM->createDDO(documentPidString);
				ddoDocument->retrieve();
                DKSequentialCollection* list = dsExtICM->getFoldersContainingDDO(ddoDocument); // Search for all folders that contain the ddoDocument Item.
                
                // Print List of IDs
                cout << "Folders Found:" << endl;
                iter = list->createIterator();          // create an iterator to go through list
                while(iter->more()){                    // while there are still items in the list, continue
                    DKDDO* ddo = (DKDDO*) iter->next()->value(); // Move pointer to next element & return that object.
                    cout << "      - Item ID:  " << ((DKPidICM*)ddo->getPidObject())->getItemId() << endl;
                    delete(ddo);                        // Free Memory
                }
                
                delete(list);                           // Free Memory
                delete(iter);                           // Contents of list not deleted when destructor is called.

            cout << "Obtained All Folders Containing DDO." << endl;

            //-------------------------------------------------------------
            // Clean Up Database After Sample
            //-------------------------------------------------------------
            // This sample will clean up / undo everything that stored
            // in the database.
            cout << "Cleaning Up..." << endl;

                ddoDocument->del();   
                ddoFolder->del();

                delete(ddoFolder);    // The folder will delete the DDOs that were added to it.
                delete(ddoDocument);  // This ddo was re-created based on pid sting, and must be deleted.

            cout << "Cleaned Up." << endl;

            //-------------------------------------------------------------
            // Disconnect from datastore & Destroy Reference
            //-------------------------------------------------------------
            // See Sample SConnectDisconnectICM for more information
            cout << "Disconnecting from datastore & destroying reference..." << endl;            

                dsICM->disconnect();
                delete(dsICM);

            cout << "Disconnected from datastore & destroying reference." << endl;
    
            //-------------------------------------------------------------
            // Sample program completed without exception
            //-------------------------------------------------------------
            cout << "\n==========================================" << endl;
            cout << "Sample program completed."                    << endl;
            cout << "==========================================\n" << endl;
        }
        //------------------------------------------------------------
        // Catch & Print Exceptions        
        //------------------------------------------------------------
        catch (DKException &exc){
            SConnectDisconnectICM::printException(exc);    // Print the exception to the screen
            return(SConnectDisconnectICM_RC_SAMPLE_FAILED);  // Report using return code that it failed
        }
        return(SConnectDisconnectICM_RC_SAMPLE_PASSED);    // If no exceptions were thrown, return a code to indicate passed.
    }// end main

    //=================================================================
    // Wrapper Functions
    //=================================================================
    // The following are wrapper functions for functionality covered
    // in this sample.  These functions may be used by other samples.

   /**
    * Print Folder Contents
    * @param folder  DDO of Semantic Type Folder.
    **/
    void SFolderICM::printFolderContents(DKDDO* folder){
        
        char* folderItemId = ((DKPidICM*)folder->getPidObject())->getItemId();

        // Get the DKFolder object.        
        short dataid = folder->dataId(DK_CM_NAMESPACE_ATTR,DK_CM_DKFOLDER);
        if(dataid==0)
            throw DKException("No DKFolder Attribute Found!  DDO is either not a Folder or Folder Contents have not been explicitly retrieved.");
        DKFolder* dkFolder = (DKFolder*)(dkCollection*) folder->getData(dataid); 
        
        // Print the list        
        cout << "Folder (" << folderItemId << ") Contents:" << endl;
        dkIterator* iter = dkFolder->createIterator(); // Create an Iterator go to through DDO Collection.
        while(iter->more()){                           // While there are items still to be printed, continue
            DKDDO* ddo = (DKDDO*) iter->next()->value();        // Move pointer to next element & return that object.
            cout << "     Item Id:  " << ((DKPidICM*)ddo->getPidObject())->getItemId() << " (" << ddo->getPidObject()->getObjectType() << ")" << endl;
        }
        delete iter;
    }
