/*BEGINPROLOGUE************************************************************************
 * @copyright(disclaimer)                                                             *
 * Licensed Materials - Property of IBM                                               *
 * IBM Information Integrator for Content V8.2  (program number 5724-B43)             *
 * (c ) Copyright IBM Corp. 1998, 2002, 2003. All Rights Reserved.                    *
 *                                                                                    *
 * US Government Users Restricted Rights                                              *
 * Use, duplication or disclosure restricted by GSA ADP Schedule                      *
 * Contract with IBM Corp.                                                            *
 *                                                                                    *
 * DISCLAIMER OF WARRANTIES :                                                         *
 *                                                                                    *
 * Permission is granted to copy and modify this Sample code, and to                  *
 * distribute modified versions provided that both the copyright                      *
 * notice,- and this permission notice and warranty disclaimer appear in all copies   * 
 * and modified versions.                                                             *
 *                                                                                    *
 * THIS SAMPLE CODE IS LICENSED TO YOU AS-IS.                                         *
 * IBM AND ITS SUPPLIERS AND LICENSORS DISCLAIM ALL WARRANTIES, EITHER EXPRESS OR     *
 * IMPLIED, IN SUCH SAMPLE CODE, INCLUDING THE WARRANTY OF NON-INFRINGEMENT AND THE   *
 * IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE. IN NO   *
 * EVENT WILL IBM OR ITS LICENSORS OR SUPPLIERS BE LIABLE FOR ANY DAMAGES ARISING OUT *
 * OF THE USE OF OR INABILITY TO USE THE SAMPLE CODE, DISTRIBUTION OF THE SAMPLE CODE,*
 * OR COMBINATION OF THE SAMPLE CODE WITH ANY OTHER CODE. IN NO EVENT SHALL IBM OR ITS*
 * LICENSORS AND SUPPLIERS BE LIABLE FOR ANY LOST REVENUE, LOST PROFITS OR DATA, OR   *
 * FOR DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL,INCIDENTAL OR PUNITIVE DAMAGES,       *
 * HOWEVER CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY,-, EVEN IF IBM OR ITS     *
 * LICENSORS OR SUPPLIERS HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.       *
 *                                                                                    *
 * @endCopyright                                                                      *
 **************************************************************************************

Pre-requisit Informaiton:
    For information on any of the following in terms of definition creation, accessing, 
    listing, printing, updating, or deleting, please refer to the SItemTypeCreationICM 
    Sample.
        - Datamodeling Concepts
        - Child Component Explanation
        - Multivalue Attribute Explanation
        - Item Type Classifications & Explanations
        - Resource XDO Classes
        - Item Types that store Resource Content
        - Delete Rules
        - Sample Data Model:  Insurance Model
        - +++
    
    This sample will assume knowledge and understanding of the concepts above and those
    outlined in the headers, wrappers, and functions in the pre-requisit samples:
        - SConnectDisconnectICM                  - SReferenceAttrDefCreationICM
        - SAttributeDefinitionCreationICM        - SReferenceAttrDefRetrievalICM	 
        - SAttributeDefinitionRetrievalICM       - SReferenceAttrDefUpdateICM
        - SAttributeDefinitionUpdateICM          - SReferenceAttrDefDeletionICM 
        - SAttributeDefinitionDeletionICM        - SItemTypeCreationICM
        - SAttributeGroupDefCreationICM          - SItemTypeRetrievalICM 
        - SAttributeGroupDefRetrievalICM         - SItemTypeUpdateICM  
        - SAttributeGroupDefUpdateICM            - SItemTypeDeletionICM   
        - SAttributeGroupDefDeletionICM    

Definitions:
    
    - "Items" refer in general to an instance of any Item Type.  
    - "Components" make up the hierarchial tree of data that makes up an Item.  
      Components are also the instances of multivalue attributes.  Each component is
      one value of a multi-value attribute.
    - "Root Component" refers to the highest level or parent DDO from which all child
      components are derived.
    - "Child" or "Child Component" refers to a sub-component of a parent.
    - "DDO", or "Dynamic Data Object" refers to an instance of the DKDDO class object
      which make up one Component (including Root Components).
    - "XDO", or "Extended Data Object" extends DDO objects for Resource Functionality.

Creating Items
    Items area created as DKDDOs.  DKDDOs should always be created using the 
    DKDatastoreICM's createDDO() methods.  Using the DKDatastoreICM's createDDO methods
    allow the system to automatically setup important information in the DKDDO structure 
    and which allows for easier & greater functionality such as Resources, 
    CM Document Model, & Folders.  Creating DDOs through the DKDatastoreICM is the 
    official and recommended way to create items.

    DKDatastoreICM.createDDO(<Overall Item Type>, <Item Property Type / Semantic Type>):
        Overall Item Type (Parameter 1):
            The first parameter takes the Overall Item Type, defining the entire Item.  
            This may also be considered the Root Item Type, since the DDO returned is
            the Root Component DDO.
        Item Property / Semantic Type (Parameter 2):
            The second parameter determines the Item Property & Semantic Type of the DDO, 
            such as Document, Folder, Item, or User-Defined.  For more information, 
            refer to the Item Property & Semantic Type sections below.
        Returned:
            The DDO returned is a new DKDDO of the Root Component.
    
    Resource Items
        The Item Type Definition determines what type of DDO is created.  If it is 
        a resource, the correct XDO will be returned, depending on the XDO Class 
        specified in the Item Type Definition.  The returned DDO may be cast to the
        correct structure, since will be setup correctly according to the Item Type
        Definition.  
        
        Please refer to the SItemTypeCreationICM sample for more information on Resource
        Classification, & XDO Classes.
    
    Item Properties / Item Type Properties
        An Item or DDO must have a generic "type" property associated with it.  This generic
        type of the DDO is Document, Folder, or Item.  When creating Items, the Item Property
        must be specified as the second parameter of the Datastore's createDDO function.
        
        The value is stored in the DDO's property named, "DK_CM_PROPERTY_ITEM_TYPE", which 
        should not be confused with the overall Item Type Definition that describes this 
        the structure of this Item.  "PROPERTY_ITEM_TYPE" refers to the "Item Property",
        not the Item Type Definition, something specific to the ICM connector.

        Available Item Property Types:

        Item Property   Constant        Definition  
        -------------   --------------  ------------------------------------------------------
        Document        DK_CM_DOCUMENT  Item will represent a document or stored data.  The 
                                        information contained in this item will may form a 
                                        document.  This can be considered a "Common Document" 
                                        since it does not rigidly mean an implementation of a
                                        specific document model.
        ......................................................................................
        Folder          DK_CM_FOLDER    Item will represent an object somehow containing or
                                        referencing contents / objects.  This can be 
                                        considered a "Common Folder" since it does not rigidly
                                        mean an implementation of a specific document model.
                                        In the ICM Connector, it will automatically create an
                                        empty DKFolder collection which can hold contents.
        ......................................................................................
        Item (Default)  DK_CM_ITEM	    Generic Item.  This is the Item Property Type for all
                                        Semantic Types that are not Document or Folder.
        --------------------------------------------------------------------------------------        
        Note:  Constants shown above are defined in DKConstant.h

	Semantic Type
	    When creating Items, a semantic type can be specified as the second parameter of 
	    the Datastore's createDDO function.  A semantic type in general is a metaphor for
	    an item.  It gives it an enforceable characteristic for basic operations.  Where 
	    applicable, semantic types are enforced within API and can be enforced 
	    externally as customers see fit.  

        The value is stored in the DDO's property DKConstantICM.DK_ICM_PROPERTY_SEMANTIC_TYPE,
        which in many cases may contain the same value as the Item Property Type.
	    
        Available Semantic Types:

        Semantic Type   Constant          Definition  
        -------------   ----------------  ------------------------------------------------
        <Any>           <User Specified>  CreateDDO will assign any value specified to 
                                          the semantic type value.  Since user-programs
                                          may enforce semantics as they wish, no values
                                          will be disallowed, except for Item Type
                                          Definitions of classification Document Model.
        ..................................................................................
        Document        DK_ICM_SEMANTIC_  Item will represent a document or stored data.  
                        TYPE_DOCUMENT     The information contained in this item will may
                                          form a document.  This can be considered a 
                                          "Common Document" since it does not rigidly 
                                          mean an implementation of a specific document 
                                          model.
        ..................................................................................
        Folder          DK_ICM_SEMANTIC_  Item will represent an object somehow 
                        TYPE_FOLDER       containing or referencing contents / objects.  
                                          This can be considered a "Common Folder" since
                                          it does not rigidly mean an implementation of a 
                                          specific document model.  In the ICM Connector,
                                          it will automatically create an empty DKFolder
                                          collection which can hold contents.
        ..................................................................................
        Container       DK_ICM_SEMANTIC_  Item will represent a container or generic item.
                        TYPE_CONTAINER    This is equivalent to the Item Property Type of
                                          DK_CM_ITEM.
        ..................................................................................
        Annotation      DK_ICM_SEMANTIC_  CM Document Model Annotation Part.  For Items
                        TYPE_ANNOTATION   created of an Item Type with a classification
                                          declaring CM Document Model, parts may be
                                          created and added to make up a document.
        ..................................................................................
        History         DK_ICM_SEMANTIC_  CM Document Model History Part.  For Items
                        TYPE_HISTORY      created of an Item Type with a classification
                                          declaring CM Document Model, parts may be
                                          created and added to make up a document.
        ..................................................................................
        Note            DK_ICM_SEMANTIC_  CM Document Model Note Part.  For Items
                        TYPE_NOTE         created of an Item Type with a classification
                                          declaring CM Document Model, parts may be
                                          created and added to make up a document.
        ..................................................................................
        Base            DK_ICM_SEMANTIC_  CM Document Model Base Part as a LOB Resource.
                        TYPE_BASE         For Items created of an Item Type with a
                                          classification declaring CM Document Model,
                                          parts may be created to make up a document.
        ..................................................................................
        Base Text       DK_ICM_SEMANTIC_  CM Document Model Base Part as a Text Resource.
                        TYPE_BASETEXT     For Items created of an Item Type with a
                                          classification declaring CM Document Model,
                                          parts may be created to make up a document.
        ..................................................................................
        Base Stream     DK_ICM_SEMANTIC_  CM Document Model Base Part as a Stream Resource.
                        TYPE_BASESTREAM   For Items created of an Item Type with a
                                          classification declaring CM Document Model,
                                          parts may be created to make up a document.
        ..................................................................................
        <User Defined>  <User Defined>    A User Defined Semantic Type.  Please refer to
                                          the Semantic Type Samples for more information.
        ----------------------------------------------------------------------------------        
        Note:  Constants show above are defined in DKConstantICM.h

Creating Child Components
    Child components are created as DKDDOs.  For the same reasons as Creating Items
    explained above, the DKDatastoreICM.createChildDDO() function.  It requires two
    parameters, the overall Item Type Name that it will be a part of, and the specific
    Component Type Name.  

    DKDatastoreICM.createChildDDO(<Overall Item Type>, <Child Component Type>):
        Overall Item Type (Parameter 1):
            The first parameter takes the Overall Item Type that defines the entire Item.  
            This may also be considered the Root Item Type.  It does not refer to the
            Item or Component Type of the parent.  Child Component Types have unique names,
            which will allow the system to figure out where it fits in the overall hierarchy.
        Child Component Type (Parameter 2):
            The second parameter takes the specific component type of the child component 
            to create.
        Returned:
            The DDO returned is a new DKDDO / Component of the specified Component Type.

Setting / Modifying Attributes
    When setting individual attributes, use the individual attribute definition name.
    In order to access attributes that belong to an Attribute Group, the correct 
    attribute name follows the following format:
    
        Grouped Attribute's Name Format:  <Attribute Group Name>.<Attribute Name>
        
        For more information, please refer to the SAttributeGroupDefCreationICM Sample.

    Set the attributes with the value passed in & retrieved as a DKAny, which encapsulates
    all possible data types.  Assignment operators are defined in DKAny.hpp which can automatically
    assign these data types to and in some cases from a DKAny object without requiring a DKAny object
    to be explicitly created or used by the User-Application.
    
    As stated in SAttributeDefinitionCreationICM, the following objects are used depending
    on the Attribute Type:
    
  	Attr Type   Constant           Object               DKAny Type Code Format (C++ Only)
	----------- ------------------ -------------------- --------------- --------------------------
	Blob        DK_CM_BLOB         DKAny(DKByteArray)   tc_bytearray    n/a
	Char        DK_CM_CHAR         DKAny(char)          tc_char         n/a
	Clob        DK_CM_CLOB         DKAny(DKString)      tc_string       n/a
	Date        DK_CM_DATE         DKAny(DKDate)        tc_date         yyyy-mm-dd
	Decimal     DK_CM_DECIMAL      DKAny(DKDecimal)     tc_decimal      n/a
	Double      DK_CM_DOUBLE       DKAny(double)        tc_double       n/a
	Integer     DK_CM_INTEGER      DKAny(long)          tc_long         n/a
	Short       DK_CM_SHORT        DKAny(short)         tc_short        n/a
	Time        DK_CM_TIME         DKAny(DKTime)        tc_time         hh.mm.ss
	Timestamp   DK_CM_TIMESTAMP    DKAny(DKTimestamp)   tc_timestamp    yyyy-mm-dd-hh.mm.ss.nnnnnn
	Varchar     DK_CM_VARCHAR      DKAny(DKString)      tc_string       n/a
    ----------------------------------------------------------------------------------------------
    Note:  Constants shown are defined in DKConstant.h

Setting / Modifying Reference Attributes
    Reference Attributes appear in a DDO just as attributes do.  They are set, 
    modified, and accessed just as attributes are, not attribute groups.  In 
    this case, a DKAny encapsulating a DKDDO object is passed in or retrieved.

	Attr Type            Object                   DKAny Type Code
	-------------------- ------------------------ -------------------
	Reference Attribute  DKAny(dkDataObjectBase)  tc_dobase

    Item Types
        Reference Attributes may not reference an Item of the same Item Type
        without restriction.  References are meant to reference Items of 
        different Item Types.  It is possible to reference an Item of the same
        Item Type, but setting & modifying the reference attribute must made
        on an item with all other changes already persistent, and modifications 
        to the reference attribute value only made persistent without other changes.
            
        Example Procedure For This Special Case:
            1. Create DDO A & B.
            2. Set any other values in A & B, including Adding Child Components.
            3. Add BOTH A & B to the datastore..
            4. Check Out / Lock A.
            5. Set A's Reference Attribute to B.
            6. Update A.

    Important:  At the time the Reference Attribute is set, the ddo it will reference must
                be persistent in the datastore.  This means that the DKDDO.add() operation
                has been completed on the DKDDO that it will reference, or it has been
                recreated given the full PIDs string (See SItemRetrievalICM Sample), or
                it has been retrieved through Retrieval or Query/Search.

    For a detailed explanation of Reference Attributes please refer to 
    SReferenceAttrDefCreationICM Sample.

DKAny & Conversion Operators
    In C++, DDO properties are always set and accessed as DKAny objects, which encapsulate
    all possible data types to be used with a DDO.

    Assignment operators are defined in DKAny.hpp which can automatically assign these
    data types to and (in some cases) from a DKAny object without requiring a DKAny object
    to be explicitly created or used by the User-Application.

    Example
        For example, "(dkCollection*)" is one such operator that can convert to and from DKAny
        so that the User-Application does not need to ever use DKAny in this case.

        DKChildCollection* childColl = (DKChildCollection*)(dkCollection*) ddo.getData(...);
        DKLinkCollection*   linkColl =  (DKLinkCollection*)(dkCollection*) ddo.getData(...);
        DKFolder*           dkFolder =          (DKFolder*)(dkCollection*) ddo.getData(...);

    Use DKAny to programmatically manage DDOs.

Detecting Non-Existant Attributes in DDOs
    If an attribute does not exist in a DDO, either because it was not defined in the 
    Item / Component Type Definition, or because it was not created through the 
    DKDatastoreICM.createDDO / createChildDDO function, the dataId() function will 
    return '0'.

Namespace
    Attribute names, Item / Component Type names, Link Type Name, etc., are in seperate 
    namespaces.  Names only need to be unique among their own namespace.  Therefore, when
    using names, the correct namespace must be used when common methods are used.  The 
    Attribute Namespace is the default.

    Namespaces needed when using DDOs:  (Not included here are those already seperated by
    the system.)

    Namespace   Constant
    ---------   --------------------------------------
    Attribute   DK_CM_NAMESPACE_ATTR  (default)
    Link        DK_CM_NAMESPACE_LINK
    Child       DK_CM_NAMESPACE_CHILD
    --------------------------------------------------
    Note:  Constants shown are defined in DKConstant.h

Adding / Saving New Items to the Persistent Datastore
    When new Items are created, they are only in local memory.  Once modification
    is complete, the entire item is added to the datastore through the root
    component / item.  Each child component should not be added individually, but
    instead simply the root item needs to be added.  The system will handle the rest.
    
    When modifying, updating, & deleting items, the root item / component should be
    used in all cases as a starting point for modification and end point for the add(),
    update(), and del() operations.

********************************************************************/

// Includes / Imports
#include <iostream.h>
#include <DKConstant.h>
#include <DKConstantICM.h>
#include <DKDatastoreICM.hpp>
#include <DKChildCollection.hpp>
#include "SConnectDisconnectICM.hpp"
#include "SItemCreationICM.hpp"

/************************************************************************************************
 *          FILENAME: SItemCreationICM.cpp
 *                    ---------------------------------------------------------------------------
 *       DESCRIPTION: Creating Items, Item Property, Semantic Type, Namespace, Setting Attributes,
 *                    Setting Attribute Group Values, Creating Child Components / Multivalue
 *                    Attributes, Setting Reference Attributes.
 *                    ---------------------------------------------------------------------------
 *     DEMONSTRATION: Item Properties / Item Type Properties
 *                    Semantic Types
 *                    Creating a Simple Item
 *                    Creating Items of Different Item Properties
 *                    Namespace
 *                    Setting Attributes (All Attribute Types)
 *                    Setting Attributes that belong to Attribute Groups
 *                    Setting / Modifying Reference Attributes
 *                    Creating an Item with a child component / multi-value attribute
 *                    Creating an Item with multiple Children / Values for a Mult-value attribute
 *                    ---------------------------------------------------------------------------
 * COMMANDLINE USAGE: SItemCreationICM <database> <userName> <password>
 *                    ---------------------------------------------------------------------------
 *      PREREQUISITS: The Data Model must be defined.  If needed please run the following Samples  
 *                        - SAttributeDefinitionCreationICM
 *                        - SAttributeGroupDefCreationICM
 *                        - SReferenceAttrDefCreationICM
 *                        - SItemTypeCreationICM
 *                    ---------------------------------------------------------------------------
 *    FILES REQUIRED: SConnectDisconnectICM.hpp/cpp
 *                    SItemCreationICM_launch.cpp
 *                    SItemCreationICM.hpp
 ************************************************************************************************/

    //-------------------------------------------------------------
    // Main
    //-------------------------------------------------------------
    /**
     * Run the Sample.
     * @param argc   Argument Count / Number of Arguments Specified
     * @param argv[] String Array containing arguments.  Optional arguments are <databse> <userName> <password>.
     * @return  Returns SConnectDisconnectICM_RC_SAMPLE_PASSED (0) if no sample program completed, SConnectDisconnectICM_RC_SAMPLE_FAILED (-1) if any errors / exceptions were caught.
     **/
    int SItemCreationICM::main(int argc, char *argv[]){

        // Defaults for connecting to the database.
        char * database = SConnectDisconnectICM_DEFAULT_DATABASE;
        char * userName = SConnectDisconnectICM_DEFAULT_USERNAME;
        char * password = SConnectDisconnectICM_DEFAULT_PASSWORD;

        //------------------------------------------------------------
        // Checking for input parameters
        //--------------------------------------------------------------
        if (argc < 4) { // if not all 3 arguments were specified, use defaults and report correct usage.
            cout << "Usage: "                                                  << endl;
            cout << "  SItemCreationICM <database> <userName> <password>"      << endl;
            cout << "  *** Some parameters not specified, using defaults..."   << endl;
            cout << endl;
        } else {  // otherwise enough parameters were specified, use the first 3 for the 3 parameters.
            if (argc > 1) database = argv[1];
            if (argc > 2) userName = argv[2];
            if (argc > 3) password = argv[3];
        }//end else
        char * ver = SConnectDisconnectICM_EIP_VERSION;

        cout << "==========================================="     << endl;
        cout << "IBM Information Integrator for Content v" << ver << endl;
        cout << "Sample Program:  SItemCreationICM"               << endl;
        cout << "-------------------------------------------"     << endl;
        cout << " Database: " << database                         << endl;
        cout << " UserName: " << userName                         << endl;
        cout << "===========================================\n"   << endl;

        try{
            //-------------------------------------------------------------
            // Connect to datastore
            //-------------------------------------------------------------
            // See Sample SConnectDisconnectICM for more information
            cout << "Connecting to datastore (Database '" << database << "', UserName '" << userName << "')..." << endl;

                DKDatastoreICM* dsICM = new DKDatastoreICM();  // Create new datastore object.
                dsICM->connect(database,userName,password,""); // Connect to the datastore.

            cout << "Connected to datastore (Database '" << dsICM->datastoreName() << "', UserName '" << dsICM->userName() << "')." << endl;

            //-------------------------------------------------------------
            // Create Items / DDOs / Root Components
            //-------------------------------------------------------------
            // create 3 Items, one of Item Property type Document, one Folder, & one Item.
            // See the documentation in the header for more information on Item Properties and
            // Semantic Types.  This section will use the simple Item Type with a Child, 
            // S_withChild Item Type created in SItemTypeCreationICM.
            //
            // User-Defined Semantic Types may be substituted here.  The Item Property will
            // default to "Item".
            //
            // For explanations of the parameters of the createDDO function, refer to the
            // "Creating Items" section in the header of this sample.
            cout << "Creating Items / DDOs / Root Components..." << endl;

	            DKDDO* ddoDocument = dsICM->createDDO("S_withChild", DK_CM_DOCUMENT); // createDDO(<Overall Item Type>, <Item Property / Semantic Type>);
	            DKDDO* ddoFolder   = dsICM->createDDO("S_withChild", DK_CM_FOLDER);   //   For more information, refer to header of this sample.
	            DKDDO* ddoItem     = dsICM->createDDO("S_withChild", DK_CM_ITEM);     // User-Defined Semantic Types may be substituted here.

            cout << "Created Items / DDOs / Root Components." << endl;

            //-------------------------------------------------------------
            // Create child components / DDOs / Multivalue Attributes
            //-------------------------------------------------------------
            // Create a child of component type "S_simpleChild" for the "S_withChild" Item Type.
            //
            // As explained in the SItemTypeCreationICM Sample, Multivalue Attributes are created
            // through Child Components.  Multile Child components for a single Component Type
            // demonstrate the multiple values of a multi-value attribute.
            //
            // For explanations of the parameters of the createChildDDO function, refer to the
            // "Creating Child Components" section in the header of this sample.
            cout << "Creating child component / DDOs..." << endl;

	            DKDDO* ddoDocumentChild1 = dsICM->createChildDDO("S_withChild", "S_simpleChild");   // createChildDDO(<Overall Item Type>, <Child Component Type>);
	            DKDDO* ddoDocumentChild2 = dsICM->createChildDDO("S_withChild", "S_simpleChild");   //    For more information, refer to header of this sample.
	            DKDDO* ddoDocumentChild3 = dsICM->createChildDDO("S_withChild", "S_simpleChild");  

            cout << "Created child component / DDOs." << endl;

            //-------------------------------------------------------------
            // Adding Child Components / Multivalue Attributes to DDOs
            //-------------------------------------------------------------
            // Add the children created above to the Document DDO created in the first section.
            cout << "Adding Child Components to DDOs..." << endl;
            
                // First, get the child colleciton within the parent for the child object type.
                short dataid = ddoDocument->dataId(DK_CM_NAMESPACE_CHILD,ddoDocumentChild1->getObjectType());
                DKChildCollection* children = (DKChildCollection*)(dkCollection*) ddoDocument->getData(dataid);

                if(children==NULL){ // if no collection exists, create one.
                    children = new DKChildCollection();
                    ddoDocument->setData(ddoDocument->dataId(DK_CM_NAMESPACE_CHILD, ddoDocumentChild1->getObjectType()), children);
                }
        
                children->addElement(ddoDocumentChild1); // add children to the collection.  These children
                children->addElement(ddoDocumentChild2); // are all of the same object type, so they belong
                children->addElement(ddoDocumentChild3); // to the same children collection.

            cout << "Added Child Components to DDOs." << endl;

            //----------------------------------------------------------------------------------
            // Setting / Modifying Attributes
            //----------------------------------------------------------------------------------
            // Set the attribute with the value passed in as a DKAny.  Assignment operators 
            // defined in DKAny.hpp will convert the data types to and (in some cases) from
            // the DKAny data type.  Use DKAny to programmatically manage your DDOs.
            //
            // For a list of java.lang.Object subclasses to use depending on the attribute type,
            // please refer to the section in the header named "Setting / Modifying Attributes".
            //
            // Note that the Attribute Namespace is used.  Attribute names, Item / Component 
            // Type names, Link Type Name, etc., are in seperate namespaces.  Names only need 
            // to be unique among their own namespace.  Therefore, when using names, the correct
            // namespace must be used when common methods are used.  For more info, see header.
            cout << "Setting / Modifying Attributes..." << endl;

                // Define byte for value to set in Blob attribute.
                char picture_bytes[] = {(int)1,(int)2,(int)3,(int)4,(int)5,(int)6,(int)7,(int)8,(int)9,(int)10,(int)11,(int)12,(int)13,(int)14,(int)15,(int)16,(int)17,(int)18,(int)19 };

                // Set Attribute Values
                ddoDocument->setData(ddoDocument->dataId(DK_CM_NAMESPACE_ATTR,"S_varchar"),  DKString("this is a string value")); 
                ddoDocument->setData(ddoDocument->dataId(DK_CM_NAMESPACE_ATTR,"S_blob"),     DKByteArray(picture_bytes,19)); 
                ddoDocument->setData(ddoDocument->dataId(DK_CM_NAMESPACE_ATTR,"S_char"),     'A');
                ddoDocument->setData(ddoDocument->dataId(DK_CM_NAMESPACE_ATTR,"S_clob"),     DKString("Large amount of text can go here.  Size depends on attribute definition.")); 
                ddoDocument->setData(ddoDocument->dataId(DK_CM_NAMESPACE_ATTR,"S_date"),     DKDate("2001-08-12")); 
                ddoDocument->setData(ddoDocument->dataId(DK_CM_NAMESPACE_ATTR,"S_decimal"),  DKDecimal("12.45")); 
                ddoDocument->setData(ddoDocument->dataId(DK_CM_NAMESPACE_ATTR,"S_double"),   (double) 123); 
                ddoDocument->setData(ddoDocument->dataId(DK_CM_NAMESPACE_ATTR,"S_integer"),  (long) 123); 
                ddoDocument->setData(ddoDocument->dataId(DK_CM_NAMESPACE_ATTR,"S_short"),    (short) 5);
                ddoDocument->setData(ddoDocument->dataId(DK_CM_NAMESPACE_ATTR,"S_time"),     DKTime("10.00.00")); 
                ddoDocument->setData(ddoDocument->dataId(DK_CM_NAMESPACE_ATTR,"S_timestamp"),DKTimestamp("2001-08-12-10.00.00.123456"));

            cout << "Set / Modified Attributes." << endl;

            //-------------------------------------------------------------
            // Detecting Non-Existant Attributes in DDOs
            //-------------------------------------------------------------
            // If an attribute does not exist in a DDO, either because it was not defined
            // in the Item / Component Type Definition, or because it was not created through
            // the DKDatastoreICM.createDDO function, the dataId() function will return '0'.
            cout << "Detecting Non-Existant Attributes in DDOs..." << endl;

                // The Attribute "S_notInItem" was not defined for this Item Type.
                dataid = ddoDocument->dataId(DK_CM_NAMESPACE_ATTR,"S_notInItem");
                if(dataid == 0)
                    cout << "Attribute S_notInItem is not part of this DDO." << endl;
                else
                    cout << "Attribute S_notInItem is part of this DDO." << endl;
    
            cout << "Detected Non-Existant Attributes in DDOs." << endl;

            //-------------------------------------------------------------
            // Setting / Modifying Attributes Belonging to Attribute Groups
            //-------------------------------------------------------------
            // When accessing attributes, use the individual attribute definition names. 
            // In order to access attributes that belong to a particular attribute group,
            // the correct attribute name follows the following format:
            //
            // Grouped Attribute's Name Format:  <Attribute Group Name>.<Attribute Name>
            cout << "Setting / Modifying Attributes Belonging to Attribute Groups..." << endl;

                ddoDocument->setData(ddoDocument->dataId(DK_CM_NAMESPACE_ATTR,"S_address.S_street"), DKString("123 First Street")); 
                ddoDocument->setData(ddoDocument->dataId(DK_CM_NAMESPACE_ATTR,"S_address.S_city"),   DKString("Kingville")); 
                ddoDocument->setData(ddoDocument->dataId(DK_CM_NAMESPACE_ATTR,"S_address.S_state"),  DKString("CA")); 
                ddoDocument->setData(ddoDocument->dataId(DK_CM_NAMESPACE_ATTR,"S_address.S_zipCode"),DKString("12345-6789")); 

            cout << "Set / Modified Attributes Belonging to Attribute Groups." << endl;

            //-------------------------------------------------------------
            // Add / Save items to persistent datastore
            //-------------------------------------------------------------
            // When all modification is complete, the items can be added 
            // to the persistant datastore, which can be later accessed
            // if the PID is known or through Query.  See Item Retrieval
            // Sample and Query sample for more information.
            //
            // Only the root items need to be added.  The system will take 
            // care of the rest.
            cout << "Adding / Saving Items to Persistent Datastore..." << endl;

                ddoDocument->add(); // Adds entire item tree, including all children
                ddoFolder->add();

            cout << "Added / Saved Items to Persistent Datastore..." << endl;

            //-------------------------------------------------------------
            // Setting / Modifying Reference Attributes
            //-------------------------------------------------------------
            // When using Reference Attributes, they are set as regular
            // attributes are set.  A Reference Attribute is set using
            // a ddo.  At the time the Reference Attribute is set, the
            // ddo it will reference must be persistent in the datastore.
            // In this example, it will use the ddoDocument DKDDO object
            // that the "add()" operation was processed above.
            //
            // For explanation of Reference Attributes, please refer to 
            // SReferenceAttrDefCreationICM Sample.
            cout << "Setting / Modifying Reference Attributes..." << endl;

                // May make copy of object given as an attr value, or may use original.
                // If original is used, do not delete the memory, because it is then owned
                // item it was added to and will be deleted with it.
                DKDDO* ddoDocumentCopy = dsICM->createDDO(((DKPidICM*)ddoDocument->getPidObject())->pidString());
                ddoItem->setData(ddoItem->dataId(DK_CM_NAMESPACE_ATTR,"S_myRef"),ddoDocumentCopy); // Using DKDDO named 'ddoDocument' made persistent above.

            cout << "Seting / Modified Reference Attributes..." << endl;

            //-----------------------------------------------------------------
            // Add / Save Item with Reference Attribute to persistent datastore
            //-----------------------------------------------------------------
            cout << "Adding / Saving Item to Persistent Datastore..." << endl;

                ddoItem->add();

                delete(ddoDocument);   // DDOs will also delete all information contained within.
                delete(ddoFolder);
                delete(ddoItem);
                
            cout << "Added / Saved Item to Persistent Datastore..." << endl;

            //-------------------------------------------------------------
            // Disconnect from datastore & Destroy Reference
            //-------------------------------------------------------------
            // See Sample SConnectDisconnectICM for more information
            cout << "Disconnecting from datastore & destroying reference..." << endl;            

                dsICM->disconnect();
                delete(dsICM);

            cout << "Disconnected from datastore & destroying reference." << endl;
    
            //-------------------------------------------------------------
            // Sample program completed without exception
            //-------------------------------------------------------------
            cout << "\n==========================================" << endl;
            cout << "Sample program completed."                    << endl;
            cout << "==========================================\n" << endl;
        }
        //------------------------------------------------------------
        // Catch & Print Exceptions        
        //------------------------------------------------------------
        catch (DKException &exc){
            SConnectDisconnectICM::printException(exc);    // Print the exception to the screen
            return(SConnectDisconnectICM_RC_SAMPLE_FAILED);  // Report using return code that it failed
        }
        return(SConnectDisconnectICM_RC_SAMPLE_PASSED);    // If no exceptions were thrown, return a code to indicate passed.
    }// end main

    //=================================================================
    // Wrapper Functions
    //=================================================================
    // The following are wrapper functions for functionality covered
    // in this sample.  These functions may be used by other samples.

   /**
    * Create a simple Item Type with 3 child components for use with Samples.
    * @param dsICM         Connected DKDatastoreICM object.
    * @param itemProperty  Item Property or Semantic Type of Item (DKConstant.DK_CM_xxxxxx  Ex: DOCUMENT, FOLDER, ITEM, <user defined>).
    * @return  Returns a created Item of Item Type "S_withChild" with a Root DDO and three Child Components / DDOs.
    **/
    DKDDO* SItemCreationICM::createSimpleItemWithChildren(DKDatastoreICM * dsICM, short itemPropertyOrSemanticType){
        cout << "Creating simple Item with 3 Children for Sample Purposes..." << endl;

            // Create an item / DDO / Root Component
	        DKDDO* ddo = dsICM->createDDO("S_withChild", itemPropertyOrSemanticType); //createDDO(<Overall Item Type>, <Item Property / Semantic Type>);

            // Create child components / DDOs / Multivalue Attributes
	        DKDDO* ddoChild1 = dsICM->createChildDDO("S_withChild", "S_simpleChild");   // createChildDDO(<Overall Item Type>, <Child Component Type>);
	        DKDDO* ddoChild2 = dsICM->createChildDDO("S_withChild", "S_simpleChild");   //    For more information, refer to header of this sample.
	        DKDDO* ddoChild3 = dsICM->createChildDDO("S_withChild", "S_simpleChild");  

            // Adding Child Components / Multivalue Attributes to DDOs
            short dataid = ddo->dataId(DK_CM_NAMESPACE_CHILD,ddoChild1->getObjectType());
            DKChildCollection* children = (DKChildCollection*)(dkCollection*) ddo->getData(dataid);
            if(children==NULL){ // if no collection exists, create one.                children = new DKChildCollection();
                ddo->setData(ddo->dataId(DK_CM_NAMESPACE_CHILD, ddoChild1->getObjectType()), children);
            }
            children->addElement(ddoChild1); // add children to the collection.  These children
            children->addElement(ddoChild2); // are all of the same object type, so they belong
            children->addElement(ddoChild3); // to the same children collection.

            // Setting / Modifying Attributes
            char picture_bytes[] = {(int)1,(int)2,(int)3,(int)4,(int)5,(int)6,(int)7,(int)8,(int)9,(int)10,(int)11,(int)12,(int)13,(int)14,(int)15,(int)16,(int)17,(int)18,(int)19 };
            ddo->setData(ddo->dataId(DK_CM_NAMESPACE_ATTR,"S_varchar"),  DKString("this is a string value")); 
            ddo->setData(ddo->dataId(DK_CM_NAMESPACE_ATTR,"S_blob"),     DKByteArray(picture_bytes,19)); 
            ddo->setData(ddo->dataId(DK_CM_NAMESPACE_ATTR,"S_char"),     'A');
            ddo->setData(ddo->dataId(DK_CM_NAMESPACE_ATTR,"S_clob"),     DKString("Large amount of text can go here.  Size depends on attribute definition.")); 
            ddo->setData(ddo->dataId(DK_CM_NAMESPACE_ATTR,"S_date"),     DKDate("2001-08-12")); 
            ddo->setData(ddo->dataId(DK_CM_NAMESPACE_ATTR,"S_decimal"),  DKDecimal("12.45")); 
            ddo->setData(ddo->dataId(DK_CM_NAMESPACE_ATTR,"S_double"),   (double) 123); 
            ddo->setData(ddo->dataId(DK_CM_NAMESPACE_ATTR,"S_integer"),  (long) 123); 
            ddo->setData(ddo->dataId(DK_CM_NAMESPACE_ATTR,"S_short"),    (short) 5);
            ddo->setData(ddo->dataId(DK_CM_NAMESPACE_ATTR,"S_time"),     DKTime("10.00.00")); 
            ddo->setData(ddo->dataId(DK_CM_NAMESPACE_ATTR,"S_timestamp"),DKTimestamp("2001-08-12-10.00.00.123456"));

            // Setting / Modifying Attributes Belonging to Attribute Groups
            ddo->setData(ddo->dataId(DK_CM_NAMESPACE_ATTR,"S_address.S_street"), DKString("123 First Street")); 
            ddo->setData(ddo->dataId(DK_CM_NAMESPACE_ATTR,"S_address.S_city"),   DKString("Kingville")); 
            ddo->setData(ddo->dataId(DK_CM_NAMESPACE_ATTR,"S_address.S_state"),  DKString("CA"));
            ddo->setData(ddo->dataId(DK_CM_NAMESPACE_ATTR,"S_address.S_zipCode"),DKString("12345-6789")); 

            // Add / Save item to persistent datastore
            ddo->add(); // Adds entire item tree, including all children

        cout << "Created Simple Item with 3 Children for Sample Purposes." << endl;
        return(ddo);
    }
