/*BEGINPROLOGUE************************************************************************
 * @copyright(disclaimer)                                                             *
 * Licensed Materials - Property of IBM                                               *
 * IBM Information Integrator for Content V8.2  (program number 5724-B43)             *
 * (c ) Copyright IBM Corp. 1998, 2002, 2003. All Rights Reserved.                    *
 *                                                                                    *
 * US Government Users Restricted Rights                                              *
 * Use, duplication or disclosure restricted by GSA ADP Schedule                      *
 * Contract with IBM Corp.                                                            *
 *                                                                                    *
 * DISCLAIMER OF WARRANTIES :                                                         *
 *                                                                                    *
 * Permission is granted to copy and modify this Sample code, and to                  *
 * distribute modified versions provided that both the copyright                      *
 * notice,- and this permission notice and warranty disclaimer appear in all copies   * 
 * and modified versions.                                                             *
 *                                                                                    *
 * THIS SAMPLE CODE IS LICENSED TO YOU AS-IS.                                         *
 * IBM AND ITS SUPPLIERS AND LICENSORS DISCLAIM ALL WARRANTIES, EITHER EXPRESS OR     *
 * IMPLIED, IN SUCH SAMPLE CODE, INCLUDING THE WARRANTY OF NON-INFRINGEMENT AND THE   *
 * IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE. IN NO   *
 * EVENT WILL IBM OR ITS LICENSORS OR SUPPLIERS BE LIABLE FOR ANY DAMAGES ARISING OUT *
 * OF THE USE OF OR INABILITY TO USE THE SAMPLE CODE, DISTRIBUTION OF THE SAMPLE CODE,*
 * OR COMBINATION OF THE SAMPLE CODE WITH ANY OTHER CODE. IN NO EVENT SHALL IBM OR ITS*
 * LICENSORS AND SUPPLIERS BE LIABLE FOR ANY LOST REVENUE, LOST PROFITS OR DATA, OR   *
 * FOR DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL,INCIDENTAL OR PUNITIVE DAMAGES,       *
 * HOWEVER CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY,-, EVEN IF IBM OR ITS     *
 * LICENSORS OR SUPPLIERS HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.       *
 *                                                                                    *
 * @endCopyright                                                                      *
 **************************************************************************************

NOTE:  When retrieving Item Type, Component Type, or any Entity Definitions, if it 
       does not exist, null is returned.  Usage of the null object will result in 
       a NullPointerException.

Listing Attribute Options
    When listing attributes & Attribute Groups of Item or Component Type Definitions,
    options may be specified to obtain resource, system, and user-defined attributes.
    
    ------------------------------------------------------------------------------                                  
    Option                        Description
    ----------------------------  ------------------------------------------------
    DK_ICM_ALL_ATTR               List attributes that match all the options below 
    DK_ICM_USER_ATTR (Default)    List user defined attributes (Default)
    DK_ICM_RESOURCE_ATTR          List resource manager attributes 
    DK_ICM_SYSTEM_ATTR            List system-defined attributes 
    DK_ICM_USER_ATTR_NO_RESOURCE  List user defined attributes without resource 
                                  manager attributes 
    ------------------------------------------------------------------------------                                  
    Note: Constants defined in DKConstantICM.h


********************************************************************/

// Includes / Imports
#include <iostream.h>
#include <DKDatastoreICM.hpp>
#include <DKAttrDefICM.hpp>
#include <DKAttrGroupDefICM.hpp>
#include <DKItemTypeDefICM.hpp>
#include <DKComponentTypeDefICM.hpp>
#include <DKConstant.h>
#include <DKConstantICM.h>
#include "SConnectDisconnectICM.hpp"
#include "SItemTypeRetrievalICM.hpp"

/************************************************************************************************
 *          FILENAME: SItemTypeRetrievalICM.cpp
 *                    ---------------------------------------------------------------------------
 *       DESCRIPTION: Listing All Item Types Names & Definitions in System, Lisitng Attributes of
 *                    an Item Type Definition, Retrieving Individual Item Type Definitions, and
 *                    Printing Item Type Definitions
 *                    ---------------------------------------------------------------------------
 *     DEMONSTRATION: Listing All Item Type Names Defined in the System
 *                    Listing All Item Type Definitions Defined in the System
 *                    Listing Attribute Options
 *                    Listing All Attribute Names of an Item or Component Type
 *                    Listing All Attribute Group Names of an Item or Component Type
 *                    Listing All Attribute Definitions of an Item or Component Type
 *                    Listing All Sub-Entities / Children / Multivalue Attributes of an Item Type or Child Component.
 *                    Retrieving an Individual Item Type Definition
 *                    Printing an Item Type or Component Type Definition
 *                    Sequantial Collections
 *                    Iterators
 *                    ---------------------------------------------------------------------------
 * COMMANDLINE USAGE: SItemTypeRetrievalICM <database> <userName> <password>
 *                    ---------------------------------------------------------------------------
 *      PREREQUISITS: Item Types & Component Types must be defined.  If not, please run Sample 
 *                    SItemTypeCreationICM first.
 *                    ---------------------------------------------------------------------------
 *    FILES REQUIRED: SConnectDisconnectICM.hpp/cpp
 *                    SItemTypeRetrievalICM_launch.cpp
 *                    SItemTypeRetrievalICM.hpp
 ************************************************************************************************/

    //-------------------------------------------------------------
    // Main
    //-------------------------------------------------------------
    /**
     * Run the Sample.
     * @param argc   Argument Count / Number of Arguments Specified
     * @param argv[] String Array containing arguments.  Optional arguments are <databse> <userName> <password>.
     * @return  Returns SConnectDisconnectICM_RC_SAMPLE_PASSED (0) if no sample program completed, SConnectDisconnectICM_RC_SAMPLE_FAILED (-1) if any errors / exceptions were caught.
     **/
    int SItemTypeRetrievalICM::main(int argc, char *argv[]){

        // Defaults for connecting to the database.
        char * database = SConnectDisconnectICM_DEFAULT_DATABASE;
        char * userName = SConnectDisconnectICM_DEFAULT_USERNAME;
        char * password = SConnectDisconnectICM_DEFAULT_PASSWORD;

        //------------------------------------------------------------
        // Checking for input parameters
        //--------------------------------------------------------------
        if (argc < 4) { // if not all 3 arguments were specified, use defaults and report correct usage.
            cout << "Usage: "                                                   << endl;
            cout << "  SItemTypeRetrievalICM <database> <userName> <password>"  << endl;
            cout << "  *** Some parameters not specified, using defaults..."    << endl;
            cout << endl;
        } else {  // otherwise enough parameters were specified, use the first 3 for the 3 parameters.
            if (argc > 1) database = argv[1];
            if (argc > 2) userName = argv[2];
            if (argc > 3) password = argv[3];
        }//end else
        char * ver = SConnectDisconnectICM_EIP_VERSION;

        cout << "==========================================="     << endl;
        cout << "IBM Information Integrator for Content v" << ver << endl;
        cout << "Sample Program:  SItemTypeRetrievalICM"          << endl;
        cout << "-------------------------------------------"     << endl;
        cout << " Database: " << database                         << endl;
        cout << " UserName: " << userName                         << endl;
        cout << "===========================================\n"   << endl;

        try{
            //-------------------------------------------------------------
            // Connect to datastore
            //-------------------------------------------------------------
            // See Sample SConnectDisconnectICM for more information
            cout << "Connecting to datastore (Database '" << database << "', UserName '" << userName << "')..." << endl;

                DKDatastoreICM* dsICM = new DKDatastoreICM();  // Create new datastore object.
                dsICM->connect(database,userName,password,""); // Connect to the datastore.

            cout << "Connected to datastore (Database '" << dsICM->datastoreName() << "', UserName '" << dsICM->userName() << "')." << endl;

            //-------------------------------------------------------------------
            // Accessing / Listing All Item Type Names Defined in System
            //-------------------------------------------------------------------
            // Access a list of Item Type Names defined in the Datastore Definition
            // and print them for the user.
            cout << "Accessing / Listing All Item Type Names Defined in System..." << endl;

                int  i;
                long arraySize = -1;  // Variable passed in as reference that will be modified by the OOAPI.
                DKString* itemTypeNames = dsICM->listEntityNames(arraySize);  // Get String Array of all Entity Types.

                // Print Names to Commandline
                cout << "\nItem Type Names in System:  (" << arraySize << ')' << endl;

                    for(i=0; i < arraySize; i++)
                        cout << "     - " << itemTypeNames[i] << endl;

                cout << endl;

                delete[] itemTypeNames;   // Free Memory

            cout << "Accessed / Listed All Item Type Names Defined in System." << endl;

            //-------------------------------------------------------------------
            // Accessing / Listing All Item Type Definitions Defined in System
            //-------------------------------------------------------------------
            // Access a list of all Item Type Definitions defined in the Datastore Definition
            // and print them for the user.
            //
            // NOTE:  For a complete listing all properties and attributes everything defined
            //        in the sytem, please refer to the section covered later in this sample
            //        named "Printing all Definition Informaiton in the System".
            cout << "Accessing / Listing All Item Type Definitions Defined in System..." << endl;

                DKSequentialCollection* itemTypeColl = (DKSequentialCollection*) dsICM->listEntities();  // Get a collection containing all Item Type Definitions.

                // Accessing each and printing the name.  For printing all information, please
                // refer to later sections in this sample.
                cout << "\nItem Type Names in System:  (" << itemTypeColl->cardinality() << ')' << endl;

                    dkIterator* iter = itemTypeColl->createIterator(); // Create an iterator to iterate through the collection
                    while(iter->more()){ // while there are still items in the list, continue
                        DKItemTypeDefICM* itemType = (DKItemTypeDefICM*) iter->next()->value(); // Move pointer to next element & return that object.
                        cout << "     - " << itemType->getName() << ":  " << itemType->getDescription() << endl;

                        delete(itemType);   // Free Memory
                    }

                cout << endl;

                delete(iter);               // Free Memory
                delete(itemTypeColl);  

            cout << "Accessed / Listed All Item Type Definitions Defined in System." << endl;

            //-------------------------------------------------------------------
            // Retrieving an Individual Item Type Definition
            //-------------------------------------------------------------------
            // Retrieve a single Item Type Definition given the Item Type Name.
            //
            // NOTE:  When retrieving Item Type, Component Type, or any Entity Definitions, if it 
            //        does not exist, null is returned.  Usage of the null object will result in 
            //        a NullPointerException.
            cout << "Retrieving Item Type Definition of 'S_adjustrReport'..." << endl;

                DKDatastoreDefICM* dsDefICM = (DKDatastoreDefICM*) dsICM->datastoreDef(); // Get the datastore definition object from the connected datastore.
                DKItemTypeDefICM* itemType = (DKItemTypeDefICM*) dsDefICM->retrieveEntity("S_adjustrReport"); // Retrieve the Item Type Definition.

            cout << "Retrieved Item Type Definition of 'S_adjustrReport'." << endl;

            //-------------------------------------------------------------------
            // Listing All Attributes Names of a Component Type
            //-------------------------------------------------------------------
            // Access a list of Attribute Names for a particular Item Type and print
            // it to the commandline.  Note that Item Type extends Component Type.
            //
            // The list function can also take options to display system-defined
            // attributes as well.  By default, only user-defined attributes are
            // listed.  More information can be found at the top of this sample.
            //
            // The DKComponentTypeDefICM object can be used to retrieve attributes 
            // defined for this Component Type.
            cout << "Accessing & Listing All Attribute Names Used in Item Type 'S_adjustrReport'..." << endl;

                DKString* attrs = itemType->listAttrNames(arraySize);  // Get String Array of all Attribute Names for this Component / Item Type.

                // Print Names to Commandline
                cout << "\nAttributes of Item Type 'S_adjustrReport':  (" << arraySize << ')' << endl;

                    for(i=0; i < arraySize; i++)
                        cout << "     - " << attrs[i] << endl;

                cout << endl;

                delete[] attrs;        // Free Memory

            cout << "Accessed & Listed All Attribute Names Used in Item Type 'S_adjustrReport'." << endl;

            //------------------------------------------------------------------------------
            // Listing All Attribute Group & Reference Attribute Names of an Component Type
            //------------------------------------------------------------------------------
            // Access a list of Attribute Group and Reference Attribute Names for a 
            // particular Item Type and print it to the commandline.  Note that Item Type 
            // extends Component Type
            //
            // As in SReferenceAttrDefCreationICM, the Reference Attribute is handled in 
            // definitions as an Attribute Group.
            //
            // The DKComponentTypeDefICM object can be used to retrieve attribute groups
            // defined for this Component Type.
            cout << "Accessing & Listing All Attribute Group & Reference Attribute Names Used in Item Type 'S_adjustrReport'..." << endl;

                DKString* attrGroups = itemType->listAttrGroupNames(arraySize);  // Get String Array of all Attribute Group Names for this Component / Item Type.

                // Print Names to Commandline
                cout << "\nAttribute Groups & Reference Attributes of 'S_adjustrReport':  (" << arraySize << ')' << endl;

                    for(i=0; i < arraySize; i++)
                        cout << "     - " << attrGroups[i] << endl;

                cout << endl;

                delete[] attrGroups;     // Free Memory

            cout << "Accessed & Listed All Attribute Group Names Used in Item Type 'S_adjustrReport'." << endl;

            //-------------------------------------------------------------------
            // Listing All Attributes Definitions of an Item or Component Type
            //-------------------------------------------------------------------
            // Access a list of Attribute Definitions for a particular Item or 
            // Component Type.  This particular list will include all Attributes,
            // including those belonging to Attribute Groups defined for this 
            // Item or Component Type.  Refer to the Programming Reference documentation
            // for ather methods tat are available.
            //
            // Note that with the default list attribute options, only user-defined
            // attributes will be listed.  As covered in SReferenceAttrDefICM, Reference 
            // Attributes are actually an Attribute Group with system-defined Attributes,
            // which means that they will not appear in the list below.
            //
            // For more information on listing and printing individual attribute properties,
            // pease refer to Sample SAttributeDefinitionRetrievalICM.
            cout << "Accessing & Listing All Attribute Definitions Used in Item Type 'S_adjustrReport'..." << endl;

                DKSequentialCollection* attrColl = (DKSequentialCollection*) itemType->listAllAttributes();  // Get a collection containing all Attribute Definitions for the Item or Component Type.

                // Accessing each and printing the name & description.  For printing all information, please
                // refer to later sections in this sample.
                cout << "\nAttributes of Item Type 'S_adjustrReport':  (" << attrColl->cardinality() << ')' << endl;

                    iter = attrColl->createIterator(); // Create an iterator to iterate through the collection
                    while(iter->more()){ // while there are still items in the list, continue
                        DKAttrDefICM* attr = (DKAttrDefICM*) iter->next()->value(); // Move pointer to next element & return that object.
                        cout << "     - " << attr->getName() << ":  "<< attr->getDescription() << endl;
                        delete(attr);    // Free Memory
                    }

                cout << endl;

                delete(iter);            // Free Memory
                delete(attrColl);        // Free Memory

            cout << "Accessed & Listed All Attribute Definitions Used in Item Type 'S_adjustrReport." << endl;

            //-------------------------------------------------------------------
            // Retrieving the Insurance Policy Item Type Definition
            //-------------------------------------------------------------------
            cout << "Retrieving Item Type Definition of 'S_insurePolicy'..." << endl;

                itemType = (DKItemTypeDefICM*) dsDefICM->retrieveEntity("S_insurePolicy"); // Retrieve the Item Type Definition.

            cout << "Retrieved Item Type Definition of 'S_insurePolicy'." << endl;

            //-------------------------------------------------------------------
            // Listing All Sub-Entities / Children / Multivalue Attributes
            //-------------------------------------------------------------------
            // Listing works for both Item Types and Child Component.  This method
            // can be applied to the children to obtain sub-entities of them.
            cout << "Accessing / Listing Sub-Entities / Multivalue Attributes / Children..." << endl;
                
                DKSequentialCollection* compTypes = (DKSequentialCollection*) itemType->listSubEntities(); // Obtain the collection of child component types.

                // Accessing each and printing the name and description. 
                cout << "\nSub Entities of 'S_insurePolicy':  (" << compTypes->cardinality() << ')' << endl;

                    iter = compTypes->createIterator(); // Create an iterator to iterate through the collection
                    while(iter->more()){                // while there are still items in the list, continue
                        DKComponentTypeDefICM* compType = (DKComponentTypeDefICM*) iter->next()->value(); // Move pointer to next element & return that object.
                        cout << "     - " << compType->getName() << ":  " << compType->getDescription() << endl;
                        delete(compType);               // Free Memory
                    }

                    delete(iter);                       // Free Memory

                    // Since Reference Attributes consist of system-defined attributes and the default list 
                    // option was used above, references will be added on the end, using attribute groups.
                    DKSequentialCollection* attrGroupDefs = (DKSequentialCollection*) itemType->listAttrGroups();  // Get a collection containing all Attribute Group Definitions for the Item / Component Type.
                    iter = attrGroupDefs->createIterator();                                                         // Create an iterator to iterate through the collection
                    while(iter->more()){                                                   // while there are still items in the list, continue
                        DKAttrGroupDefICM* attrGroup = (DKAttrGroupDefICM*) iter->next()->value();  // Move pointer to next element & return that object.
                        if(attrGroup->isReferenceAttributeGroup())                         // If it is a reference attribute, print it, otherwise just skip
                            cout << "     - " << attrGroup->getName() << ": " << attrGroup->getDescription() << endl;
                        delete(attrGroup);              // Free Memory
                    }

                    delete(iter);                       // Free Memory
                    delete(attrGroupDefs);              // Free Memory

                cout << endl;

                delete(compTypes);                      // Free Memory

            cout << "Accessed / Listed Sub-Entities / Multivalue Attributes / Children." << endl;

            //-------------------------------------------------------------------
            // Printing an Item Type Definition
            //-------------------------------------------------------------------
            // Item Type Definition already retrieve from the section above.
            //
            // Uses wrapper function defined at end of sample file.
            cout << "Printing the Item Type Definition of 'S_insurePolicy'..." << endl;

                printItemType(itemType); // Uses wrapper funciton defined at end of sample file.

            cout << "Printed the Item Type Definition of 'S_insurePolicy'." << endl;

            //-------------------------------------------------------------------
            // Printing Adjuster Report with Attribute Group & Reference Attribute
            //-------------------------------------------------------------------
            cout << "Printing the Item Type Definition of 'S_adjustrReport'..." << endl;

                printItemType(dsICM,"S_adjustrReport"); // Uses wrapper function defined at end of sample file.

            cout << "Printing the Item Type Definition of 'S_adjustrReprot'..." << endl;

            //-------------------------------------------------------------
            // Disconnect from datastore & Destroy Reference
            //-------------------------------------------------------------
            // See Sample SConnectDisconnectICM for more information
            cout << "Disconnecting from datastore & destroying reference..." << endl;            

                dsICM->disconnect();
                delete(dsICM);

            cout << "Disconnected from datastore & destroying reference." << endl;
    
            //-------------------------------------------------------------
            // Sample program completed without exception
            //-------------------------------------------------------------
            cout << "\n==========================================" << endl;
            cout << "Sample program completed."                    << endl;
            cout << "==========================================\n" << endl;
        }
        //------------------------------------------------------------
        // Catch & Print Exceptions        
        //------------------------------------------------------------
        catch (DKException &exc){
            SConnectDisconnectICM::printException(exc);    // Print the exception to the screen
            return(SConnectDisconnectICM_RC_SAMPLE_FAILED);  // Report using return code that it failed
        }
        return(SConnectDisconnectICM_RC_SAMPLE_PASSED);    // If no exceptions were thrown, return a code to indicate passed.
    }// end main

    //=================================================================
    // Wrapper Functions
    //=================================================================
    // The following are wrapper functions for functionality covered
    // in this sample.  These functions may be used by other samples.

   /**
    * Printing All Item Type Names Defined in System.  Accesses a list
    * of Item Type Definitions defined in the Datastore Definition and
    * prints the name and descripiton them to the commandline.
    * @param dsICM  Conncted DKDatastoreICM object.
    **/
    void SItemTypeRetrievalICM::printAllItemTypeNames(DKDatastoreICM * dsICM){

        DKSequentialCollection* itemTypeColl = (DKSequentialCollection*) dsICM->listEntities();  // Get a collection containing all Item Type Definitions.

        // Accessing each and printing the name & description.  
        cout << "\nItem Type Names in System:  (" << itemTypeColl->cardinality() << ')' << endl;

            dkIterator* iter = itemTypeColl->createIterator(); // Create an iterator to iterate through the collection
            while(iter->more()){                               // while there are still items in the list, continue
                DKItemTypeDefICM* itemType = (DKItemTypeDefICM*) iter->next()->value(); // Move pointer to next element & return that object.
                cout << "     - " << itemType->getName() << ": " << itemType->getDescription() << endl;
                delete(itemType);                              // Free Memory
            }

        cout << endl;

        delete(iter);
        delete(itemTypeColl);
    }

   /**
    * Printing All Attributes Defined in particular Item Type.  Accesses a 
    * list of Attribute Definitions for a particular Item Type, and prints 
    * the name and description to the commandline.
    * @param dsICM         Conncted DKDatastoreICM object.
    * @param itemTypeName  Name of the Item Type for which Print the Attributes.
    **/
    void SItemTypeRetrievalICM::printAllAttributesOfItemType(DKDatastoreICM * dsICM, char * itemTypeName){

        DKSequentialCollection* attrColl = (DKSequentialCollection*) dsICM->listEntityAttrs(itemTypeName);  // Get a collection containing all Attribute Definitions for the Item Type.

        // Accessing each and printing the name & description.  For printing all information, please
        // refer to later sections in this sample.
        cout << "\nAttributes of Item Type '" << itemTypeName << "':  (" << attrColl->cardinality() << ')' << endl;

            dkIterator* iter = attrColl->createIterator(); // Create an iterator to iterate through the collection
            while(iter->more()){                           // while there are still items in the list, continue
                DKAttrDefICM* attr = (DKAttrDefICM*) iter->next()->value(); // Move pointer to next element & return that object.
                cout << "     - " << attr->getName() << ": " << attr->getDescription() << endl;
                delete(attr);
            }

        cout << endl;

        delete(iter);                                      // Free Memory
        delete(attrColl);
    }

   /**
    * Printing All Sub-Entities / Multivalue Attributes / Child Components defined in the 
    * particular Item Type.  Accesses a list of SubEntity Definitions for a particular Item Type,
    * and prints the name and description to the commandline.
    * @param dsICM         Conncted DKDatastoreICM object.
    * @param itemTypeDef  Name of the Item Type for which Print the SubEntities.
    **/
    void SItemTypeRetrievalICM::printAllAttributesOfItemType(DKDatastoreICM * dsICM, DKItemTypeDefICM * itemTypeDef){

        DKSequentialCollection* compTypes = (DKSequentialCollection*) itemTypeDef->listSubEntities(); // Obtain the collection of child component types.

        // Accessing each and printing the name and description. 
        cout << "\nSub Entities of '" << itemTypeDef->getName() << "':  (" << compTypes->cardinality() << ')' << endl;

            dkIterator* iter = compTypes->createIterator(); // Create an iterator to iterate through the collection
            while(iter->more()){                            // while there are still items in the list, continue
                DKComponentTypeDefICM* compType = (DKComponentTypeDefICM*) iter->next()->value(); // Move pointer to next element & return that object.
                cout << "     - " << compType->getName() << ": " << compType->getDescription() << endl;
                delete(compType);                           // Free Memory
            }

        cout << endl;

        delete(iter);                                       // Free Memory
        delete(compTypes);
    }
    
   /**
    * Print an Item Type Definition.
    * @param itemTypeDef  The DKItemTypeDefICM object to print.
    **/
    void SItemTypeRetrievalICM::printItemType(DKItemTypeDefICM * itemTypeDef){

        cout << "--------------------------------------------------"        << endl;
        cout << "               Item Type Definition"                       << endl;
        cout << "--------------------------------------------------"        << endl;
        cout << "              Name:  " << itemTypeDef->getName()           << endl;
        cout << "       Description:  " << itemTypeDef->getDescription()    << endl;
        cout << "         Entity ID:  " << itemTypeDef->getIntId()          << endl;
        switch(itemTypeDef->getClassification()){
            case DK_ICM_ITEMTYPE_CLASS_ITEM:
                cout << "    Classification:  Item"                         << endl;
                break;
            case DK_ICM_ITEMTYPE_CLASS_RESOURCE_ITEM:
                cout << "    Classification:  Resource Item"                << endl;
                break;
            case DK_ICM_ITEMTYPE_CLASS_DOC_MODEL:
                cout << "    Classification:  ICM Document Model Item"      << endl;
                break;
            case DK_ICM_ITEMTYPE_CLASS_DOC_PART:
                cout << "    Classification:  ICM Document Model Part"      << endl;
                break;
            default:
                cout << "    Classification:  Other (" << itemTypeDef->getClassification() << ')' << endl;
                break;
        }//end switch on classification
        cout << "         XDO Class:  " << itemTypeDef->getXDOClassID() << " - " << itemTypeDef->getXDOClassName() << endl;
        cout << " Default Coll Code:  " << itemTypeDef->getDefaultCollCode() << endl;
        cout << "   Default RM Code:  " << itemTypeDef->getDefaultRMCode()   << endl;
        // Dispay Version Control Information
        switch(itemTypeDef->getVersionControl()){
            case DK_ICM_VERSION_CONTROL_NEVER:
                cout << "   Version Control:  Off / Never"                 << endl;
                break;
            case DK_ICM_VERSION_CONTROL_ALWAYS:
                cout << "   Version Control:  Always / Automatic"          << endl;
                break;
            case DK_ICM_VERSION_CONTROL_BY_APPLICATION:
                cout << "   Version Control:  By Application"              << endl;
                break;
            default:
                cout << "   Version Control:  Unknown (" << itemTypeDef->getVersionControl() << ')' << endl;
                break;
        }//end switch on version control
        switch(itemTypeDef->getVersioningType()){
            case DK_ICM_ITEM_VERSIONING_FULL:
                cout << "   Versioning Type:  Full"                        << endl;
                break;
            case DK_ICM_ITEM_VERSIONING_OPTIMIZED:
                cout << "   Versioning Type:  Optimized / Only Resource Content" << endl;
                break;
            default:
                cout << "   Versioning Type:  Unknown (" << itemTypeDef->getVersioningType() << ')' << endl;
            break;
        }//end switch on versioning type
        if(itemTypeDef->getDefaultItemRetention()==0)
            cout << " Default Retention:  never expires"                   << endl;
        else
            cout << " Default Retention:  " << itemTypeDef->getDefaultItemRetention() << endl;
        switch(itemTypeDef->getDefaultRetentionUnit()){
            case 3:
                cout << " Default Ret. Unit:  day"                         << endl;
                break;
            case 2:
                cout << " Default Ret. Unit:  week"                        << endl;
                break;
            case 1:
                cout << " Default Ret. Unit:  month"                       << endl;
                break;
            case 0:
                cout << " Default Ret. Unit:  year"                        << endl;
                break;
            default:
                cout << " Default Ret. Unit:  other (" << itemTypeDef->getDefaultRetentionUnit() << ')' << endl;
                break;
        }
        // List Sub Entities 
        cout << "       Sub-Entities:  -" << endl;
        DKSequentialCollection* compTypes = (DKSequentialCollection*) itemTypeDef->listSubEntities(); // Obtain the collection of child component types.
        // Accessing each and printing the name and description. 
        dkIterator* iter = compTypes->createIterator(); // Create an iterator to iterate through the collection
        while(iter->more()){                            // while there are still items in the list, continue
            DKComponentTypeDefICM* compType = (DKComponentTypeDefICM*) iter->next()->value(); // Move pointer to next element & return that object.
            cout << "                     " << compType->getName() << ": " << compType->getDescription() << endl;
            delete(compType);                           // Free memory
        }
        delete(iter);                                   // Free Memory
        delete(compTypes);
        // List Attributes
        cout << "         Attributes:  -" << endl;
        DKSequentialCollection* attrColl = (DKSequentialCollection*) itemTypeDef->listAllAttributes();  // Get a collection containing all Attribute Definitions for the Item Type.
        iter = attrColl->createIterator(); // Create an iterator to iterate through the collection
        while(iter->more()){               // while there are still items in the list, continue
            DKAttrDefICM* attr = (DKAttrDefICM*) iter->next()->value(); // Move pointer to next element & return that object.
            cout << "                     " << attr->getName() << ": " << attr->getDescription() << endl;
            delete(attr);                  // Free Memory
        }   
        delete(iter);                      // Free Memory
        delete(attrColl);
        // Since Reference Attributes consist of system-defined attributes and the default list 
        // option was used above, references will be added on the end, using attribute groups.
        DKSequentialCollection* attrGroups = (DKSequentialCollection*) itemTypeDef->listAttrGroups();  // Get a collection containing all Attribute Group Definitions for the Item / Component Type.
        iter = attrGroups->createIterator(); // Create an iterator to iterate through the collection
        while(iter->more()){                 // while there are still items in the list, continue
            DKAttrGroupDefICM* attrGroup = (DKAttrGroupDefICM*) iter->next()->value(); // Move pointer to next element & return that object.
            // If it is a reference attribute, print it, otherwise just skip
            if(attrGroup->isReferenceAttributeGroup())
                cout << "                     " << attrGroup->getName() << ": "<< attrGroup->getDescription() << endl;
            delete(attrGroup);               // Free Memory
        }
        delete(iter);                        // Free Memory
        delete(attrGroups);
        cout << "---------------------------------------------------------\n" << endl;

        // Print Sub-Components
        DKSequentialCollection* subCompTypes = (DKSequentialCollection*) itemTypeDef->listSubEntities(); // Obtain the collection of child component types.
        iter = subCompTypes->createIterator(); // Create an iterator to iterate through the collection
        while(iter->more()){                   // while there are still items in the list, continue
            DKComponentTypeDefICM* subCompType = (DKComponentTypeDefICM*) iter->next()->value(); // Move pointer to next element & return that object.
            printComponentType(subCompType);   // print the sub component
            delete(subCompType);               // Free Memory
        }
        delete(iter);                          // Free Memory
        delete(subCompTypes);
    }
    
   /**
    * Print an Item Type Definition.  First retrieves the specified Item Type Definition, and
    * calls another wrapper function to print it.
    * @param dsICM         Conncted DKDatastoreICM object.
    * @param itemTypeName  Name of the Item Type to Print.
    **/
    void SItemTypeRetrievalICM::printItemType(DKDatastoreICM * dsICM, char * itemTypeName){

        DKDatastoreDefICM* dsDefICM = (DKDatastoreDefICM*) dsICM->datastoreDef(); // Get the datastore definition object from the connected datastore.
        DKItemTypeDefICM* itemType = (DKItemTypeDefICM*) dsDefICM->retrieveEntity(itemTypeName); // Retrieve the Item Type Definition.
        printItemType(itemType);    // Call the wrapper funciton to print it.
        delete(itemType);           // Free Memory
    }

   /**
    * Print an Component Type Definition.
    * @param compTypeDef  The DKComponentTypeDefICM object to print.
    **/
    void SItemTypeRetrievalICM::printComponentType(DKComponentTypeDefICM * compTypeDef){

        cout << "-------------------------------------------------"           << endl;
        cout << "            Component Type Definition"                       << endl;
        cout << "-------------------------------------------------"           << endl;
        cout << "               Name: " << compTypeDef->getName()             << endl;
        cout << "        Description: " << compTypeDef->getDescription()      << endl;
        cout << "          Entity ID: " << compTypeDef->getIntId()            << endl;
        switch(compTypeDef->getDeleteRule()){
            case DK_ICM_DELETE_RULE_RESTRICT:
                cout << "        Delete Rule: Restrict (Default)"             << endl;
                break;
            case DK_ICM_DELETE_RULE_CASCADE:
                cout << "        Delete Rule: Cascade"                        << endl;
                break;
            case DK_ICM_DELETE_RULE_SET_NULL:
                cout << "        Delete Rule: Set Null"                       << endl;
                break;
            case DK_ICM_DELETE_RULE_NO_ACTION:
                cout << "        Delete Rule: No Action"                      << endl;
                break;
            default:
                cout << "        Delete Rule: Other (" << compTypeDef->getDeleteRule() << ')' << endl;
                break;
        }
        
        // List Sub Entities 
        cout << "       Sub-Entities:  -" << endl;
        DKSequentialCollection* subCompTypes = (DKSequentialCollection*) compTypeDef->listSubEntities(); // Obtain the collection of child component types.
        // Accessing each and printing the name and description. 
        dkIterator* iter = subCompTypes->createIterator(); // Create an iterator to iterate through the collection
        while(iter->more()){                               // while there are still items in the list, continue
            DKComponentTypeDefICM* subCompType = (DKComponentTypeDefICM*) iter->next()->value(); // Move pointer to next element & return that object.
            cout << "                     " << subCompType->getName() << ": " << subCompType->getDescription() << endl;
            delete(subCompType);                           // Free Memory
        }
        delete(iter);                                      // Free Memory
        delete(subCompTypes);                              // Free Memory
        // List Attributes
        cout << "         Attributes:  -" << endl;
        DKSequentialCollection* attrColl = (DKSequentialCollection*) compTypeDef->listAllAttributes();  // Get a collection containing all Attribute Definitions for the Item / Component Type.
        iter = attrColl->createIterator(); // Create an iterator to iterate through the collection
        while(iter->more()){               // while there are still items in the list, continue
            DKAttrDefICM* attr = (DKAttrDefICM*) iter->next()->value(); // Move pointer to next element & return that object.
            cout << "                     " << attr->getName() << ": " << attr->getDescription() << endl;
            delete(attr);                  // Free Memory
        }
        delete(iter);                      // Free Memory
        delete(attrColl);
        // Since Reference Attributes consist of system-defined attributes and the default list 
        // option was used above, references will be added on the end, using attribute groups.
        DKSequentialCollection* attrGroups = (DKSequentialCollection*) compTypeDef->listAttrGroups();  // Get a collection containing all Attribute Group Definitions for the Item / Component Type.
        iter = attrGroups->createIterator(); // Create an iterator to iterate through the collection
        while(iter->more()){                 // while there are still items in the list, continue
            DKAttrGroupDefICM* attrGroup = (DKAttrGroupDefICM*) iter->next()->value(); // Move pointer to next element & return that object.
            // If it is a reference attribute, print it, otherwise just skip
            if(attrGroup->isReferenceAttributeGroup())
                cout << "                     " << attrGroup->getName() << ": " << attrGroup->getDescription() << endl;
            delete(attrGroup);               // Free Memory
        }
        delete(iter);                        // Free Memory
        delete(attrGroups);                  // Free Memory
        cout << "---------------------------------------------------------\n" << endl;

        // Print Sub-Components
        subCompTypes = (DKSequentialCollection*) compTypeDef->listSubEntities(); // Obtain the collection of child component types.
        iter = subCompTypes->createIterator(); // Create an iterator to iterate through the collection
        while(iter->more()){                   // while there are still items in the list, continue
            DKComponentTypeDefICM* subCompType = (DKComponentTypeDefICM*) iter->next()->value(); // Move pointer to next element & return that object.
            printComponentType(subCompType);   // print the sub component
            delete(subCompType);               // Free Memory
        }
        delete(iter);                          // Free Memory
        delete(subCompTypes);
    }
    
   /**
    * Print a Component Type Definition.  First retrieves the specified Component Type Definition, and
    * calls another wrapper function to print it.
    * @param dsICM         Conncted DKDatastoreICM object.
    * @param compTypeName  Name of the Component Type to Print.
    **/
    void SItemTypeRetrievalICM::printComponentType(DKDatastoreICM * dsICM, char * compTypeName){

        DKDatastoreDefICM* dsDefICM = (DKDatastoreDefICM*) dsICM->datastoreDef(); // Get the datastore definition object from the connected datastore.
        DKComponentTypeDefICM* compType = (DKComponentTypeDefICM*) dsDefICM->retrieveEntity(compTypeName); // Retrieve the Item Type Definition.
        printComponentType(compType); // Call the wrapper funciton to print it.
        delete(compType);             // Free Memory
    }

   /**
    * Determine if the Entity exists.
    * @param dsICM       Conncted DKDatastoreICM object.
    * @param entityName  Name of the Entity.
    * @return  Returns true if it exists, false if not.
    */
    bool SItemTypeRetrievalICM::doesEntityExist(DKDatastoreICM * dsICM, char * entityName){

        bool retVal;

        // First, we need acces to the Datastore Definition object.
        DKDatastoreDefICM* dsDefICM = (DKDatastoreDefICM*) dsICM->datastoreDef(); // Get the datastore definition object from the connected datastore.
        dkEntityDef* entityTypeDef  = dsDefICM->retrieveEntity(entityName);       // Retrieve the definition.
        if(entityTypeDef==NULL)    // If null is returned, it does not exist.
            retVal = false;
        else{
            delete(entityTypeDef); // Free Memory
            retVal = true;
        }

        return(retVal);
	}
