/*BEGINPROLOGUE************************************************************************
 * @copyright(disclaimer)                                                             *
 * Licensed Materials - Property of IBM                                               *
 * IBM Information Integrator for Content V8.2  (program number 5724-B43)             *
 * (c ) Copyright IBM Corp. 1998, 2002, 2003. All Rights Reserved.                    *
 *                                                                                    *
 * US Government Users Restricted Rights                                              *
 * Use, duplication or disclosure restricted by GSA ADP Schedule                      *
 * Contract with IBM Corp.                                                            *
 *                                                                                    *
 * DISCLAIMER OF WARRANTIES :                                                         *
 *                                                                                    *
 * Permission is granted to copy and modify this Sample code, and to                  *
 * distribute modified versions provided that both the copyright                      *
 * notice,- and this permission notice and warranty disclaimer appear in all copies   * 
 * and modified versions.                                                             *
 *                                                                                    *
 * THIS SAMPLE CODE IS LICENSED TO YOU AS-IS.                                         *
 * IBM AND ITS SUPPLIERS AND LICENSORS DISCLAIM ALL WARRANTIES, EITHER EXPRESS OR     *
 * IMPLIED, IN SUCH SAMPLE CODE, INCLUDING THE WARRANTY OF NON-INFRINGEMENT AND THE   *
 * IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE. IN NO   *
 * EVENT WILL IBM OR ITS LICENSORS OR SUPPLIERS BE LIABLE FOR ANY DAMAGES ARISING OUT *
 * OF THE USE OF OR INABILITY TO USE THE SAMPLE CODE, DISTRIBUTION OF THE SAMPLE CODE,*
 * OR COMBINATION OF THE SAMPLE CODE WITH ANY OTHER CODE. IN NO EVENT SHALL IBM OR ITS*
 * LICENSORS AND SUPPLIERS BE LIABLE FOR ANY LOST REVENUE, LOST PROFITS OR DATA, OR   *
 * FOR DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL,INCIDENTAL OR PUNITIVE DAMAGES,       *
 * HOWEVER CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY,-, EVEN IF IBM OR ITS     *
 * LICENSORS OR SUPPLIERS HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.       *
 *                                                                                    *
 * @endCopyright                                                                      *
 **************************************************************************************

Deletes All Items Created of Item Types of the Sample Data Model.
Deletes All Item Types of the Sample Data Model.

IMPORTANT: File must be modfied for RM Configuration / Preferences.  
           Please modify the "Defaults for resource manager" section.

********************************************************************/

// Includes / Imports
#include <iostream.h>
#include <DKDatastoreICM.hpp>
#include "SConnectDisconnectICM.hpp"
#include "SAttributeGroupDefDeletionICM.hpp"
#include "SAttributeDefinitionDeletionICM.hpp"
#include "SItemDeletionICM.hpp"
#include "SItemTypeDeletionICM.hpp"
#include "SLinkTypeDefinitionDeletionICM.hpp"
#include "SReferenceAttrDefDeletionICM.hpp"
#include "SResourceMgrDefDeletionICM.hpp"
#include "SVideoItemTypeDeleteICM.hpp"
#include "SSMSCollectionDefDeletionICM.hpp"
#include "SSampleModelDeleteICM.hpp"

/************************************************************************************************
 *          FILENAME: SSampleModelDeleteICM.cpp
 *                    ---------------------------------------------------------------------------
 *       DESCRIPTION: Deletes Items & Item Types of the Sample Data Model using Samples.
 *                    ---------------------------------------------------------------------------
 *     DEMONSTRATION: Samples
 *                    ---------------------------------------------------------------------------
 * COMMANDLINE USAGE: SSampleModelDeleteICM <database> <userName> <password>
 *                    ---------------------------------------------------------------------------
 *      PREREQUISITS: File must be modfied for RM Configuration / Preferences.  Please modify
 *                    the "Defaults for resource manager" section.
 *                    ---------------------------------------------------------------------------
 *    FILES REQUIRED: SConnectDisconnectICM.hpp/cpp
 *                    SAttributeGroupDefDeletionICM.hpp/cpp
 *                    SAttributeDefinitionDeletionICM.hpp/cpp
 *                    SItemCreationICM.hpp/cpp
 *                    SItemDeletionICM.hpp/cpp
 *                    SItemTypeRetrievalICM.hpp/cpp
 *                    SItemTypeDeletionICM.hpp/cpp
 *                    SLinkTypeDefinitionDeletionICM.hpp/cpp
 *                    SReferenceAttrDefDeletionICM.hpp/cpp
 *                    SResourceMgrDefDeletionICM.hpp/cpp
 *                    SResourceMgrDefRetrievalICM.hpp/cpp
 *                    SSMSCollectionDefDeletionICM.hpp/cpp
 *                    SSMSCollectionDefRetrievalICM.hpp/cpp
 *                    SVideoItemTypeDeleteICM.hpp/cpp
 *                    SSampleModelDeleteICM_launch.hpp/cpp
 *                    SampleModelDeleteICM.hpp
 ************************************************************************************************/

    //-------------------------------------------------------------
    // Main
    //-------------------------------------------------------------
    /**
     * Run the Sample.
     * @param argc   Argument Count / Number of Arguments Specified
     * @param argv[] String Array containing arguments.  Optional arguments are <databse> <userName> <password>.
     * @return  Returns SConnectDisconnectICM_RC_SAMPLE_PASSED (0) if no sample program completed, SConnectDisconnectICM_RC_SAMPLE_FAILED (-1) if any errors / exceptions were caught.
     **/
    int SSampleModelDeleteICM::main(int argc, char *argv[]){

        // Defaults for connecting to the database.
        char * database = SConnectDisconnectICM_DEFAULT_DATABASE;
        char * userName = SConnectDisconnectICM_DEFAULT_USERNAME;
        char * password = SConnectDisconnectICM_DEFAULT_PASSWORD;

        // Defaults for resource manager
        char * rmReferenceName  = "MY_RM";

        //------------------------------------------------------------
        // Checking for input parameters
        //--------------------------------------------------------------
        if (argc < 4) { // if not all 3 arguments were specified, use defaults and report correct usage.
            cout << "Usage: "                                                       << endl;
            cout << "  SSampleModelDeleteICM <database> <userName> <password>"   << endl;
            cout << "  *** Some parameters not specified, using defaults..."        << endl;
            cout << endl;
        } else {  // otherwise enough parameters were specified, use the first 3 for the 3 parameters.
            if (argc > 1) database = argv[1];
            if (argc > 2) userName = argv[2];
            if (argc > 3) password = argv[3];
        }//end else
        char * ver = SConnectDisconnectICM_EIP_VERSION;

        cout << "==========================================="     << endl;
        cout << "IBM Information Integrator for Content v" << ver << endl;
        cout << "Sample Program:  SSampleModelDeleteICM"          << endl;
        cout << "-------------------------------------------"     << endl;
        cout << " Database: " << database                         << endl;
        cout << " UserName: " << userName                         << endl;
        cout << "===========================================\n"   << endl;

        try{

            char ** args = new char*[4];
            int argsc = 4;
            args[0] = new char[256];
            args[1] = new char[256];
            args[2] = new char[256];
            args[3] = new char[256];
            // specify the arguments to pass to object's main method
            strcpy(args[0], argv[0]);
            strcpy(args[1], database);
            strcpy(args[2], userName);
            strcpy(args[3], password);

            char ** rmArgs = new char*[5];
            int rmArgc = 5;
            rmArgs[0] = new char[256];
            rmArgs[1] = new char[256];
            rmArgs[2] = new char[256];
            rmArgs[3] = new char[256];
            rmArgs[4] = new char[256];
            // specify the arguments to pass to object's main method
            strcpy(rmArgs[0], argv[0]);
            strcpy(rmArgs[1], rmReferenceName);
            strcpy(rmArgs[2], database);
            strcpy(rmArgs[3], userName);
            strcpy(rmArgs[4], password);

            // Run Samples
            
            // Delete Items
            DKDatastoreICM* dsICM = SConnectDisconnectICM::connect(database,userName,password,"");
            SItemDeletionICM::deleteAllItems(dsICM, "S_simple");
            SItemDeletionICM::deleteAllItems(dsICM, "S_withMultAttrs");
            SItemDeletionICM::deleteAllItems(dsICM, "S_autoClaim");
            SItemDeletionICM::deleteAllItems(dsICM, "S_adjustrReport");
            SItemDeletionICM::deleteAllItems(dsICM, "S_policeReport");
            SItemDeletionICM::deleteAllItems(dsICM, "S_insurePolicy");
            SItemDeletionICM::deleteAllItems(dsICM, "S_autoPhoto");
            SItemDeletionICM::deleteAllItems(dsICM, "S_lob");
            SItemDeletionICM::deleteAllItems(dsICM, "S_text");
            SItemDeletionICM::deleteAllItems(dsICM, "S_image");
            SItemDeletionICM::deleteAllItems(dsICM, "S_withChild");
            SItemDeletionICM::deleteAllItems(dsICM, "S_videoType");
            SConnectDisconnectICM::disconnect(dsICM);

            // Video Item Type
            SVideoItemTypeDeleteICM::main(argsc,args);

            // Link Type Definition
            SLinkTypeDefinitionDeletionICM::main(argsc,args);
            
            // Delete Data Model Definition
            SItemTypeDeletionICM::main(argsc,args);
            SReferenceAttrDefDeletionICM::main(argsc,args);
            SAttributeGroupDefDeletionICM::main(argsc,args);
            SAttributeDefinitionDeletionICM::main(argsc,args);
            
            // Resource Mgr Setup
            SSMSCollectionDefDeletionICM::main(rmArgc,rmArgs);
            SResourceMgrDefDeletionICM::main(rmArgc,rmArgs);   
            
            delete(dsICM);  // Free Memory
    
            //-------------------------------------------------------------
            // Sample program completed without exception
            //-------------------------------------------------------------
            cout << "\n==========================================" << endl;
            cout << "Sample program completed."                    << endl;
            cout << "==========================================\n" << endl;
        }
        //------------------------------------------------------------
        // Catch & Print Exceptions        
        //------------------------------------------------------------
        catch (DKException &exc){
            SConnectDisconnectICM::printException(exc);    // Print the exception to the screen
            return(SConnectDisconnectICM_RC_SAMPLE_FAILED);  // Report using return code that it failed
        }
        return(SConnectDisconnectICM_RC_SAMPLE_PASSED);    // If no exceptions were thrown, return a code to indicate passed.
    }// end main

    //=================================================================
    // Wrapper Functions
    //=================================================================
    // The following are wrapper functions for functionality covered
    // in this sample.  These functions may be used by other samples.

    // No additional wrapper functions from this sample.
