/*BEGINPROLOGUE************************************************************************
 * @copyright(disclaimer)                                                             *
 * Licensed Materials - Property of IBM                                               *
 * IBM Information Integrator for Content V8.2  (program number 5724-B43)             *
 * (c ) Copyright IBM Corp. 1998, 2002, 2003. All Rights Reserved.                    *
 *                                                                                    *
 * US Government Users Restricted Rights                                              *
 * Use, duplication or disclosure restricted by GSA ADP Schedule                      *
 * Contract with IBM Corp.                                                            *
 *                                                                                    *
 * DISCLAIMER OF WARRANTIES :                                                         *
 *                                                                                    *
 * Permission is granted to copy and modify this Sample code, and to                  *
 * distribute modified versions provided that both the copyright                      *
 * notice,- and this permission notice and warranty disclaimer appear in all copies   * 
 * and modified versions.                                                             *
 *                                                                                    *
 * THIS SAMPLE CODE IS LICENSED TO YOU AS-IS.                                         *
 * IBM AND ITS SUPPLIERS AND LICENSORS DISCLAIM ALL WARRANTIES, EITHER EXPRESS OR     *
 * IMPLIED, IN SUCH SAMPLE CODE, INCLUDING THE WARRANTY OF NON-INFRINGEMENT AND THE   *
 * IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE. IN NO   *
 * EVENT WILL IBM OR ITS LICENSORS OR SUPPLIERS BE LIABLE FOR ANY DAMAGES ARISING OUT *
 * OF THE USE OF OR INABILITY TO USE THE SAMPLE CODE, DISTRIBUTION OF THE SAMPLE CODE,*
 * OR COMBINATION OF THE SAMPLE CODE WITH ANY OTHER CODE. IN NO EVENT SHALL IBM OR ITS*
 * LICENSORS AND SUPPLIERS BE LIABLE FOR ANY LOST REVENUE, LOST PROFITS OR DATA, OR   *
 * FOR DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL,INCIDENTAL OR PUNITIVE DAMAGES,       *
 * HOWEVER CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY,-, EVEN IF IBM OR ITS     *
 * LICENSORS OR SUPPLIERS HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.       *
 *                                                                                    *
 * @endCopyright                                                                      *
 **************************************************************************************

NOTE:  When retrieving Semantic Type Definitions, if it does not exist, null is returned.  
       Usage of the null object will result in a NullPointerException.

********************************************************************/

// Includes / Imports
#include <iostream.h>
#include <DKDatastoreICM.hpp>
#include <DKDatastoreDefICM.hpp>
#include <DKSemanticTypeDefICM.hpp>
#include <DKConstant.h>
#include "SConnectDisconnectICM.hpp"
#include "SSemanticTypeCreationICM.hpp"
#include "SSemanticTypeRetrievalICM.hpp"

/************************************************************************************************
 *          FILENAME: SSemanticTypeRetrievalICM.cpp
 *                    ---------------------------------------------------------------------------
 *       DESCRIPTION: Retrieving, Printing, & Listing Semantic Type Definitions in the System.
 *                    ---------------------------------------------------------------------------
 *     DEMONSTRATION: Retrieving Semantic Type Definitions
 *                    Printing Semantic Type Definitions
 *                    Listing All Semantic Types Available in the System
 *                    Obtaining All Semantic Type Definitions in the System
 *                    ---------------------------------------------------------------------------
 * COMMANDLINE USAGE: SSemanticTypeRetrievalICM <database> <userName> <password>
 *                    ---------------------------------------------------------------------------
 *      PREREQUISITS: Semantic Types must already be defined.  If not, please run Sample
 *                    SSemanticTypeCreationICM first.
 *                    ---------------------------------------------------------------------------
 *    FILES REQUIRED: SConnectDisconnectICM.hpp/cpp
 *                    SSemanticTypeRetrievalICM_launch.cpp
 *                    SSemanticTypeRetrievalICM.hpp
 ************************************************************************************************/

    //-------------------------------------------------------------
    // Main
    //-------------------------------------------------------------
    /**
     * Run the Sample.
     * @param argc   Argument Count / Number of Arguments Specified
     * @param argv[] String Array containing arguments.  Optional arguments are <databse> <userName> <password>.
     * @return  Returns SConnectDisconnectICM_RC_SAMPLE_PASSED (0) if no sample program completed, SConnectDisconnectICM_RC_SAMPLE_FAILED (-1) if any errors / exceptions were caught.
     **/
    int SSemanticTypeRetrievalICM::main(int argc, char *argv[]){

        // Defaults for connecting to the database.
        char * database = SConnectDisconnectICM_DEFAULT_DATABASE;
        char * userName = SConnectDisconnectICM_DEFAULT_USERNAME;
        char * password = SConnectDisconnectICM_DEFAULT_PASSWORD;

        //------------------------------------------------------------
        // Checking for input parameters
        //--------------------------------------------------------------
        if (argc < 4) { // if not all 3 arguments were specified, use defaults and report correct usage.
            cout << "Usage: "                                                        << endl;
            cout << "  SSemanticTypeRetrievalICM <database> <userName> <password>"   << endl;
            cout << "  *** Some parameters not specified, using defaults..."         << endl;
            cout << endl;
        } else {  // otherwise enough parameters were specified, use the first 3 for the 3 parameters.
            if (argc > 1) database = argv[1];
            if (argc > 2) userName = argv[2];
            if (argc > 3) password = argv[3];
        }//end else
        char * ver = SConnectDisconnectICM_EIP_VERSION;

        cout << "==========================================="     << endl;
        cout << "IBM Information Integrator for Content v" << ver << endl;
        cout << "Sample Program:  SSemanticTypeRetrievalICM"      << endl;
        cout << "-------------------------------------------"     << endl;
        cout << " Database: " << database                         << endl;
        cout << " UserName: " << userName                         << endl;
        cout << "===========================================\n"   << endl;

        try{
            //-------------------------------------------------------------
            // Connect to datastore
            //-------------------------------------------------------------
            // See Sample SConnectDisconnectICM for more information
            cout << "Connecting to datastore (Database '" << database << "', UserName '" << userName << "')..." << endl;

                DKDatastoreICM* dsICM = new DKDatastoreICM();  // Create new datastore object.
                dsICM->connect(database,userName,password,""); // Connect to the datastore.

            cout << "Connected to datastore (Database '" << dsICM->datastoreName() << "', UserName '" << dsICM->userName() << "')." << endl;

            //----------------------------------------------------------------
            // Obtaining all Semantic Type Definitions in the System
            //----------------------------------------------------------------
            cout << "Obtaining All Semantic Type Definitions in the System..." << endl;
        
        	    DKDatastoreDefICM* dsDefICM = (DKDatastoreDefICM*) dsICM->datastoreDef(); // Get the datastore definition object from the connected datastore.
        	    
        	    dkCollection* semTypeColl = dsDefICM->listSemanticTypes(); // Obtain all of the Semantic Type Definition objects in the system.
        	    // NOTE: dsDefICM->listSemanticTypeName() could also be used to just obtain the names.

            cout << "Obtained All Semantic Type Definitions in the System." << endl;

            //----------------------------------------------------------------
            // List all Semantic Types Available in the System
            //----------------------------------------------------------------
            // Uses the semantic type definitions retrieved above.
            cout << "Listing All Semantic Types Available In System..." << endl;

        	    // Print each in the list retrieved above.
        	    cout << "All Semantic Types in System:  (" << semTypeColl->cardinality() << ")" << endl;
        	    dkIterator* iter = semTypeColl->createIterator(); // create an iterator go iterate through collection.
                while(iter->more()){                              // while there still more items in the colleciton, continue
        	        DKSemanticTypeDefICM* semTypeDef = (DKSemanticTypeDefICM*) iter->next()->value(); // Move pointer to next element & return that object.
        	        cout << "     - " << semTypeDef->getName() << ":  " << semTypeDef->getDescription() << endl;
                    delete(semTypeDef);                           // Free Memory.  Contents are not deleted when collection deleted.
        	    }
                delete(iter);                                     // Free Memory.
                delete(semTypeColl);                              // Free Memory.  Contents are not deleted in this call.

            cout << "Listed All Semantic Types Available In System." << endl;

            //----------------------------------------------------------------
            // Retrieve the S_policeReports Attribute Definition
            //----------------------------------------------------------------
            // For more information about the defined Semantic Types, please consult
            // the SSemanticTypeCreationICM sample.  For more information
            // on usage, please consult the SItemCreationICM Sample.
            //
            // NOTE:  When retrieving Semantic Type Definitions, if it does not exist, null is 
            //        returned.  Usage of the null object will result in a NullPointerException.
            cout << "Retrieving the Semantic Type Definition of 'S_myDocument'..." << endl;

                DKSemanticTypeDefICM* semTypeDef = (DKSemanticTypeDefICM*) dsDefICM->retrieveSemanticType("S_myDocument"); // Retrieve the definition.
                
            cout << "Retrieved the Semantic Type Definition of 'S_myDocument'." << endl;
            
            //---------------------------------------------------------------
            // Print the Semantic Type Definition
            //---------------------------------------------------------------
            cout << "Printing the Semantic Type Definition of 'S_myDocument'..." << endl;

                printSemanticTypeDef(semTypeDef); // Use wrapper funciton from this Sample.
                delete(semTypeDef);               // Free Memory

            cout << "Printed the Semantic Type Definition of 'S_myDocument'." << endl;

            //-------------------------------------------------------------
            // Disconnect from datastore & Destroy Reference
            //-------------------------------------------------------------
            // See Sample SConnectDisconnectICM for more information
            cout << "Disconnecting from datastore & destroying reference..." << endl;            

                dsICM->disconnect();
                delete(dsICM);

            cout << "Disconnected from datastore & destroying reference." << endl;
    
            //-------------------------------------------------------------
            // Sample program completed without exception
            //-------------------------------------------------------------
            cout << "\n==========================================" << endl;
            cout << "Sample program completed."                    << endl;
            cout << "==========================================\n" << endl;
        }
        //------------------------------------------------------------
        // Catch & Print Exceptions        
        //------------------------------------------------------------
        catch (DKException &exc){
            SConnectDisconnectICM::printException(exc);    // Print the exception to the screen
            return(SConnectDisconnectICM_RC_SAMPLE_FAILED);  // Report using return code that it failed
        }
        return(SConnectDisconnectICM_RC_SAMPLE_PASSED);    // If no exceptions were thrown, return a code to indicate passed.
    }// end main

    //=================================================================
    // Wrapper Functions
    //=================================================================
    // The following are wrapper functions for functionality covered
    // in this sample.  These functions may be used by other samples.

   /**
    * List all Semantic Types available in the system.
    * @param dsICM  Connected DKDatastoreICM object.
    **/
    void SSemanticTypeRetrievalICM::listSemanticTypes(DKDatastoreICM* dsICM){

        // Obtaining all Semantic Type Definitions in the System
        DKDatastoreDefICM* dsDefICM = (DKDatastoreDefICM*) dsICM->datastoreDef(); // Get the datastore definition object from the connected datastore.
        dkCollection* semTypeColl = dsDefICM->listSemanticTypes();    // Obtain all of the Semantic Type Definition objects in the system.

        // List all Semantic Types Available in the System
        cout << "All Semantic Types in System:  (" << semTypeColl->cardinality() << ")" << endl;
        dkIterator* iter = semTypeColl->createIterator(); // create an iterator go iterate through collection.
        while(iter->more()){                              // while there still more items in the colleciton, continue
        	DKSemanticTypeDefICM* semTypeDef = (DKSemanticTypeDefICM*) iter->next()->value(); // Move pointer to next element & return that object.
        	cout << "     - " << semTypeDef->getName() << ":  " << semTypeDef->getDescription() << endl;
            delete(semTypeDef);                           // Free Memory.  Will not be deleted with collection.
        }
        delete(iter);                                     // Free Memory.
        delete(semTypeColl);
    }//end listSemanticTypes

   /**
    * Print the specified Semantic Type Definition
    * @param semTypeDef  Semantic Type Definition, DKSemanticTypeDefICM.
    **/
    void SSemanticTypeRetrievalICM::printSemanticTypeDef(DKSemanticTypeDefICM* semTypeDef){
        cout << "--------------------------------------------"       << endl;
        cout << "            Semantic Type Definition"               << endl;
        cout << "---------------------------------------------"      << endl;
        cout << "           Name:  " << semTypeDef->getName()        << endl;
        cout << "    Description:  " << semTypeDef->getDescription() << endl;
        cout << "        ID Code:  " << semTypeDef->getCode()        << endl;
        cout << "---------------------------------------------"      << endl;
    }

   /**
    * Print the specified Semantic Type Definition
    * @param dsICM         Connected DKDatastoreICM object.
    * @param semTypeName   Semantic Type Name.
    **/
    void SSemanticTypeRetrievalICM::printSemanticTypeDef(DKDatastoreICM* dsICM, char* semanticTypeName){
        // First, we need acces to the Datastore Definition object.
        DKDatastoreDefICM* dsDefICM = (DKDatastoreDefICM*) dsICM->datastoreDef(); // Get the datastore definition object from the connected datastore.
        DKSemanticTypeDefICM* semTypeDef = (DKSemanticTypeDefICM*) dsDefICM->retrieveSemanticType(semanticTypeName); // Retrieve the definition.
        printSemanticTypeDef(semTypeDef);   
        delete(semTypeDef);                                         // Free Memory
    }
