/*
@copyright(disclaimer)

Licensed Materials - Property of IBM
IBM  Enterprise Information Portal  for Multiplatforms V8.1  (program number 5724-B43)
(c) Copyright IBM Corp. 1998,  2002. All Rights Reserved.

US Government Users Restricted Rights
Use, duplication or disclosure restricted by GSA ADP Schedule
Contract with IBM Corp.

DISCLAIMER OF WARRANTIES :

Permission is granted to copy and modify this  Sample code, and to
distribute modified versions provided that both the copyright
notice,- and this permission notice and warranty disclaimer  appear in all copies and modified
versions.

THIS SAMPLE CODE IS LICENSED TO YOU AS-IS.
IBM  AND ITS SUPPLIERS AND LICENSORS  DISCLAIM
ALL WARRANTIES, EITHER EXPRESS OR IMPLIED, IN SUCH SAMPLE CODE, INCLUDING THE WARRANTY
OF NON-INFRINGEMENT AND THE IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A
PARTICULAR PURPOSE. IN NO EVENT WILL IBM OR ITS LICENSORS OR SUPPLIERS BE LIABLE FOR ANY
DAMAGES ARISING OUT OF THE USE OF  OR INABILITY TO USE THE SAMPLE CODE, DISTRIBUTION OF
THE SAMPLE CODE, OR COMBINATION OF THE SAMPLE CODE WITH ANY OTHER CODE. IN NO EVENT SHALL
IBM OR ITS LICENSORS AND SUPPLIERS BE LIABLE FOR ANY LOST REVENUE, LOST PROFITS OR DATA,
OR FOR DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL,INCIDENTAL OR PUNITIVE DAMAGES, HOWEVER
CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY,-, EVEN IF IBM OR ITS LICENSORS OR SUPPLIERS
HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.

@endCopyright
*/

//*****************************************************************************
// Class Name  = SampleGuiResults
// Description = Sample visual schema management bean
// Author      = mimivo
// Date        = 04/01/1999
//
//*****************************************************************************

import com.ibm.mm.beans.*;
import com.ibm.mm.beans.util.*;

import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.FocusEvent;
import java.awt.event.FocusAdapter;
import java.awt.event.FocusListener;
import java.beans.*;
import java.io.*;
import java.util.*;

import javax.swing.*;
import javax.swing.border.*;
import javax.swing.table.*;
import javax.swing.event.*;

//*****************************************************************************
// class/interface definition
//*****************************************************************************
/**
 * Sample GUI bean to prompt for user ID and password. Can be used with CM
 * connection bean to perform the functions.
 *
 * <pre>
 *   standalone properties  cache, traceEnabled.
 *   source of events       CMBSchemaRequestEvent     - to list catalog info
 * </pre>
 */
public class SampleGuiResults
    implements Serializable, CMBResultListener, CMBConnectionReplyListener, TemplateChangeListener, CMBUtilConstant {

    //*****************************************************************************
    // constructor
    //*****************************************************************************
    /**
     * Default constructor.
     */
    public SampleGuiResults() {
        listenerVector = new Vector();
        initGUI();
    }

    public SampleGuiResults(CMBConnection connBean) {
        listenerVector = new Vector();
        dataBean = connBean.getDataManagement();
        initGUI();
    }

    /**
     * Destructor,  dispose the window.
     */
    public void finalize() {
        frame.dispose();
    }

    public void onCMBConnectionReply(CMBConnectionReplyEvent evt) {
        int status = evt.getStatus();
        if (status == CMBBaseConstant.CMB_STATUS_FAILED) {
            return;
        }
        System.out.println("In onCMBConnectionReply method: ....");
        System.out.println("status = " + status);

        if (evt.getID() == CMBConnectionReplyEvent.CMB_REPLY_CONNECT) {
            CMBConnection connBean = (CMBConnection) evt.getData();
            dataBean = connBean.getDataManagement();
        }
    }

    public void setConnection(CMBConnection connBean) {
        dataBean = connBean.getDataManagement();
        schemaBean = connBean.getSchemaManagement();
    }

    public void onCMBResult(CMBResultEvent evt) {
        int id = evt.getID();
        CMBSearchResults res = (CMBSearchResults) evt.getSource();
        //Vector  resList = (Vector)evt.getData();

        System.out.println("Got CMBResutlEvent .......................");
        System.out.println("id=" + id);
        switch (id) {
            case CMBResultEvent.CMB_RESULT_UPDATED :
                CMBItem[] hitList = res.getItem();
                //CMBItem[] hitList = new CMBItem[resList.size()];
                //resList.copyInto(hitList);
                System.out.println("SampleGuiResults: Updating search result with " + hitList.length + " hit items.");
                dataModel.loadData(hitList);
                break;
            case CMBResultEvent.CMB_RESULT_CLEARED :
                System.out.println("SampleGuiResults: clearing...");
                dataModel.resetData();
                frame.repaint();
                contentPane.repaint();
                break;
        }

    }

    public void onTemplateChange(TemplateChangeEvent evt) {
        int id = evt.getID();
        String stName = (String) evt.getData();
        CMBSearchTemplate stObj = null;

        if (stName == null || stName.length() == 0) {
            System.out.println("Template name is null === returning....");
            return;
        }

        try {

            CMBSchemaManagement schemaBean = (CMBSchemaManagement) evt.getExtraData();
            stObj = schemaBean.getSearchTemplate(stName);

        } catch (Exception ex) {
            ex.printStackTrace();
        }

        updateUI(stObj);
    }

    //*****************************************************************************
    // ItemRequestEvent
    //*****************************************************************************
    /**
     * ItemRequestEvent registration method.
     * It ignores a registration request if the listener is already registered.
     * @param listener listener to be registered
     */
    public void addItemRequestListener(ItemRequestListener listener) {
        if (!listenerVector.contains(listener))
            listenerVector.addElement(listener);
    }

    /**
     * ItemRequestEvent unregistration method.
     * @param listener listener to be unregistered
     */
    public void removeItemRequestListener(ItemRequestListener listener) {
        if (listenerVector.contains(listener))
            listenerVector.removeElement(listener);
    }

    /**
     *	Get the template change event object.
     *  @return ItemRequestEvent object
     */
    public ItemRequestEvent getItemRequestEvent() {
        return itemRequestEvt;
    }

    //*****************************************************************************
    // property accessors
    //*****************************************************************************
    /**
     * Property getter.
     * @return  window title
     */
    public String getWindowTitle() {
        return frame.getTitle();
    }

    /**
     * Property setter.
     * @param   newValue new window title
     */
    public void setWindowTitle(String newValue) {
        if (newValue != null)
            frame.setTitle(newValue);
    }

    public boolean isVisible() {
        return frame.isVisible();
    }

    public void updateUI(CMBSearchTemplate stObj) {
        String pName = stObj.getName();

        System.out.println("new  stName = " + pName);
        System.out.println("prev stName = " + plateName);

        if (pName.equals(plateName))
            return;

        String[] dispList = null;

        try {
            //First get the display list
            dispList = stObj.getCriterionDisplayName();

            //if not there, just get the criteria list
            if (dispList == null)
                dispList = stObj.getCriterionName();
        } catch (Exception ex) {
            ex.printStackTrace();
        }

        Vector names = new Vector();
        for (int i = 0; dispList != null && i < dispList.length; i++) {
            names.addElement(dispList[i]);
        }

        dataModel = new UtilTableModel(names);
        table = new JTable(dataModel);
        dataModel.setTable(table);
        table.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);

        table.setAutoResizeMode(JTable.AUTO_RESIZE_ALL_COLUMNS);
        //table.sizeColumnsToFit(-1);
        scrollPane = new JScrollPane(table);

        contentPane.removeAll();
        contentPane.add(scrollPane, BorderLayout.CENTER);

        System.out.println("prv template Name = " + plateName);
        frame.setVisible(true);
        frame.repaint();
        contentPane.doLayout();
        contentPane.repaint();
        frame.requestFocus();
    }

    //*****************************************************************************
    // Private methods
    //*****************************************************************************
    // Serialize transient data
    //*****************************************************************************
    /**
     * @serialData the data is serialized in the following sequence:
     * <pre>
     *    this object
     *    event listeners if the event listener is serializable
     *    a null object   tells the end of serialization
     * </pre>
     */
    private void writeObject(ObjectOutputStream stream) throws IOException {
        stream.defaultWriteObject();

        Vector v = null;
        synchronized (this) {
            if (listenerVector.size() > 0)
                v = (Vector) listenerVector.clone();
        }

        if (v != null) {
            int cnt = v.size();
            CMBSchemaReplyListener listener = null;
            for (int i = 0; i < cnt; ++i) {
                listener = (CMBSchemaReplyListener) v.elementAt(i);
                if (listener instanceof Serializable)
                    stream.writeObject(listener);
            }
        }

        stream.writeObject(null); // tells the end of listener
    }

    //*****************************************************************************
    // De-serialize/initialize transient data
    //*****************************************************************************
    /**
    * @serialData the data is deserialized in the following sequence:
    * <pre>
    *    this object
    *    event listeners may be none
    *    a null object   the marker of serialization
    * </pre>
    */
    private void readObject(ObjectInputStream stream) throws Exception {
        listenerVector = new Vector(); // always initialize listener list
        stream.defaultReadObject();

        Object o;
        while ((o = stream.readObject()) != null)
            listenerVector.addElement((CMBSchemaReplyListener) o);
        frame.setVisible(true);
    }

    //*****************************************************************************
    // initGUI
    //*****************************************************************************
    private void initGUI() {

        frame = new JFrame("Search Results");
        mainPane = new CMBBasePanel();

        Border raisedBorder = new SoftBevelBorder(BevelBorder.RAISED);
        Border loweredBorder = new SoftBevelBorder(BevelBorder.LOWERED);

        //Menu bar
        itemMenu.add(retrieveMenu);
        itemMenu.add(copyMenu);
        attrMenu.add(viewMenu);
        attrMenu.add(updateMenu);
        attrMenu.addSeparator();
        attrMenu.add(reindexMenu);
        attrMenu.add(unindexMenu);
        helpMenu.add(aboutMenu);
        menuBar.add(itemMenu);
        menuBar.add(attrMenu);
        menuBar.add(helpMenu);
        frame.setJMenuBar(menuBar);

        _setDefaultEvents(); //for all menu items

        //*****************************************************************************
        // Widgets creation and event listener setup
        //*****************************************************************************
        // Center panel
        //*****************************************************************************
        contentPane = new JPanel();
        contentPane.setLayout(new BorderLayout());

        //*****************************************************************************
        // Add widgets to window frame
        //*****************************************************************************
        mainPane.addCenterPanel(contentPane);
        frame.getContentPane().add(mainPane, BorderLayout.CENTER);
        frame.setSize(400, 300);
        //frame.pack();
        CMBUtil.setLocation(frame, CMBUtil.CENTER);
        frame.setVisible(false);
        //frame.requestFocus();
    }

    private void _setDefaultEvents() {
        retrieveMenu.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent ae) {
                action = ItemRequestEvent.ITEM_RETRIEVE;
                _invokeViewer();
            }
        });

        copyMenu.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent ae) {
                action = ItemRequestEvent.ITEM_COPYCREATE;
                _invokeAttrEditor();
            }
        });

        viewMenu.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent ae) {
                action = ItemRequestEvent.ATTR_VIEW;
                _invokeAttrEditor();
            }
        });

        updateMenu.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent ae) {
                action = ItemRequestEvent.ATTR_UPDATE;
                _invokeAttrEditor();
            }
        });

        reindexMenu.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent ae) {
                action = ItemRequestEvent.ATTR_REINDEX;
                _invokeAttrEditor();
            }
        });

        unindexMenu.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent ae) {
                action = ItemRequestEvent.ATTR_UNINDEX;
                _invokeAttrEditor();
            }
        });

        aboutMenu.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent ae) {
                String msg =
                    "Welcome to IBM Enterprise Information Portal Sample Application\n\n"
                        + "IBM Enterprise Information Portal for Multiplatforms V6.1 (program number 5648-D03)\n"
                        + "(c ) Copyright IBM Corp. 1998, 1999.\n"
                        + "IBM Content Manager for Multiplatforms V6.1 (program number 5648-D04)\n"
                        + "(c ) Copyright IBM Corp. 1994, 1999.";
                JOptionPane.showMessageDialog(
                    frame,
                    msg,
                    "About: IBM Enterprise Information Portal Sample Application",
                    JOptionPane.INFORMATION_MESSAGE);
            }
        });
    }

    private void _invokeViewer() {
        int selIndex = table.getSelectedRow();
        CMBItem item = dataModel.getItemObject(selIndex);
        if (item == null)
            System.out.println("SampleGuiResults:_invokeAttrEditor() - item is null");
        else
            System.out.println("SampleGuiResults:_invokeAttrEditor() - item is not null");
        try {
            dataBean.setDataObject(item);

            // Make sure there is content
            if (dataBean.getContentCount() == 0) {
                JOptionPane.showMessageDialog(
                    frame,
                    "There is no content for this document",
                    "No Content",
                    JOptionPane.INFORMATION_MESSAGE);
                return;
            }

            // Determine application to launch for mimetype
            Properties mime2appProps = schemaBean.getDefaultMime2App();
            String mimeType = dataBean.getItemMimeType();
            String appString = (String) mime2appProps.get(mimeType);
            if (appString == null) {
                JOptionPane.showMessageDialog(
                    frame,
                    "There is no viewer defined to view this document (mimetype: "
                        + mimeType
                        + ")\n"
                        + "Define an application for viewing "
                        + mimeType
                        + " using EIP Administration.",
                    "No Viewer",
                    JOptionPane.INFORMATION_MESSAGE);
                return;
            }
            System.out.println(appString);
            int appIndex = appString.indexOf("a=");
            int extIndex = appString.indexOf("x=");
            String app;
            String ext;
            if (extIndex > 0) {
                app = appString.substring(appIndex + 2, extIndex - 1);
                ext = appString.substring(extIndex + 2);
            } else {
                app = appString.substring(appIndex + 2);
                ext = "cmb";
            }

            // Write document to file
            String docFileName = "Doc" + docCounter + ext;
            docCounter++;
            FileOutputStream docFile = new FileOutputStream(docFileName);
            docFile.write(dataBean.getContent(0).getData());
            docFile.close();

            // Launch application to view document
            try {
                Runtime.getRuntime().exec(app + " " + docFileName);
            } catch (Exception e) {
                JOptionPane.showMessageDialog(
                    frame,
                    "The viewer application "
                        + app
                        + " could not be launched.\n"
                        + "Make sure the application is available, or change the application defined\n"
                        + "to view document of type "
                        + mimeType
                        + " using EIP Administration.",
                    "No Viewer",
                    JOptionPane.INFORMATION_MESSAGE);
            }
        } catch (Exception ex) {
            ex.printStackTrace();
        }
    }

    private void _invokeAttrEditor() {
        int selIndex = table.getSelectedRow();
        CMBItem item = dataModel.getItemObject(selIndex);
        if (item == null)
            System.out.println("SampleGuiResults:_invokeAttrEditor() - item is null");
        else
            System.out.println("SampleGuiResults:_invokeAttrEditor() - item is not null");

        CMBItem unMappedItem = item;
        try {
            dataBean.setDataObject(item);
            unMappedItem = dataBean.getUnMappedItem(item);
        } catch (CMBException ex) {
            ex.printStackTrace();
        }

        itemRequestEvt = new ItemRequestEvent(this, action, unMappedItem);
        fireItemRequest(itemRequestEvt);
    }

    private void fireItemRequest(ItemRequestEvent evt) {
        itemRequestEvt = evt;

        System.out.println("About to fireItemRequest event ...");
        Vector v;
        synchronized (this) {
            v = (Vector) listenerVector.clone();
        }

        int cnt = v.size();
        ItemRequestListener listener = null;

        for (int i = 0; i < cnt; ++i) {
            System.out.println("About fire ItemRequestEvent to listener #" + i);
            listener = (ItemRequestListener) v.elementAt(i);
            listener.onItemRequest(evt);
        }
    }

    //*****************************************************************************
    // private data
    //*****************************************************************************

    Vector paneList = new Vector(); // element type: spaneWidgets
    Vector paneNames = new Vector(); // element type: String template name
    int paneCount = 0;

    String plateName = ""; // current plate name
    JScrollPane scrollpane = null; // current widget set

    JTable table = null;
    UtilTableModel dataModel = null;
    //*****************************************************************************
    // GUI widgets, all labels are editable properties
    //*****************************************************************************
    private JFrame frame = null;
    private CMBBasePanel mainPane = null;
    private JPanel contentPane = null;
    private JScrollPane scrollPane = null;

    private JMenuBar menuBar = new JMenuBar();

    private JMenu itemMenu = new JMenu("Items");
    private JMenuItem retrieveMenu = new JMenuItem("Retrieve");
    private JMenuItem copyMenu = new JMenuItem("Copy and Create New");

    private JMenu attrMenu = new JMenu("Attributes");
    private JMenuItem updateMenu = new JMenuItem("Update");
    private JMenuItem reindexMenu = new JMenuItem("Re-Index");
    private JMenuItem unindexMenu = new JMenuItem("Un-Index");
    private JMenuItem viewMenu = new JMenuItem("View");

    private JMenu helpMenu = new JMenu("Help");
    private JMenuItem aboutMenu = new JMenuItem("About Sample...");

    private transient CMBDataManagement dataBean = null;
    private transient CMBSchemaManagement schemaBean = null;

    //*****************************************************************************
    //*****************************************************************************
    private int action = ItemRequestEvent.ATTR_VIEW;

    //*****************************************************************************
    // Transient data members
    //*****************************************************************************
    private transient Vector listenerVector = new Vector();
    public transient ItemRequestEvent itemRequestEvt = null;
    private transient int docCounter = 0;

    //*****************************************************************************
    // Copyright
    //*****************************************************************************
    private static final String ibmid = "5648-D03 and 5648-D04 " + "(C) Copyright IBM Corp. 1994,1999";

}