/*BEGINPROLOGUE************************************************************************
 * @copyright(disclaimer)                                                             *
 * Licensed Materials - Property of IBM                                               *
 * IBM Enterprise Information Portal for Multiplatforms V8.1 (program number 5724-B43)*
 * (c ) Copyright IBM Corp. 1998,  2002. All Rights Reserved.                         *
 *                                                                                    *
 * US Government Users Restricted Rights                                              *
 * Use, duplication or disclosure restricted by GSA ADP Schedule                      *
 * Contract with IBM Corp.                                                            *
 *                                                                                    *
 * DISCLAIMER OF WARRANTIES :                                                         *
 *                                                                                    *
 * Permission is granted to copy and modify this Sample code, and to                  *
 * distribute modified versions provided that both the copyright                      *
 * notice,- and this permission notice and warranty disclaimer appear in all copies   * 
 * and modified versions.                                                             *
 *                                                                                    *
 * THIS SAMPLE CODE IS LICENSED TO YOU AS-IS.                                         *
 * IBM AND ITS SUPPLIERS AND LICENSORS DISCLAIM ALL WARRANTIES, EITHER EXPRESS OR     *
 * IMPLIED, IN SUCH SAMPLE CODE, INCLUDING THE WARRANTY OF NON-INFRINGEMENT AND THE   *
 * IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE. IN NO   *
 * EVENT WILL IBM OR ITS LICENSORS OR SUPPLIERS BE LIABLE FOR ANY DAMAGES ARISING OUT *
 * OF THE USE OF OR INABILITY TO USE THE SAMPLE CODE, DISTRIBUTION OF THE SAMPLE CODE,*
 * OR COMBINATION OF THE SAMPLE CODE WITH ANY OTHER CODE. IN NO EVENT SHALL IBM OR ITS*
 * LICENSORS AND SUPPLIERS BE LIABLE FOR ANY LOST REVENUE, LOST PROFITS OR DATA, OR   *
 * FOR DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL,INCIDENTAL OR PUNITIVE DAMAGES,       *
 * HOWEVER CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY,-, EVEN IF IBM OR ITS     *
 * LICENSORS OR SUPPLIERS HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.       *
 *                                                                                    *
 * @endCopyright                                                                      *
 **************************************************************************************/

import com.ibm.mm.sdk.common.*;
import com.ibm.mm.sdk.server.*;
import java.io.*;
import java.lang.*;

public class TAddDDOSampDL implements DKConstantDL 
{
  // Main method
  public static void main(String argv[])
  {
    System.out.println("begin...");
   
    // Checking for input parameters
    if (argv.length < 3) {
       System.out.println("Usage: " );
       System.out.println("  java TAddDDOSampDL <LibServer> <userid> <password>" );
       return;
    }
    
    String libSrv = argv[0];
    String userid = argv[1];
    String pw     = argv[2];
    
    String indexclass = "DLSAMPLE";

    System.out.println("index class = " + indexclass );

    DKDatastoreDL dl = null;
    try 
    {
       dl = new DKDatastoreDL();
           
       System.out.println("Connecting to datastore DL : " + libSrv);
       dl.connect(libSrv, userid, pw, "");
       System.out.println(" datastore connected" );
                   
       DKPid pid = new DKPid();
       pid.setObjectType(indexclass);
       
       DKDDO ddo = new DKDDO( dl, pid);
       
       // set the item type to be a document
       ddo.addProperty(DK_CM_PROPERTY_ITEM_TYPE, new Short(DK_CM_DOCUMENT));
       
       short dataId;
       String[] attrName = {
                            "DLSEARCH_Author",
                            "DLSEARCH_Publisher",
                            "DLSEARCH_DocType",
                            "DLSEARCH_Date",
                            "DLSEARCH_Title",
                            "DLSEARCH_Subject",
                            "DLSEARCH_Authors",
                            "DLSEARCH_Editors",
                            "DLSEARCH_Source",
                            "DLSEARCH_Volume",
                            "DLSEARCH_Issue",
                            "DLSEARCH_Relation"
                           };
       Object[] attrValue = new Object[12];
        attrValue[0] = new String("Robert");
        attrValue[1] = new String("IBM");
        attrValue[2] = new String("html");
        attrValue[3] = new String("2000");
        attrValue[4] = new String("Time after time");
        attrValue[5] = new String("Space and Time");
        attrValue[6] = new String("Robert, Jonathan");
        attrValue[7] = new String("Green publishing");
        attrValue[8] = new String("Space Quarterly");
        attrValue[9] = new String("2");
        attrValue[10] = new String("1");
        attrValue[11] = new String("");

       short[] attr_prop_type = {
                                 DK_CM_DATAITEM_TYPE_STRING,
                                 DK_CM_DATAITEM_TYPE_STRING,
                                 DK_CM_DATAITEM_TYPE_STRING,
                                 DK_CM_DATAITEM_TYPE_STRING,
                                 DK_CM_DATAITEM_TYPE_STRING,
                                 DK_CM_DATAITEM_TYPE_STRING,
                                 DK_CM_DATAITEM_TYPE_STRING,
                                 DK_CM_DATAITEM_TYPE_STRING,
                                 DK_CM_DATAITEM_TYPE_STRING,
                                 DK_CM_DATAITEM_TYPE_STRING,
                                 DK_CM_DATAITEM_TYPE_STRING,
                                 DK_CM_DATAITEM_TYPE_STRING
                                };

       short attrCount = (short) attrName.length;
       
       int i;
       for (i=0; i < attrCount; i++)
       {
          dataId = ddo.addData(attrName[i]);
          ddo.addDataProperty(dataId, DK_CM_PROPERTY_TYPE, new Short(attr_prop_type[i]));                // mandatory
          ddo.addDataProperty(dataId, DK_CM_PROPERTY_NULLABLE, new Integer(DK_CM_TRUE));                    // optional
          ddo.addDataProperty(dataId, DK_CM_PROPERTY_ACCESS_RIGHT, new Integer(DK_CM_ATTRIBUTE_READWRITE)); // optional
          ddo.setData(dataId, attrValue[i]);
       }
       
       System.out.println("\n *** displaying the ddo before adding it ***\n ");
       displayDDO(ddo);
      
       // add the document DDO to the DL persistent store
       System.out.println("\n *** adding the ddo *** \n" );

       ddo.add();

       pid = ddo.getPid();
       System.out.println(" the new added ddo itemId = " + pid.getPrimaryId() );     

       // retrieve it back to see it is really the same one
       System.out.println(" retrieve the newly added ddo, with itemId = " + pid.getPrimaryId() );     
       
       // create a new DDO for retrieving it
       DKDDO nddo = new DKDDO(dl, pid);
       try
       {
          nddo.retrieve();
       }
       catch (DKException e)
       {
         System.out.println(" item does not exist " + e.toString() );
         return;
       }
       catch (Exception exc)
       {
    	System.out.println("Exception message " + exc.getMessage());
    	exc.printStackTrace();
         return;
       }  
    
       System.out.println(" displaying the retrieved ddo" );
       displayDDO(nddo);

       // disconnect from DL datastore
       dl.disconnect();
       System.out.println("datastore disconnected");
       dl.destroy();
    }
    catch(DKException exc)
    {
     try {
     dl.destroy();
     }
     catch (Exception e)
     {
      e.printStackTrace();
     }
       System.out.println("Exception name " + exc.name());
       System.out.println("Exception message " + exc.getMessage());
       System.out.println("Exception error code " + exc.errorCode());
       System.out.println("Exception error state " + exc.errorState());
       exc.printStackTrace();
    }   
    catch (Exception exc)
    {
     try {
     dl.destroy();
     }
     catch (Exception e)
     {
      e.printStackTrace();
     }
     System.out.println("Exception message " + exc.getMessage());
     exc.printStackTrace();
    }

    System.out.println(" done" );
  }

  public static void displayDDO(DKDDO ddo)
  {
    short i, j;
    String dataName;

    try {
      System.out.println(" data count = " + ddo.dataCount());
      short itemType = ((Short)ddo.getPropertyByName(DK_CM_PROPERTY_ITEM_TYPE)).shortValue();
      if (itemType == DK_CM_FOLDER)
          System.out.println(" this item is a Folder, itemId = " + ddo.getPid().getPrimaryId());
      else if (itemType == DK_CM_DOCUMENT)
          System.out.println(" this item is a Document, itemId = " + ddo.getPid().getPrimaryId());
      else 
          System.out.println(" this item is of unknown type, type = " + itemType +
                                             ", itemId = " + ddo.getPid().getPrimaryId() );
      // print the properties of this DDO
      System.out.println(" properties of this DDO: " + ddo.propertyCount());
      for (i = 1; i <= ddo.propertyCount(); i++) {
           System.out.println("   property_name(" + i + ") = " + 
                                  ddo.getPropertyName(i) + ", value = " +
                                  ddo.getProperty(i));
      }

      // print out the content
      Object value;
      DKParts  parts;
      DKFolder folder;
      DKBlobDL blob;
      dkIterator iter;
      
      for ( i=1; i <= ddo.dataCount(); i++) {
           dataName = ddo.getDataName(i);
           System.out.print(" data item name (" + i + ") = " + dataName + " ");
           value = ddo.getData(i);
           if (value != null) 
                System.out.println("  \t value (" + i + ") = " + value);
           else System.out.println("  \t value (" + i + ") = null" );
           
           System.out.println(" properties of this data item: " + ddo.dataPropertyCount(i));
           for ( j=1; j <= ddo.dataPropertyCount(i); j++) {
               System.out.println("   property_name(" + i + "," + j + ") = " + 
                                      ddo.getDataPropertyName(i,j) + ", value = " +
                                      ddo.getDataProperty(i,j));
           }
           
           if (dataName.equals(DKFOLDER)) {
	         if (value != null) {
                folder = (DKFolder) value;
       		    iter = folder.createIterator();
       		    DKDDO mddo;
       		    while(iter.more()) {
       		      mddo = (DKDDO) iter.next();
       		      mddo.retrieve();               // retrieve a folder member
       		      displayDDO(mddo);
       		    }         
             }
           }
           
           if (dataName.equals(DKPARTS)) {
              if (value != null) {
                parts = (DKParts) value;
                iter  = parts.createIterator();
                while(iter.more()) {
                     blob = (DKBlobDL) iter.next();   
                     System.out.println(" \t\t blob part id " +
                       ((DKPidXDODL) (blob.getPidObject())).getPartId() +
                            " content class = " +
                            blob.getContentClass() +
                            " rep type " +
                            blob.getRepType() +
                            " aff type " +
                            blob.getAffiliatedType() );
                     blob.retrieve();                // retrieve a part
                }
              }
           }
      }
    }
    catch (DKException exc)
    {
       System.out.println(" Exception name " + exc.name());
       System.out.println(" Exception message " + exc.getMessage());
       System.out.println("Exception error code " + exc.errorCode());
       System.out.println("Exception error state " + exc.errorState());
       exc.printStackTrace();
    }
    catch (Exception exc)
    {
     System.out.println("Exception message " + exc.getMessage());
     exc.printStackTrace();
    }
  }
}
