/*BEGINPROLOGUE************************************************************************
 * @copyright(disclaimer)                                                             *
 * Licensed Materials - Property of IBM                                               *
 * IBM Enterprise Information Portal for Multiplatforms V8.1 (program number 5724-B43)*
 * (c ) Copyright IBM Corp. 1998,  2002. All Rights Reserved.                         *
 *                                                                                    *
 * US Government Users Restricted Rights                                              *
 * Use, duplication or disclosure restricted by GSA ADP Schedule                      *
 * Contract with IBM Corp.                                                            *
 *                                                                                    *
 * DISCLAIMER OF WARRANTIES :                                                         *
 *                                                                                    *
 * Permission is granted to copy and modify this Sample code, and to                  *
 * distribute modified versions provided that both the copyright                      *
 * notice,- and this permission notice and warranty disclaimer appear in all copies   * 
 * and modified versions.                                                             *
 *                                                                                    *
 * THIS SAMPLE CODE IS LICENSED TO YOU AS-IS.                                         *
 * IBM AND ITS SUPPLIERS AND LICENSORS DISCLAIM ALL WARRANTIES, EITHER EXPRESS OR     *
 * IMPLIED, IN SUCH SAMPLE CODE, INCLUDING THE WARRANTY OF NON-INFRINGEMENT AND THE   *
 * IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE. IN NO   *
 * EVENT WILL IBM OR ITS LICENSORS OR SUPPLIERS BE LIABLE FOR ANY DAMAGES ARISING OUT *
 * OF THE USE OF OR INABILITY TO USE THE SAMPLE CODE, DISTRIBUTION OF THE SAMPLE CODE,*
 * OR COMBINATION OF THE SAMPLE CODE WITH ANY OTHER CODE. IN NO EVENT SHALL IBM OR ITS*
 * LICENSORS AND SUPPLIERS BE LIABLE FOR ANY LOST REVENUE, LOST PROFITS OR DATA, OR   *
 * FOR DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL,INCIDENTAL OR PUNITIVE DAMAGES,       *
 * HOWEVER CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY,-, EVEN IF IBM OR ITS     *
 * LICENSORS OR SUPPLIERS HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.       *
 *                                                                                    *
 * @endCopyright                                                                      *
 **************************************************************************************/

/*
 * Filename       : TCallbackExecuteDL.java
 * Description    : Connect to DL Datastore.
 *                  Create query string.
 *                  Specify to retrieve folder and documents.
 *                  Specify to retrieve a max result of 20 itemss.
 *                  Get the result in a resultset cursor.
 *                  Loop thru results.
 *                  Display documents found and their parts and attributes,
 *                   if folders found display them and their documents/
 *                   folders.
 *                  Disconnect from DL Datastore.
 */

import com.ibm.mm.sdk.common.*;
import com.ibm.mm.sdk.server.*;
import java.io.*;
import java.lang.*;
import java.util.*;

// TCallbackExecuteDL class
public class TCallbackExecuteDL implements DKConstant
{

  // Main method
  public static void main(String argv[])
  {
      // Indicate this is a Console Application on Windows NT/95 for error
      // message display in dos console code page. If this was a Windows
      // GUI (Graphic User Interface) Application this would not be necessary
      // because DK_CM_SS_WINDOWS is the default.
      DKEnvironment.setSubSystem(DK_CM_SS_CONSOLE);

      //-------------------------------------------------------------- 
      // Checking for input parameters
      //-------------------------------------------------------------- 
      if (argv.length < 3){
          System.out.println("Invoke Syntax: " );
          System.out.println("  java TCallbackExecuteDL <libSrv> <userid> <pw>" );
          return;
      }
      
      String libSrv = argv[0];
      String userid = argv[1];
      String pw     = argv[2];
       
    
      DKDatastoreDL dsDL = null;
      try {
         dsDL = new DKDatastoreDL();
         dkResultSetCursor pCur = null;
         DKNVPair parms[] = null;
         TCallbackDL cbkDL = new TCallbackDL();
         InputStreamReader isreader = new InputStreamReader(System.in);
         
         System.out.println("connecting to datastore"); 
         dsDL.connect(libSrv,userid,pw,"");
         System.out.println("datastore connected libSrv " + libSrv + " userid " + userid ); 

         String cmd = "SEARCH=(INDEX_CLASS=DLSAMPLE," +
                          "MAX_RESULTS=20," +
                          "COND=(DLSEARCH_DocType <> NULL));" +
                          "OPTION=(CONTENT=YES;"       +
                          "TYPE_QUERY=DYNAMIC;" +
                          "TYPE_FILTER=FOLDERDOC)";
         System.out.println("query string " + cmd);

         System.out.println("executing query");
         
         dsDL.executeWithCallback(cmd,DK_CM_PARAMETRIC_QL_TYPE,parms,(dkCallback)cbkDL);
         System.out.println("datastore executed query");
         boolean isDone = false;
         DKException ckExc = null;
         while (isDone == false){
               System.out.println("press <enter> to continue...");
               isreader.read();
               isDone = cbkDL.isDone();
         }

    	 ckExc = cbkDL.getException();
    	 if (ckExc != null)
    	 {
    	  System.out.println("Exception name " + ckExc.name());
    	  System.out.println("Exception message " + ckExc.getMessage());
    	  System.out.println("Exception error code " + ckExc.errorCode());
    	  System.out.println("Exception error state " + ckExc.errorState());
    	  ckExc.printStackTrace();
    	  cbkDL.resetException(); // reset since it is not needed any more
    	 }    

         System.out.println("query results processed");
         
         dsDL.disconnect();
         System.out.println("datastore disconnected"); 
         dsDL.destroy();
      }
      catch (DKException exc){
     	 try {
     	     dsDL.destroy();
     	 }
     	 catch (Exception e)
     	 {
     	  e.printStackTrace();
     	 }      
         System.out.println("Exception name " + exc.name());
         System.out.println("Exception message " + exc.getMessage());
         System.out.println("Exception error code " + exc.errorCode());
         System.out.println("Exception error state " + exc.errorState());
         exc.printStackTrace();
      }
      catch (Exception exc){
     	 try {
     	     dsDL.destroy();
     	 }
     	 catch (Exception e)
     	 {
     	  e.printStackTrace();
     	 }      
         System.out.println("Exception message " + exc.getMessage());
         exc.printStackTrace();
      }

  }
}

