/*BEGINPROLOGUE************************************************************************
 * @copyright(disclaimer)                                                             *
 * Licensed Materials - Property of IBM                                               *
 * IBM Enterprise Information Portal for Multiplatforms V8.1 (program number 5724-B43)*
 * (c ) Copyright IBM Corp. 1998,  2002. All Rights Reserved.                         *
 *                                                                                    *
 * US Government Users Restricted Rights                                              *
 * Use, duplication or disclosure restricted by GSA ADP Schedule                      *
 * Contract with IBM Corp.                                                            *
 *                                                                                    *
 * DISCLAIMER OF WARRANTIES :                                                         *
 *                                                                                    *
 * Permission is granted to copy and modify this Sample code, and to                  *
 * distribute modified versions provided that both the copyright                      *
 * notice,- and this permission notice and warranty disclaimer appear in all copies   * 
 * and modified versions.                                                             *
 *                                                                                    *
 * THIS SAMPLE CODE IS LICENSED TO YOU AS-IS.                                         *
 * IBM AND ITS SUPPLIERS AND LICENSORS DISCLAIM ALL WARRANTIES, EITHER EXPRESS OR     *
 * IMPLIED, IN SUCH SAMPLE CODE, INCLUDING THE WARRANTY OF NON-INFRINGEMENT AND THE   *
 * IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE. IN NO   *
 * EVENT WILL IBM OR ITS LICENSORS OR SUPPLIERS BE LIABLE FOR ANY DAMAGES ARISING OUT *
 * OF THE USE OF OR INABILITY TO USE THE SAMPLE CODE, DISTRIBUTION OF THE SAMPLE CODE,*
 * OR COMBINATION OF THE SAMPLE CODE WITH ANY OTHER CODE. IN NO EVENT SHALL IBM OR ITS*
 * LICENSORS AND SUPPLIERS BE LIABLE FOR ANY LOST REVENUE, LOST PROFITS OR DATA, OR   *
 * FOR DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL,INCIDENTAL OR PUNITIVE DAMAGES,       *
 * HOWEVER CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY,-, EVEN IF IBM OR ITS     *
 * LICENSORS OR SUPPLIERS HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.       *
 *                                                                                    *
 * @endCopyright                                                                      *
 **************************************************************************************/

/*
 * Filename       : TXmlExportDL.java
 * Description    : Test XML import then export from DKDDO
 *                 
 * Setup : 
 *   1. Make sure that the XML data format is created using Content Manager
 *      System Admin with the data format name XML and the mime type text/xml
 *   2. The xml file must be updated to the proper entity name and datastore
 *      name.
 * 
 * Examples of how to run :
 *  1. Using File
 *    a) NT     
 *      java TXmlExportDL -d LIBSRVRN -n frnadmin 
 *         -p password -o . -v -f .\dlsamp01.xml 
 *    b) AIX     
 *      java TXmlExportDL -d LIBSRVRX -n frnadmin 
 *         -p password -o . -v -f ./dlsamp01.xml
 *
 *  2. Using URL
 *    a) NT     
 *      java TXmlExportDL -d LIBSRVRN -n frnadmin 
 *         -p password -o . -v -u file:////d://myxml//dlsamp01.xml 
 *    b) AIX     
 *      java TXmlExportDL -d LIBSRVRX -n frnadmin 
 *         -p password -o . -v -u file://////home//clt//myxml//dlsamp01.xml 
 *
 *  3. Using STDIN
 *    a) NT     
 *    type dlsamp01.xml | java TXmlExportDL -d LIBSRVRN -n frnadmin 
 *         -p password -o . -v -s 
 *    b) AIX     
 *    cat dlsamp01.xml | java TXmlExportDL -d LIBSRVRX -n frnadmin 
 *         -p password -o . -v -s 
 *
 *  4. Using File to internal buffer
 *    a) NT     
 *      java TXmlExportDL -d LIBSRVRN -n frnadmin 
 *         -p password -o . -v -b .\dlsamp01.xml 
 *    b) AIX     
 *      java TXmlExportDL -d LIBSRVRX -n frnadmin 
 *         -p password -o . -v -b ./dlsamp01.xml
 *
 */

import com.ibm.mm.sdk.common.*;
import com.ibm.mm.sdk.server.*;

import java.io.*;

public class TXmlExportDL implements DKConstantDL
{
   public static void main(String argv[])
      throws DKException, Exception
   {
      if ( argv.length == 0 ) {
         printUsage();
         System.exit(1);
      }

      String dsname = "";
      String userid = "";
      String passwd = "";
      String outputPath = "";
      
      String arg = "";
      String resource = "";

      int importOptions = 0;

      DKNVPair xmlSource = null;
      dkDatastore ds = null;
      
      try {
         for (int i = 0; i < argv.length; i++) {
            arg = argv[i];
            if (arg.startsWith("-")) {
               if (arg.equals("-h")) {
                  printUsage();
                  System.exit(1);
               } else if (arg.equals("-d")) {
                  if (i == argv.length - 1) {
                     System.err.println("error: missing a datastore name");
                     System.exit(1);
                  }

                  dsname = argv[++i];
                  System.out.println("datastore name: " + "'" + dsname + "'");
               } else if (arg.equals("-n")) {
                  if (i == argv.length - 1) {
                     System.err.println("error: missing a user id");
                     System.exit(1);
                  }

                  userid = argv[++i];
                  System.out.println("user id: " + "'" + userid + "'");
               } else if (arg.equals("-p")) {
                  if (i == argv.length - 1) {
                     System.err.println("error: missing a password");
                     System.exit(1);
                  }

                  passwd = argv[++i];
                  System.out.println("password: " + "'" + passwd + "'");
               } else if (arg.equals("-o")) {
                  if (i == argv.length - 1) {
                     System.err.println("error: missing an output path");
                     System.exit(1);
                  }

                  outputPath = argv[++i];
                  System.out.println("output path: " + "'" + outputPath + "'");
               } else if (arg.equals("-v")) {
                  importOptions = DK_CM_XML_VALIDATION;
               } else if (arg.equals("-f")) {
                  if (i == argv.length - 1) {
                     System.err.println("error: missing a file name");
                     System.exit(1);
                  }

                  String xmlFile = argv[++i];
                  resource = "*** input from FILE ***" + " - " + "'" + xmlFile + "'";
                  xmlSource = new DKNVPair("FILE", xmlFile);                  
               } else if (arg.equals("-b")) {
                  if (i == argv.length - 1) {
                     System.err.println("error: missing a file name for its content to be stored in a buffer");
                     System.exit(1);
                  }

                  String xmlFile = argv[++i];
                  resource = "*** input from FILE for its content to be stored in a buffer ***" + " - " + "'" + xmlFile + "'";
                
                  File file = new File(xmlFile);
                  int fileSize = (int) file.length();
                  byte[] data = new byte[fileSize];
                  DataInputStream dis = new DataInputStream(new FileInputStream(file));
                  dis.readFully(data);
                  resource += "\n" + "*** input from BUFFER ***";                                          
                  //xmlSource = new DKNVPair("BUFFER", data);
                  String strBuffer = new String(data);
                  xmlSource = new DKNVPair("BUFFER", strBuffer);
               } else if (arg.equals("-u")) {
                  if (i == argv.length - 1) {
                     System.err.println("error: missing an URL");
                     System.exit(1);
                  }

                  String urlSource = argv[++i];
                  resource = "*** input from URL ***" + " - " + "'" + urlSource + "'";                  
                  xmlSource = new DKNVPair("URL", urlSource);
               } else if (arg.equals("-s")) {
                  resource = "*** input from STDIN ***";
                  xmlSource = new DKNVPair("STDIN", null);
               }                              
            }
         }
         
         if (dsname.length() == 0) {
            System.err.println("error: missing a datastore name");
            System.exit(1);                
         }
         if (userid.length() == 0) {
            System.err.println("error: missing a user id");
            System.exit(1);                
         }
         if (passwd.length() == 0) {
            System.err.println("error: missing a password");
            System.exit(1);                
         }
         if (outputPath.length() == 0) {
            System.err.println("error: missing an output path");
            System.exit(1);                
         }
         if (xmlSource == null) {
            System.err.println("error: missing XML resource");
            System.exit(1);                           
         }
          
         System.out.println(resource);        
            
         if ((importOptions & DK_CM_XML_VALIDATION) == DK_CM_XML_VALIDATION) {
            System.out.println("Validation is ON");
         } else {
            System.out.println("Validation is OFF");            
         }
         
         DKDDO ddo = (DKDDO)DKDDO.fromXML(xmlSource, importOptions);
         System.out.println("*** ddo instantiated ***");
     
         ds = ddo.getDatastore();            
         if (ds == null) {
            ds = new DKDatastoreDL();
            ds.connect(dsname, userid, passwd, "");
            System.out.println("*** connected to datastore ***"); 
            ddo.setDatastore(ds);               
         } else {
            if (!ds.isConnected()) {
               ds.connect(dsname, userid, passwd, "");
               System.out.println("*** connected to datastore ***");                 
            }
         }
                        
         addDDO(ddo);
            
         // retrieve it back
         DKPid pid = new DKPid();            
         pid = ddo.getPidObject();
         DKDDO nddo = new DKDDO(ds, pid);
         System.out.println("");
         System.out.println("=====> Now retrieve it from the datastore <=====");
            
         retrieveDDO(nddo);            
         DKNVPair xmlDestination = null;
            
         //+++++ for STDOUT +++++
         //xmlDestination = new DKNVPair("STDOUT", null);                        
            
         //+++++ for BUFFER +++++
         //xmlDestination = new DKNVPair("BUFFER", new Object());
            
         //+++++ for FILE   +++++
         String xmlFile = "export.xml";
         String strOS = System.getProperty("os.name");
         if (strOS.indexOf("Win") != -1) { // Windows OS
            xmlFile = outputPath + "\\" + xmlFile;
         } else { // other than Win OS
            xmlFile = outputPath + "/" + xmlFile;            
         }
         
	  xmlDestination = new DKNVPair("FILE", xmlFile);
	 //       xmlDestination = new DKNVPair("STDOUT", null);
	  //String buffer = "";
	 //xmlDestination = new DKNVPair("BUFFER", buffer);
            
	  nddo.toXML(xmlDestination, outputPath, DK_CM_XDO_REFERENCE);
	  //	         nddo.toXML(xmlDestination, outputPath, DK_CM_XDO_EMBED);
	 //buffer = (String)xmlDestination.getValue();
         //System.out.println("buffer: " + buffer);
         //((DKDatastoreDL) ds).invokeSearchEngine("SM", "TM-TMINDEX");

         ds.disconnect();
         System.out.println("*** disconnected datastore ***");
         ds.destroy();

         //+++++ for BUFFER +++++
         //System.out.println("----- To display buffer -----");
         //System.out.println((String) xmlDestination.getValue());            

      } catch (DKException exc) {
         exc.printStackTrace();
         if (ds.isConnected()) {
            ds.disconnect();
            System.out.println("*** disconnected datastore ***");
         }
         ds.destroy();
         System.exit(1);
      } catch (Exception exc) {
         exc.printStackTrace();
         if (ds.isConnected()) {
            ds.disconnect();
            System.out.println("*** disconnected datastore ***");
         }
         ds.destroy();
         System.exit(1);
      } 
   }
   
   private static void addDDO(DKDDO ddo)
      throws DKException, Exception
   {
      short i, j;
      String dataName;
      String propertyCount;
      short propertyCnt;
      Object value;
      
      System.out.println(" data count = " + ddo.dataCount());
      short itemType = ((Short)ddo.getPropertyByName(DK_CM_PROPERTY_ITEM_TYPE)).shortValue();
      if (itemType == DK_CM_FOLDER) {
         System.out.println(" this item is a Folder, itemId = " + ddo.getPidObject().getPrimaryId());
      } else if (itemType == DK_CM_DOCUMENT) {
         System.out.println(" this item is a Document, itemId = " + ddo.getPidObject().getPrimaryId());
      } else {
         System.out.println(" this item is of unknown type, type = " + itemType + ", itemId = " + ddo.getPidObject().getPrimaryId());
      }
            
      // print the properties of this DDO
      propertyCnt = ddo.propertyCount();
      propertyCount = (new Short(propertyCnt)).toString();
      System.out.println(" properties of this DDO: " + propertyCount);
      for (i = 1; i <= propertyCnt; i++) {
         System.out.println("   property_name(" + i + ") = " + ddo.getPropertyName(i) +
                            ", value = " + ddo.getProperty(i));
      }
         
      DKParts parts;
      DKBlobDL blob;
      dkIterator iter;
            
      for (i=1; i <= ddo.dataCount(); i++) {
         dataName = ddo.getDataName(i);
         value = ddo.getData(i);
         if (value != null) {
            System.out.print(" data item name (" + i + ") = " + dataName + " ");            
            System.out.println("  \t value (" + i + ") = " + value);
         } else {
            System.out.print(" data item name (" + i + ") = " + dataName + " ");            
            System.out.println("  \t value (" + i + ") = null" );
            //continue;            
         }
         
         System.out.println(" properties of this data item: " + ddo.dataPropertyCount(i));
         
         for (j=1; j <= ddo.dataPropertyCount(i); j++) {
            System.out.println("   property_name(" + i + "," + j + ") = " + ddo.getDataPropertyName(i,j) +
                               ", value = " + ddo.getDataProperty(i,j));
         }
         
         if (dataName.equals(DKFOLDER)) {
            if (value != null) {
               addFolderContent((dkCollection)value, ddo.getDatastore());
            }
         }
         
         if (dataName.equals(DKPARTS)) {
            if (value != null) {                         
               parts = (DKParts) value;
               iter  = parts.createIterator();
               while (iter.more()) {
                  blob = (DKBlobDL) iter.next();   
                  System.out.println(" \t\t blob part id " + ((DKPidXDODL) (blob.getPidObject())).getPartId() +
                                     " content class = " + blob.getContentClass() +
                                     //" content type = " + blob.getContentType() +
                                     //" mime type = " + blob.getMimeType() +
                                     " rep type " + blob.getRepType() +
                                     " aff type " + blob.getAffiliatedType() );
                                     
                  DKSearchEngineInfoDL srchEx = (DKSearchEngineInfoDL) blob.getExtension("DKSearchEngineInfoDL");
                  if (srchEx != null) {
                     String searchEngine = srchEx.getSearchEngine();
                     String searchIndex  = srchEx.getSearchIndex();
                     String searchInfo   = srchEx.getSearchInfo();
                     
                     System.out.println("searchEngine="+searchEngine+
                                        " searchIndex="+searchIndex+
                                        " searchInfo="+searchInfo);
                     
                     //if (searchIndex != null) {
                     //   blob.setToBeIndexed();   
                     //}
                  }
 
                  DKStorageManageInfoDL smi = (DKStorageManageInfoDL) blob.getExtension("DKStorageManageInfoDL");
                  if (smi != null) {
                     String smsRetention    = (new Integer(smi.getRetention())).toString();
                     String smsCollection   = smi.getCollectionName();
                     String smsMgmtClass    = smi.getManagementClass();
                     String smsStorageClass = smi.getStorageClass();
                     String smsObjServer    = smi.getStoreSite();
                     
                     System.out.println("smsRetention="+smsRetention+
                                        " smsCollection="+smsCollection+
                                        " smsMgmtClass="+smsMgmtClass+
                                        " smsStorageClass="+smsStorageClass+
                                        " smsObjServer="+smsObjServer);
                  }
                  
               }
            }
         }
      }
            
      System.out.println((ddo.getDatastore().isConnected()) ? " ddo connected " : " ddo not connected yet ");      
      System.out.println(" ddo datastore name=" + ddo.getDatastore().datastoreName());
      ddo.add();
      System.out.println(" added ddo to datastore...");
   }
  
   private static void addFolderContent(dkCollection col, dkDatastore ds)
      throws DKException, Exception
   {
      dkIterator iter = col.createIterator();
      DKDDO ddo;
      int n = 0;
      String pidString;
      
      while (iter.more()) {
         ddo = (DKDDO) iter.next();
         pidString = ddo.getPidObject().pidString();
         System.out.println(" --- folder member [" + ++n + "] = " + pidString);
         
         if (ddo.getDatastore() == null) {
            System.out.println(" datastore not set yet...");
            ddo.setDatastore(ds);
            System.out.println(" set datastore! ");
         }
         
         addDDO(ddo);
      }
   }

   private static void retrieveDDO(DKDDO ddo)
      throws DKException, Exception
   {
      DKParts parts;
      DKBlobDL blob;
      dkIterator iter;
      String dataName;
      Object value;      
      
      ddo.retrieve();
      
      for (short i=1; i<=ddo.dataCount(); i++) {
         dataName = ddo.getDataName(i);
         value = ddo.getData(i);
                  
         if (dataName.equals(DKFOLDER)) {
            if (value != null) {
               retrieveFolderContent((dkCollection) value);
            }
         } else if (dataName.equals(DKPARTS)) {
            if (value != null) {                         
               parts = (DKParts) value;
               iter  = parts.createIterator();
               while (iter.more()) {
                  blob = (DKBlobDL) iter.next();                                       
                  blob.retrieve();
               }
            }
         } else {
            
         }         
      }      
   }
  
   private static void retrieveFolderContent(dkCollection col)
      throws DKException, Exception
   {
      dkIterator iter = col.createIterator();
      DKDDO ddo;
      
      while (iter.more()) {
         ddo = (DKDDO) iter.next();
         // ddo.retrieve();

         //if (ddo != null) {
            retrieveDDO(ddo);
         //}
      }
   }
   
   private static void printUsage()
   {
      System.err.println();
      System.err.println("usage: java TXmlExportDL -d <datastore name> -n <user id> -p <password> -o <output path> <XML input> ");
      System.err.println();
      System.err.println("  <XML input> - one of the following: ");
      System.err.println("    -v  ");
      System.err.println("        If specified, then the XML Parser would perform the validation.");      
      System.err.println();
      System.err.println("    -f  <an XML fullpath file name>");
      System.err.println();
      System.err.println("    -b  <an XML fullpath file name>");
      System.err.println("        Note: To be read in to an internal buffer as input");      
      System.err.println();
      System.err.println("    -u  <the URL of an XML file>");
      System.err.println();
      System.err.println("    -s  ");
      System.err.println("        Note: To read an XML file content through STDIN");      
   }

} 
