/*BEGINPROLOGUE************************************************************************
 * @copyright(disclaimer)                                                             *
 * Licensed Materials - Property of IBM                                               *
 * IBM Enterprise Information Portal for Multiplatforms V8.1 (program number 5724-B43)*
 * (c ) Copyright IBM Corp. 1998,  2002. All Rights Reserved.                         *
 *                                                                                    *
 * US Government Users Restricted Rights                                              *
 * Use, duplication or disclosure restricted by GSA ADP Schedule                      *
 * Contract with IBM Corp.                                                            *
 *                                                                                    *
 * DISCLAIMER OF WARRANTIES :                                                         *
 *                                                                                    *
 * Permission is granted to copy and modify this Sample code, and to                  *
 * distribute modified versions provided that both the copyright                      *
 * notice,- and this permission notice and warranty disclaimer appear in all copies   * 
 * and modified versions.                                                             *
 *                                                                                    *
 * THIS SAMPLE CODE IS LICENSED TO YOU AS-IS.                                         *
 * IBM AND ITS SUPPLIERS AND LICENSORS DISCLAIM ALL WARRANTIES, EITHER EXPRESS OR     *
 * IMPLIED, IN SUCH SAMPLE CODE, INCLUDING THE WARRANTY OF NON-INFRINGEMENT AND THE   *
 * IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE. IN NO   *
 * EVENT WILL IBM OR ITS LICENSORS OR SUPPLIERS BE LIABLE FOR ANY DAMAGES ARISING OUT *
 * OF THE USE OF OR INABILITY TO USE THE SAMPLE CODE, DISTRIBUTION OF THE SAMPLE CODE,*
 * OR COMBINATION OF THE SAMPLE CODE WITH ANY OTHER CODE. IN NO EVENT SHALL IBM OR ITS*
 * LICENSORS AND SUPPLIERS BE LIABLE FOR ANY LOST REVENUE, LOST PROFITS OR DATA, OR   *
 * FOR DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL,INCIDENTAL OR PUNITIVE DAMAGES,       *
 * HOWEVER CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY,-, EVEN IF IBM OR ITS     *
 * LICENSORS OR SUPPLIERS HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.       *
 *                                                                                    *
 * @endCopyright                                                                      *
 **************************************************************************************/

/*
 * Filename       : TxdosAddFsmsDL.java
 * Description    : Test DLBlobDL for add object with specified storage collection
 *                  (add from file to be search indexed)
 * setup:  1. bring up SMS server
 *         2. bring up System Admin Client
 *         3. create migration policy(managementClass) MEIMGT1 with
 *            retension period = 30 days & storage calss = FIXED
 *         4. create workstation collection CBR.MEICOLLECT1 with
 *            migration policy = MEIMGT1 & storage group = GROUP01
 *         5. pick a known itemId from sample loader's output log
 *
 * hard code: need to modify library server/userid/password for dsDL.connect 
 *            need to modify search engine informations for yours                             
 * invoke: java TxdosAddFsmsDL <fileName to be added> <partId> <itemId>
 *
 */
// replace 'import com.ibm.mm.sdk.server.*' with 'import com.ibm.mm.sdk.client.*' 
// for client testing
import com.ibm.mm.sdk.server.*;
import com.ibm.mm.sdk.common.*;
import com.ibm.mm.sdk.internal.*;
import java.io.*;
import java.lang.*;

public class TxdosAddFsmsDL
       implements DKConstantDL 
{
  // Main method
  public static void main(String[] args)
  {
    // Indicate this is a Console Application on Windows NT/95 for error
    // message display in dos console code page. If this was a Windows
    // GUI (Graphic User Interface) Application this would not be necessary
    // because DK_CM_SS_WINDOWS is the default.
    DKEnvironment.setSubSystem(DK_CM_SS_CONSOLE);

    String fileName = "e:\\dlv200j\\test\\notepart.txt";
    int    partId = 0;   
    String itemId = "N$$MYMYDN0NKHSN5";

    if (args.length == 3)
    {
      fileName = args[0];
      partId = (int)Integer.parseInt(args[1], 10);
      itemId = args[2];
      System.out.println("You enter: java TxdosAddFsmsDL " +
      fileName + " " + partId + " " + itemId);
    }
    if (args.length == 2)
    {
      fileName = args[0];
      partId =(int)Integer.parseInt(args[1], 10);
      System.out.println("You enter: java TxdosAddFsmsDL " +
      fileName + " " + partId );
      System.out.println("The supplied default itemId = " + itemId);
    }
    if (args.length == 1)
    {
      fileName = args[0];
      System.out.println("You enter: java TxdosAddFsmsDL " + fileName);
      System.out.println("The supplied default partId = " + partId);
      System.out.println("The supplied default itemId = " + itemId);
    }
    if (args.length == 0)
    {
      System.out.println("invoke: java TxdosAddFsmsDL <fileName> <partId> <itemId>");
      System.out.println("No parameter, following defaults will be provided:");
      System.out.println("     default fileName = " + fileName);
      System.out.println("     default partId = " + partId);
      System.out.println("     default itemId = " + itemId);
    }
    DKDatastoreDL dsDL = null;
    try {
    dsDL = new DKDatastoreDL();
    System.out.println("connecting to datastore"); 
    dsDL.connect("LIBSRVRP","FRNADMIN","PASSWORD","");
    System.out.println("datastore connected"); 
    
    DKBlobDL axdo = new DKBlobDL(dsDL);
    DKPidXDODL  apid = new DKPidXDODL();
    apid.setPartId(partId);
    apid.setItemId(itemId);
    axdo.setPidObject(apid);
    axdo.setContentClass(DK_DL_CC_ASCII);
    System.out.println("itemid=" + apid.getItemId());
    System.out.println("partId=" + apid.getPartId()); 
    System.out.println("xdo:repType=" + axdo.getRepType()); 
    System.out.println("pid:repType= " 
     + ((DKPidXDODL) (axdo.getPidObject())).getRepType());
    System.out.println("contentClass=" + axdo.getContentClass());

    System.out.println("about to set searchEngine... "); 
    //---set searchEngine -----
    DKSearchEngineInfoDL aSrchEx = new DKSearchEngineInfoDL();
    aSrchEx.setSearchEngine("SM");
    aSrchEx.setSearchIndex("TM-TMINDEX");
    aSrchEx.setSearchInfo("ENU");
    axdo.setExtension("DKSearchEngineInfoDL", (dkExtension)aSrchEx);
 
    System.out.println("about to set DKStorageManageInfoDL... "); 
    //---set DKStorageManageInfoDL-----
    DKStorageManageInfoDL aSMS = new DKStorageManageInfoDL();
    aSMS.setRetention(888);
    aSMS.setCollectionName("CBR.MEICOLLECT1");
    aSMS.setManagementClass("MEIMGT1");
    aSMS.setStorageClass("FIXED");
    axdo.setExtension("DKStorageManageInfoDL", (dkExtension)aSMS);
   
    System.out.println("about to call add(fileName)");
    axdo.add(fileName);
    System.out.println("add successfully.....");
    System.out.println("after add partId = " + axdo.getPartId());

    dsDL.disconnect();
    System.out.println("datastore disconnected"); 
    dsDL.destroy();
    }
    catch (DKException exc)
    {
     try {
         dsDL.destroy();
     }
     catch (Exception e)
     {
      e.printStackTrace();
     }        
     System.out.println("Exception name " + exc.name());
     System.out.println("Exception message " + exc.getMessage());
     exc.printStackTrace();
    }
    catch (Exception exc)
    {
     try {
         dsDL.destroy();
     }
     catch (Exception e)
     {
      e.printStackTrace();
     }        
     exc.printStackTrace();
    }
  }
}
