/*BEGINPROLOGUE************************************************************************
 * @copyright(disclaimer)                                                             *
 * Licensed Materials - Property of IBM                                               *
 * IBM Enterprise Information Portal for Multiplatforms V8.1 (program number 5724-B43)*
 * (c ) Copyright IBM Corp. 1998,  2002. All Rights Reserved.                         *
 *                                                                                    *
 * US Government Users Restricted Rights                                              *
 * Use, duplication or disclosure restricted by GSA ADP Schedule                      *
 * Contract with IBM Corp.                                                            *
 *                                                                                    *
 * DISCLAIMER OF WARRANTIES :                                                         *
 *                                                                                    *
 * Permission is granted to copy and modify this Sample code, and to                  *
 * distribute modified versions provided that both the copyright                      *
 * notice,- and this permission notice and warranty disclaimer appear in all copies   * 
 * and modified versions.                                                             *
 *                                                                                    *
 * THIS SAMPLE CODE IS LICENSED TO YOU AS-IS.                                         *
 * IBM AND ITS SUPPLIERS AND LICENSORS DISCLAIM ALL WARRANTIES, EITHER EXPRESS OR     *
 * IMPLIED, IN SUCH SAMPLE CODE, INCLUDING THE WARRANTY OF NON-INFRINGEMENT AND THE   *
 * IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE. IN NO   *
 * EVENT WILL IBM OR ITS LICENSORS OR SUPPLIERS BE LIABLE FOR ANY DAMAGES ARISING OUT *
 * OF THE USE OF OR INABILITY TO USE THE SAMPLE CODE, DISTRIBUTION OF THE SAMPLE CODE,*
 * OR COMBINATION OF THE SAMPLE CODE WITH ANY OTHER CODE. IN NO EVENT SHALL IBM OR ITS*
 * LICENSORS AND SUPPLIERS BE LIABLE FOR ANY LOST REVENUE, LOST PROFITS OR DATA, OR   *
 * FOR DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL,INCIDENTAL OR PUNITIVE DAMAGES,       *
 * HOWEVER CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY,-, EVEN IF IBM OR ITS     *
 * LICENSORS OR SUPPLIERS HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.       *
 *                                                                                    *
 * @endCopyright                                                                      *
 **************************************************************************************/


/*
 * Filename       : TExecPSTFed.java
 * Description    : Retrieve search template from database and execute a query.
 *                : Need to run the following program first
 *                 java TADLServersFed       <cmbdb> <userid> <pw> add
 *                 java TADLsmFed      <cmbdb> <userid> <pw> add
 *                 java TADLstFed      <cmbdb> <userid> <pw> add
 *
 *
 */

import com.ibm.mm.sdk.server.*;
import com.ibm.mm.sdk.common.*;
import java.io.*;

public class TExecPSTFed implements DKConstantFed 
{

   // Main method
   public static void main(String argv[]){

      // Indicate this is a Console Application on Windows NT/95 for error
      // message display in dos console code page. If this was a Windows
      // GUI (Graphic User Interface) Application this would not be necessary
      // because DK_CM_SS_WINDOWS is the default.
      DKEnvironment.setSubSystem(DK_CM_SS_CONSOLE);
        
      //-------------------------------------------------------------- 
      // Checking for input parameters
      //-------------------------------------------------------------- 
      if (argv.length < 3) {
         System.out.println("Usage Syntax: ");
         System.out.println("\t java TExecPSTFed <cmbdb> <userid> <pw>" );
         return;
      }
      
      String libSrv = argv[0];
      String userid = argv[1];
      String pw     = argv[2];
        
        
      try {
         // create federated datastore
         System.out.println(" *** connecting to datastore : " + libSrv);
         DKDatastoreFed ds = new DKDatastoreFed();
         ds.connect(libSrv, userid, pw, "");
         System.out.println(" *** datastore connected *** "); 

         // set up search criteria memory variable.
         short[] ops = { DKOpCode.EQ, DKOpCode.NEQ, DKOpCode.LIKE, DKOpCode.NOTLIKE, DKOpCode.IN, DKOpCode.NOTIN };

         DKNVPair opt[] = new DKNVPair[1];
         opt[0] = new DKNVPair("MAX_RESULTS", "6");

         //===================================================================================================
         // Retrieve and execute search templates from federated database
         //===================================================================================================
         dkCollection coll = ds.listSearchTemplates();
         DKSearchTemplateFed stObj;
         DKCQExpr cqe;
       
         boolean flag = false;
         Short op;
         String value;
         if (coll != null) {
            // Retrieve list of search template 
            dkIterator iter = coll.createIterator();
            while (iter.more()) {
                stObj = (DKSearchTemplateFed) iter.next();
                stObj.setOptionList(opt);
               
                System.out.println(" ");
                System.out.println("==============Retrieving Search template: " + stObj.getName());
                System.out.println("    Description: " + stObj.getDescription());
                
                dkCollection scColl = stObj.listCriteria();
                flag =false;
                if (scColl != null) {
                    dkIterator scIter = scColl.createIterator();
                    while (scIter.more()) {
                        DKSearchCriteria sc = (DKSearchCriteria) scIter.next();
                        System.out.println("       Criteria Name: " + sc.getName());
                        
                        if (sc.getName().equals("UserID")){
                            sc.setOperator(DKOpCode.LIKE);
                            sc.setValue("'Progr%'");            
                            stObj.update(sc);                   
                            flag = true;
                        }
                        else if (sc.getName().equals("Date")){
                            sc.setOperator(DKOpCode.NEQ);
                            sc.setValue("NULL");          
                            stObj.update(sc);                    
                            flag = true;
                        }
                        else if (sc.getName().equals("Title")){
                            sc.setOperator(DKOpCode.EQ);
                            sc.setValue("'BASUKI'");                  
                            stObj.update(sc);                    
                            flag = true;
                        }
                        else
                           ;
                    }
                }
                        
                if (flag){
                // Execute search template
                    cqe = stObj.createQueryExpr();
                    dkResultSetCursor rsc = ds.execute(cqe);

                     //  get result set cursor
                     if (rsc == null) 
                          System.out.println(" *** cursor is null");
                      else {
                          DKDDO ddo;

                          while (rsc.isValid()  && !(rsc.isEnd()) )  {
                              System.out.println("before fetch, position = " + rsc.getPosition() );
                              ddo = rsc.fetchNext();
                              System.out.println("after fetch, position = " + rsc.getPosition() );
    
                              if (ddo != null) {  
                                  System.out.println(" pidString : " + ddo.getPidObject().pidString());
                                  displayDDO(ddo);
                               } 
                           }
                        rsc.destroy();
                       }         
                 }
            }
         }
         ds.disconnect();
         System.out.println("Disconnect. Done.");
         ds.destroy();
      }    
      catch (DKException exc) {
         System.out.println("Exception name " + exc.name());
         System.out.println("Exception message " + exc.getMessage());
         exc.printStackTrace();
      }
      catch (Exception exc) {
         System.out.println("Exception message " + exc.getMessage());
         exc.printStackTrace();
      }

   }

   public static void displayDDO(DKDDO ddo) throws DKException, Exception
   {
      String   dataName;
      Object a = null;
 
      if (ddo != null) {
         for (short i = 1; i <= ddo.dataCount(); i++) {
            a = ddo.getData(i);
            dataName = ddo.getDataName(i);
            System.out.print(i + ": Attribute Name :" + dataName );
            if (a != null)
               System.out.println(",\t   Value :" + a.toString());    
            else
               System.out.println(",\t   Value : null ");    
         }  
      }
   }

}

