/*BEGINPROLOGUE************************************************************************
 * @copyright(disclaimer)                                                             *
 * Licensed Materials - Property of IBM                                               *
 * IBM Enterprise Information Portal for Multiplatforms V8.1 (program number 5724-B43)*
 * (c ) Copyright IBM Corp. 1998,  2002. All Rights Reserved.                         *
 *                                                                                    *
 * US Government Users Restricted Rights                                              *
 * Use, duplication or disclosure restricted by GSA ADP Schedule                      *
 * Contract with IBM Corp.                                                            *
 *                                                                                    *
 * DISCLAIMER OF WARRANTIES :                                                         *
 *                                                                                    *
 * Permission is granted to copy and modify this Sample code, and to                  *
 * distribute modified versions provided that both the copyright                      *
 * notice,- and this permission notice and warranty disclaimer appear in all copies   * 
 * and modified versions.                                                             *
 *                                                                                    *
 * THIS SAMPLE CODE IS LICENSED TO YOU AS-IS.                                         *
 * IBM AND ITS SUPPLIERS AND LICENSORS DISCLAIM ALL WARRANTIES, EITHER EXPRESS OR     *
 * IMPLIED, IN SUCH SAMPLE CODE, INCLUDING THE WARRANTY OF NON-INFRINGEMENT AND THE   *
 * IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE. IN NO   *
 * EVENT WILL IBM OR ITS LICENSORS OR SUPPLIERS BE LIABLE FOR ANY DAMAGES ARISING OUT *
 * OF THE USE OF OR INABILITY TO USE THE SAMPLE CODE, DISTRIBUTION OF THE SAMPLE CODE,*
 * OR COMBINATION OF THE SAMPLE CODE WITH ANY OTHER CODE. IN NO EVENT SHALL IBM OR ITS*
 * LICENSORS AND SUPPLIERS BE LIABLE FOR ANY LOST REVENUE, LOST PROFITS OR DATA, OR   *
 * FOR DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL,INCIDENTAL OR PUNITIVE DAMAGES,       *
 * HOWEVER CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY,-, EVEN IF IBM OR ITS     *
 * LICENSORS OR SUPPLIERS HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.       *
 *                                                                                    *
 * @endCopyright                                                                      *
 **************************************************************************************/

// sample connect/disconnect to Federated datastore

import com.ibm.mm.sdk.common.*;
import com.ibm.mm.sdk.server.*;
import java.io.*;

// TGetEntityAttrFed class
/**
 * <pre>
 * Filename       : TGetEntityAttrFed.java
 * Description    : test the following classes and funtions in each:
 *		     	- DKDatastoreFed: 
 *                          + connect
 *                          + disconnect
 *                          + listEntityNames
 *                      - DKDatastoreDefFed:
 *                          + getEntity
 *                      - DKEntityDefFed:
 *                          + listAttrNames
 *                          + getAttr
 *		 
 * </pre>
 */

public class TGetEntityAttrFed implements DKConstantFed {

  // Main method
  public static void main(String argv[]){
    
   // Indicate this is a Console Application on Windows NT/95 for error
   // message display in dos console code page. If this was a Windows
   // GUI (Graphic User Interface) Application this would not be necessary
   // because DK_CM_SS_WINDOWS is the default.
   DKEnvironment.setSubSystem(DK_CM_SS_CONSOLE);

    try {
	DKDatastoreFed dsFed = new DKDatastoreFed();
	DKDatastoreDefFed dsDef = (DKDatastoreDefFed)dsFed.datastoreDef();

	String userName;
    	String dsName;

    	System.out.println("connecting to datastore");

     	//String url = "jdbc:db2:cmbdb";
	//dsFed.connect(url, "cmbdb","ccadmin","password");
	dsFed.connect("cmbdb","ccadmin","password","");

	System.out.println("datastore connected"); 

	userName = dsFed.userName();
	dsName = dsFed.datastoreName();

	System.out.println("user " + userName + " dsName " + dsName);   
	
	String[] names = dsFed.listEntityNames();
	for (int i=0; i<names.length; i++) {
		System.out.println("name[" + i + "] = " + names[i] );

		dkEntityDef    entityFed   = dsDef.getEntity(names[i]);
		System.out.println("\tDescription= " + entityFed.getDescription());

		String[]       attrNames   = entityFed.listAttrNames();
		if (attrNames != null) {
			for (int j=0; j<attrNames.length; j++) {	
				System.out.println("\tattrName = " + attrNames[j]);

				dkAttrDef attr = entityFed.getAttr(attrNames[j]);
				System.out.println("\t\tentityName:   " + attr.getEntityName());
				System.out.println("\t\tdsName:       " + attr.datastoreName());
				System.out.println("\t\tdsType:       " + attr.datastoreType());
				System.out.println("\t\ttype:         " + attr.getType());
				System.out.println("\t\tdescription:  " + attr.getDescription());
				System.out.println("\t\tsize:         " + attr.getSize());
				System.out.println("\t\tid:           " + attr.getId());
				System.out.println("\t\tprecision:    " + attr.getPrecision());
				System.out.println("\t\tscale:        " + attr.getScale());
				System.out.println("\t\tmax:          " + attr.getMax());
				System.out.println("\t\tmin:          " + attr.getMin());
				System.out.println("\t\tstringType:   " + attr.getStringType());

				if (attr.isNullable())
					System.out.println("\t\tisNullable:   true");
				else 
					System.out.println("\t\tisNullable:   false");
	
				if (attr.isUpdatable())
					System.out.println("\t\tisUpdatable:  true");
				else 
					System.out.println("\t\tisUpdatable:  false");
			
				if (attr.isQueryable())
					System.out.println("\t\tisQueryable:  true");
				else 
					System.out.println("\t\tisQueryable:  false");
				System.out.println("\n");
			}
		}
	}


	dsFed.disconnect();
	System.out.println("datastore disconnected"); 
	userName = dsFed.userName();
	dsName = dsFed.datastoreName();
	System.out.println("user " + userName + " dsName " + dsName);   
    }
    catch (DKException exc)
    {
	System.out.println("Exception name " + exc.name());
     	System.out.println("Exception message " + exc.getMessage());
     	exc.printStackTrace();
    }
    catch (Exception exc)
    {
    	System.out.println("Exception message " + exc.getMessage());
     	exc.printStackTrace();
    }

  }
}

