/*BEGINPROLOGUE************************************************************************
 * @copyright(disclaimer)                                                             *
 * Licensed Materials - Property of IBM                                               *
 * IBM Information Integrator for Content V8.2  (program number 5724-B43)             *
 * (c ) Copyright IBM Corp. 1998, 2002, 2003. All Rights Reserved.                    *
 *                                                                                    *
 * US Government Users Restricted Rights                                              *
 * Use, duplication or disclosure restricted by GSA ADP Schedule                      *
 * Contract with IBM Corp.                                                            *
 *                                                                                    *
 * DISCLAIMER OF WARRANTIES :                                                         *
 *                                                                                    *
 * Permission is granted to copy and modify this Sample code, and to                  *
 * distribute modified versions provided that both the copyright                      *
 * notice,- and this permission notice and warranty disclaimer appear in all copies   * 
 * and modified versions.                                                             *
 *                                                                                    *
 * THIS SAMPLE CODE IS LICENSED TO YOU AS-IS.                                         *
 * IBM AND ITS SUPPLIERS AND LICENSORS DISCLAIM ALL WARRANTIES, EITHER EXPRESS OR     *
 * IMPLIED, IN SUCH SAMPLE CODE, INCLUDING THE WARRANTY OF NON-INFRINGEMENT AND THE   *
 * IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE. IN NO   *
 * EVENT WILL IBM OR ITS LICENSORS OR SUPPLIERS BE LIABLE FOR ANY DAMAGES ARISING OUT *
 * OF THE USE OF OR INABILITY TO USE THE SAMPLE CODE, DISTRIBUTION OF THE SAMPLE CODE,*
 * OR COMBINATION OF THE SAMPLE CODE WITH ANY OTHER CODE. IN NO EVENT SHALL IBM OR ITS*
 * LICENSORS AND SUPPLIERS BE LIABLE FOR ANY LOST REVENUE, LOST PROFITS OR DATA, OR   *
 * FOR DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL,INCIDENTAL OR PUNITIVE DAMAGES,       *
 * HOWEVER CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY,-, EVEN IF IBM OR ITS     *
 * LICENSORS OR SUPPLIERS HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.       *
 *                                                                                    *
 * @endCopyright                                                                      *
 **************************************************************************************

Deleting Resource Items
    Deleting Resource Items is exactly the same method as non-resource Item deletion,
    shown in SItemDeletionICM.  Please refer to SItemDeletionICM for complete deletion
    information.  As explained in the Resource Item Creation Sample, XDOs are DDOs.  
    Therefore all functionality of DDOs, including deletion, still applies.
    
    This sample will delete all items created by the Resource Item Samples.

********************************************************************/

// Imports
import com.ibm.mm.sdk.common.*;
import com.ibm.mm.sdk.server.*;
import java.io.*;

/************************************************************************************************
 *          FILENAME: SResourceItemDeletionICM.java
 *                    ---------------------------------------------------------------------------
 *       DESCRIPTION: Deleting Resource Items.
 *                    ---------------------------------------------------------------------------
 *     DEMONSTRATION: Deleting Resource Items
 *                    ---------------------------------------------------------------------------
 * COMMANDLINE USAGE: java SResourceItemDeletionICM <database> <userName> <password>
 *                    ---------------------------------------------------------------------------
 *      PREREQUISITS: The Data Model should be defined.  Sample will still complete successfully
 *                    because it meets the desired end results.  For a demonstration, Items
 *                    should be created first.  If desired please run the following Samples  
 *                         - SAttributeDefinitionCreationICM
 *                         - SAttributeGroupDefCreationICM
 *                         - SReferenceAttrDefCreationICM
 *                         - SItemTypeCreationICM
 *                    The Item Types must be set to use the specified default Resource Manager.
 *                    Plese run the following samples.
 *                         - SResourceMgrDefSetDefaultICM
 *                         - SSMSCollectionDefSetDefaultICM
 *                    NOTE:  Sample SSampleModelBuildICM pulls together all of the above.  It 
 *                           must be edited first for the RM Confiuration information.
 *                    ---------------------------------------------------------------------------
 *    FILES REQUIRED: SConnectDisconnectICM.java
 *                    SItemTypeRetrievalICM.java      (Needed by SItemDeletionICM)
 *                    SItemCreationICM.java           (Needed by SItemDeletionICM)
 *                    SItemDeletionICM.java
 *                    SResourceItemICM_Document1.doc
 *                    SResourceItemICM_Text1.txt
 *                    SResourceItemICM_Image1.bmp
 ************************************************************************************************/
public class SResourceItemDeletionICM{

    //-------------------------------------------------------------
    // Main
    //-------------------------------------------------------------
    /**
     * Run the Sample.
     * @param argv[] String Array containing arguments.  Optional arguments are <databse> <userName> <password>.
     */
    public static void main(String argv[]) throws DKException, Exception{
    
        // Defaults for connecting to the database.
        String database = SConnectDisconnectICM.DEFAULT_DATABASE;
        String userName = SConnectDisconnectICM.DEFAULT_USERNAME;
        String password = SConnectDisconnectICM.DEFAULT_PASSWORD;
        
        //------------------------------------------------------------
        // Checking for input parameters
        //--------------------------------------------------------------
        if (argv.length < 3) { // if not all 3 arguments were specified, use defaults and report correct usage.
            System.out.println("Usage: " );
            System.out.println("  java SResourceItemDeletionICM <database> <userName> <password>" );
            System.out.println("  *** Some parameters not specified, using defaults..." );
            System.out.println("");
        } else {  // otherwise enough parameters were specified, use the first 3 for the 3 parameters.
            if (argv.length > 0) database = argv[0];
            if (argv.length > 1) userName = argv[1];
            if (argv.length > 2) password = argv[2];
        }//end else
        String ver = SConnectDisconnectICM.EIP_VERSION;
        
        System.out.println("===========================================");
        System.out.println("IBM Information Integrator for Content v"+ver);
        System.out.println("Sample Program:  SResourceItemDeletionICM");
        System.out.println("-------------------------------------------");
        System.out.println(" Database: "+database);
        System.out.println(" UserName: "+userName);
        System.out.println("===========================================");
        
        try{
            //-------------------------------------------------------------
            // Connect to datastore
            //-------------------------------------------------------------
            // See Sample SConnectDisconnectICM for more information
            System.out.println("Connecting to datastore (Database '"+database+"', UserName '"+userName+"')...");

                DKDatastoreICM dsICM = new DKDatastoreICM();  // Create new datastore object.
                dsICM.connect(database,userName,password,""); // Connect to the datastore.

            System.out.println("Connected to datastore (Database '"+dsICM.datastoreName()+"', UserName '"+dsICM.userName()+"').");

            //-------------------------------------------------------------
            // Create Resource Object for Retrieval Demonstration
            //-------------------------------------------------------------
            // In order to demonstrate Updating of Resource Items, a few
            // Resource Items must first be created.
            System.out.println("Creating Resource Objects...");

                DKLobICM     lob = (DKLobICM)   dsICM.createDDO("S_lob",   DKConstant.DK_CM_DOCUMENT); // May be any semantic type.
                DKTextICM   text = (DKTextICM)  dsICM.createDDO("S_text",  DKConstant.DK_CM_ITEM);     // Resource Type determined by XDO Classification
                DKImageICM image = (DKImageICM) dsICM.createDDO("S_image", DKConstant.DK_CM_ITEM);     // in the Item Type Definition.

                // Set Mime Type / Type of Content
                lob.setMimeType("application/msword");
                text.setMimeType("text/plain");
                image.setMimeType("image/bmp");
                
                // Store Resource Content & Save Resource Item
                lob.add("SResourceItemICM_Document1.doc");  // Add it to the datastore,
                text.add("SResourceItemICM_Text1.txt");     // storing the content from
                image.add("SResourceItemICM_Picture1.bmp"); // the specified file.

            System.out.println("Created Resource Objects.");

            //-------------------------------------------------------------
            // Deleting Resource Items
            //-------------------------------------------------------------
            // Resource Items are treated just as Non-Resource Items are in
            // regards to deletion.
            System.out.println("Deleting Resource Items...");

                lob.del();
                text.del();
                image.del();
        
            System.out.println("Deleted Resource Items.");

            //-------------------------------------------------------------
            // Delete all items created by Resource Item Samples
            //-------------------------------------------------------------
            // Deleting all Items created by SResourceItemCreationICM,
            // SResourceItemRetrievalICM, & SResourceItemUpdateICM.
            //
            // Please refer to SItemDeletionICM for more informaiton.
            System.out.println("Deleting All Items Used in Resource Item Samples.");

                SItemDeletionICM.deleteAllItems(dsICM,"S_lob");
                SItemDeletionICM.deleteAllItems(dsICM,"S_text");
                SItemDeletionICM.deleteAllItems(dsICM,"S_image");
            
            System.out.println("Deleted All Items Used in Resource Item Samples.");

            //-------------------------------------------------------------
            // Disconnect from datastore & Destroy Reference
            //-------------------------------------------------------------
            // See Sample SConnectDisconnectICM for more information
            System.out.println("Disconnecting from datastore & destroying reference...");            

                dsICM.disconnect();
                dsICM.destroy();

            System.out.println("Disconnected from datastore & destroying reference.");

            //-------------------------------------------------------------
            // Sample program completed without exception
            //-------------------------------------------------------------
            System.out.println("\n==========================================");
            System.out.println("Sample program completed.");
            System.out.println("==========================================\n");
        }
        //------------------------------------------------------------
        // Catch & Print Exceptions        
        //------------------------------------------------------------
        catch (DKException exc){
            SConnectDisconnectICM.printException(exc);  // Print the exception using the function listed below.
            throw(exc);
        } catch (Exception exc) {
            SConnectDisconnectICM.printException(exc);  // Print the exception using the function listed below.
            throw(exc);
        }
    }// end main

    //=================================================================
    // Wrapper Functions
    //=================================================================
    // The following are wrapper functions for functionality covered
    // in this sample.  These functions may be used by other samples.

    // No additional wrapper functions from this sample.

}//end class SResourceItemDeletionICM
              
