/*BEGINPROLOGUE************************************************************************
 * @copyright(disclaimer)                                                             *
 * Licensed Materials - Property of IBM                                               *
 * IBM Enterprise Information Portal for Multiplatforms V8.1 (program number 5724-B43)*
 * (c ) Copyright IBM Corp. 1998,  2002. All Rights Reserved.                         *
 *                                                                                    *
 * US Government Users Restricted Rights                                              *
 * Use, duplication or disclosure restricted by GSA ADP Schedule                      *
 * Contract with IBM Corp.                                                            *
 *                                                                                    *
 * DISCLAIMER OF WARRANTIES :                                                         *
 *                                                                                    *
 * Permission is granted to copy and modify this Sample code, and to                  *
 * distribute modified versions provided that both the copyright                      *
 * notice,- and this permission notice and warranty disclaimer appear in all copies   * 
 * and modified versions.                                                             *
 *                                                                                    *
 * THIS SAMPLE CODE IS LICENSED TO YOU AS-IS.                                         *
 * IBM AND ITS SUPPLIERS AND LICENSORS DISCLAIM ALL WARRANTIES, EITHER EXPRESS OR     *
 * IMPLIED, IN SUCH SAMPLE CODE, INCLUDING THE WARRANTY OF NON-INFRINGEMENT AND THE   *
 * IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE. IN NO   *
 * EVENT WILL IBM OR ITS LICENSORS OR SUPPLIERS BE LIABLE FOR ANY DAMAGES ARISING OUT *
 * OF THE USE OF OR INABILITY TO USE THE SAMPLE CODE, DISTRIBUTION OF THE SAMPLE CODE,*
 * OR COMBINATION OF THE SAMPLE CODE WITH ANY OTHER CODE. IN NO EVENT SHALL IBM OR ITS*
 * LICENSORS AND SUPPLIERS BE LIABLE FOR ANY LOST REVENUE, LOST PROFITS OR DATA, OR   *
 * FOR DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL,INCIDENTAL OR PUNITIVE DAMAGES,       *
 * HOWEVER CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY,-, EVEN IF IBM OR ITS     *
 * LICENSORS OR SUPPLIERS HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.       *
 *                                                                                    *
 * @endCopyright                                                                      *
 **************************************************************************************/

/*
 * Filename       : TDatastorePoolIP.java
 * Description    : test DKDatastorePool for datastore connection pool
 *
 */

import com.ibm.mm.sdk.common.*;
import com.ibm.mm.sdk.cs.*;
import java.io.*;
import java.lang.reflect.*;

// TDatastorePoolIP class
public class TDatastorePoolIP implements DKConstantIP {

  // Main method
  public static void main(String argv[]){
    
      //*** for input parameters
      String libSrv = new String("");   // libserver
      String userid = new String("");   // userid
      String pw     = new String("");   // pw
      String connect_string;    

      String fullClassName = "com.ibm.mm.sdk.server.DKDatastoreIP";

      //annotation support
      connect_string = new String( "APPL=01;FAFIP=9.67.43.83;IODMIP=9.67.43.83;FAFPORT=3061;"
                       + "IODMPORT=3082"
                       + ";FAFPROT=4000;IODMPROT=4000;FAFSITE=CS61;TRACE=ON" );

   // Indicate this is a Console Application on Windows NT/95 for error
   // message display in dos console code page. If this was a Windows
   // GUI (Graphic User Interface) Application this would not be necessary
   // because DK_CM_SS_WINDOWS is the default.
   DKEnvironment.setSubSystem(DK_CM_SS_CONSOLE);

      //-------------------------------------------------------------- 
      // Checking for input parameters
      //-------------------------------------------------------------- 
     
      if (argv.length < 3)
      {
       System.out.println("Usage: " );
       System.out.println("  java TDatastorePoolIP " +
        "<libSrv> <userid> <pw>" );
       return;
      }
      else
      {
       libSrv = argv[0];
       userid = argv[1];
       pw = argv[2];
      } 
    try {
    DKDatastorePool dsrIP = new DKDatastorePool(fullClassName);
    dsrIP.setDatastoreName(libSrv);
    dsrIP.setMaxPoolSize(10);
    dsrIP.setMinPoolSize(3);
    dsrIP.setConnectString(connect_string);

    dsrIP.initConnections(userid, pw, 2);
    dsrIP.setTimeOut(1); 
    System.out.println(" getMaxPoolSize =  " + dsrIP.getMaxPoolSize());
    System.out.println(" getMinPoolSize =  " + dsrIP.getMinPoolSize());
    System.out.println(" getTimeOut=  " + dsrIP.getTimeOut());

    dkDatastore dsIP1, dsIP2, dsIP3, dsIP4;
    dsIP1 = dsrIP.getConnection(userid, pw);             // new ds1
    System.out.println(" getConnection, dsIP1(ds1) =  " + dsIP1);
    dsIP2 = dsrIP.getConnection(userid, pw);             // new ds2
    System.out.println(" getConnection, dsIP2(ds2) =  " + dsIP2);

    DKDatastoreSession dsIdIP1, dsIdIP2, dsIdIP3, dsIdIP4;
    dsIdIP1 = dsrIP.getConnection(userid, pw, 0);        //new dsid1
    System.out.println(" getConnection, dsIdIP1(dsid1)=" +
     dsIdIP1.getDatastore() + ", sessionId=" + dsIdIP1.getId());
    dsIdIP2 = dsrIP.getConnection(userid, pw, 0);        //new dsid2
    System.out.println(" getConnection, dsIdIP2(dsid2)=" +
     dsIdIP2.getDatastore() + ", sessionId=" + dsIdIP2.getId()); 

    dsrIP.returnConnection(dsIdIP2);                     //dsid2 in _freeIdPool
    System.out.println(" returnConnection, dsIdIP2(dsid2) =  " + 
      dsIdIP2.getDatastore() + ", sessionId=" + dsIdIP2.getId());
    dsrIP.returnConnection(dsIP2);                       //ds2 in _freePool
    System.out.println(" returnConnection, dsIP2(ds2) =  " + dsIP2); 

    dsIdIP3 = dsrIP.getConnection(userid, pw, 1); //ds2 in _usedPool
    System.out.println(" getConnection, dsIdIP3(ds2)=" +
     dsIdIP3.getDatastore() + ", sessionId=" + dsIdIP3.getId());
    dsIdIP4 = dsrIP.getConnection(userid, pw, 5); //new dsid3  
    System.out.println(" getConnection, dsIdIP4(dsid3)=" +
     dsIdIP4.getDatastore() + ", sessionId=" + dsIdIP4.getId());


    dsIdIP4 = dsrIP.getConnection(userid, pw, 6); // new dsid4
    System.out.println(" getConnection, dsIdIP4(dsid4)=" +
     dsIdIP4.getDatastore() + ", sessionId=" + dsIdIP4.getId());

    dsIP1 = dsrIP.getConnection(userid, pw);  // new ds3
    System.out.println(" getConnection, dsIP1(ds3) =  " + dsIP1);
    dsIP2 = dsrIP.getConnection(userid, pw);  // new ds4
    System.out.println(" getConnection, dsIP2(ds4) =  " + dsIP2); 

    dsIP3 = dsrIP.getConnection(userid, pw);  // new ds5
    System.out.println(" getConnection, dsIP3(ds5) =  " + dsIP3);
    dsIP4 = dsrIP.getConnection(userid, pw);  // new ds6
    System.out.println(" getConnection, dsIP4(ds6) =  " + dsIP4); 


    dsIdIP3 = dsrIP.getConnection(userid, pw, 9); // dsid2 
    System.out.println(" getConnection, dsIdIP3(dsid2)=" +
     dsIdIP3.getDatastore() + ", sessionId=" + dsIdIP3.getId());


     //**** use dsIP2 to do ListCatalog
/**/
    System.out.println("||**** start: use dsIP2 to do ListCatalog ****||");       
       DKSequentialCollection pCol = null;
       dkIterator pIter = null;
       DKSequentialCollection pCol2 = null;
       dkIterator pIter2 = null;
       String strServerName = null;
       String strServerType = null;
       String strIndexClass = null;
       DKIndexClassDefDL icDef = null;
       int i = 0;
       int j = 0;   
       // List entities...
       DKEntityDefIP docDef = null;
       DKAttrDefIP attrDef = null;
       System.out.println("---List entities---");
       pCol = (DKSequentialCollection)dsIP2.listEntities();
       pIter = null;

       if ( pCol == null )
       {
         System.out.println("collection of entities is null!");
       }
       else
       {
          // Method 1:
          System.out.println("---List attributes using listAttrNames and getAttr methods---");

          pIter = pCol.createIterator();
          while (pIter.more())
          {
             docDef = (DKEntityDefIP)pIter.next();
             System.out.println(" Document type     : " + docDef.getType() );
             System.out.println(" Document type name: " + docDef.getName() );

             String[] attrNames = docDef.listAttrNames();
             int count = attrNames.length;
             for (i=0
                 ; i<count
                 ; i++ )
             {
                   attrDef = (DKAttrDefIP)docDef.getAttr( attrNames[i] );
                   System.out.println("   Attr name     : " + attrDef.getName() );
                   System.out.println("   Attr id       : " + attrDef.getId() );
                   System.out.println("   Entity name   : " + attrDef.getEntityName() );
                   System.out.println("   Datastore name: " + attrDef.datastoreName() );
                   System.out.println("   Attr type     : " + attrDef.getType() );
                   System.out.println("   Attr restrict : " + attrDef.getStringType() );
                   System.out.println("   Attr min val  : " + attrDef.getMin() );
                   System.out.println("   Attr max val  : " + attrDef.getMax() );
                   System.out.println("   Attr display  : " + attrDef.getSize() );
                   System.out.println("   Attr precision: " + attrDef.getPrecision() );
                   System.out.println("   Attr scale    : " + attrDef.getScale() );
                   System.out.println("   Attr update   ? " + attrDef.isUpdatable() );
                   System.out.println("   Attr nullable ? " + attrDef.isNullable() );
                   System.out.println("   Attr queryable? " + attrDef.isQueryable() );
                   System.out.println("" );
             }  // end for
          }  // end while

          // Method 2:
          System.out.println("---List attributes using listEntityAttrs method---");

          pIter = pCol.createIterator();
          while (pIter.more())
          {
             docDef = (DKEntityDefIP)pIter.next();
             System.out.println(" Document type     : " + docDef.getType() );
             System.out.println(" Document type name: " + docDef.getName() );

             DKSequentialCollection pAttrCol = (DKSequentialCollection)dsIP2.listEntityAttrs(docDef.getName() );
             if ( pAttrCol == null )
             {
                System.out.println("collection of entity attrs is null for entity " + docDef.getName() );
             }
             else
             {
                dkIterator pAttrIter = pAttrCol.createIterator();
                while (pAttrIter.more())
                {
                   attrDef = (DKAttrDefIP)pAttrIter.next();
                   System.out.println("   Attr name     : " + attrDef.getName() );
                   System.out.println("   Attr id       : " + attrDef.getId() );
                   System.out.println("   Entity name   : " + attrDef.getEntityName() );
                   System.out.println("   Datastore name: " + attrDef.datastoreName() );
                   System.out.println("   Attr type     : " + attrDef.getType() );
                   System.out.println("   Attr restrict : " + attrDef.getStringType() );
                   System.out.println("   Attr min val  : " + attrDef.getMin() );
                   System.out.println("   Attr max val  : " + attrDef.getMax() );
                   System.out.println("   Attr display  : " + attrDef.getSize() );
                   System.out.println("   Attr precision: " + attrDef.getPrecision() );
                   System.out.println("   Attr scale    : " + attrDef.getScale() );
                   System.out.println("   Attr update   ? " + attrDef.isUpdatable() );
                   System.out.println("   Attr nullable ? " + attrDef.isNullable() );
                   System.out.println("   Attr queryable? " + attrDef.isQueryable() );
                   System.out.println("" );
                }  // end while
             }
          }  // end while
       }
    System.out.println("||**** end: use dsIP2 to do ListCatalog ****||");
/**/
    //****end of use dsIP2 to do LListCatalog

    dsrIP.returnConnection(dsIP2);
    System.out.println(" returnConnection, dsIP2(ds4) =  " + dsIP2);
    dsIP4 = dsrIP.getConnection(userid, pw);
    System.out.print(" getConnection, dsIP4 =  " + dsIP4); 
    System.out.println("    dsIP4 == dsIP2 =  " + (dsIP4 == dsIP2)); //true

    //**** you can add codes to use dsIP4 to do other thing here  
    //**** end of use dsIP4 to do other thing
       
    dsrIP.returnConnection(dsIP1);
    System.out.println(" returnConnection, dsIP1(ds3) =  " + dsIP1);
    dsrIP.returnConnection(dsIP3);
    System.out.println(" returnConnection, dsIP3(ds5) =  " + dsIP3);
    dsrIP.returnConnection(dsIdIP4); //dsid4 in _freeIdPool
    System.out.println(" returnConnection, dsIdIP4(dsid4) =  " + 
      dsIdIP4.getDatastore() + ", sessionId=" + dsIdIP4.getId());
     

    System.out.println("**** start sleep for 120050 mini sec*******");
    Thread.sleep(120050);  //the two in freePool will be disconnected
    System.out.println("**** end sleep for 120050 mini sec*******");

    dsIP2 = dsrIP.getConnection(userid, pw);     //new ds7
    System.out.println(" getConnection, dsIP2(ds7) =  " + dsIP2); 
    System.out.println(" call destroy()..." );
    dsrIP.destroy();  //take a while for all 4 in usedPool to be disconnected

    }
    catch (DKException exc)
    {
     System.out.println("Exception name " + exc.name());
     System.out.println("Exception message " + exc.getMessage());
     exc.printStackTrace();
    }
    catch (Exception exc)
    {
     System.out.println("Exception message " + exc.getMessage());
     exc.printStackTrace();
    }

  }
}

