// IBM Confidential OCO Source Material
// 5648-C83, 5648-C84 (C) COPYRIGHT International Business Machines Corp. 1997,1998,1999
// The souce code for this program is not published or otherwise divested
// of its trade secrets, irrespective of what has been deposited with the
// U.S. Copyright office.

import java.rmi.*;
import java.io.*;
import javax.servlet.*;
import javax.servlet.http.*;
import com.ibm.websphere.security.SSOAuthenticator;

/**
The AbstractLoginServlet implements basic functionality to perform login and exploiting
Single Sign-On feature by using the SSOAuthenticator.
 **/

public abstract class AbstractLoginServlet extends HttpServlet{

    private String userid;
    private String password;
    private String redirectURL;
    private ServerSideAuthenticator authenticator;
    
    public void init() {
        authenticator = new ServerSideAuthenticator();
    }
    

    protected void setUserData(String uid, String pwd) {

        userid = uid;
        password = pwd;
    }

    protected void setRedirectURL(String url) {

        redirectURL = url;
    }

    /*
    Based on the userid and password, authenticate the user;
    If setSSO is set to true, then set up appropriate SSO cookie on the header
    so that Websphere will recognize it when secure resources are accessed.

     */
    public Object login(HttpServletRequest req, HttpServletResponse res, boolean setSSO)
    throws ServletException {

        PrintWriter out =null;
        try {
            out =  res.getWriter();
        } catch (java.io.IOException e){
        }

        boolean success = false;
        org.omg.SecurityLevel2.Credentials retCreds = null;
        try {
            retCreds = authenticator.login(userid, password, true);
            // set the invocation credentials so that
            // the postlogin method will be executed under
            // this client's identity
            authenticator.setInvocationCredentials(retCreds);
        } catch(Exception e) {
            // catch all possible exceptions if you want or handle them separately
            throw new ServletException();
        }

        // apart from performing authentication, add the SSO cookie to the browser so that
        // websphere can use that instead of prompting for userid and password
        // or in the case of Custom challenge type will not redirect the
        // user back to the login form
        if (setSSO) {
            try {
                setupSSO(userid, password, req, res);
            } catch (RemoteException e) {
                throw new ServletException("Error setting single sign-on");
            } catch (org.omg.SecurityLevel2.LoginFailed e) {
                throw new ServletException("Login Failed");
            }
        }

        // do whatever is required of postlogin
        // It should be called here so that the accessID is set properly and
        // any action inside postLogin (e.g., creating an IBM Session) will
        // work under the client's identity.
        postLogin(req, res);

        // redirect may fail if setupSSO failed
        try {
            res.sendRedirect(redirectURL);
        } catch (java.io.IOException e) {
            throw new ServletException("Error redirecting to URL: "+redirectURL);
        }

        return retCreds;

    }

    public abstract void postLogin(HttpServletRequest req, HttpServletResponse res)
    throws ServletException;

    public void logout(HttpServletRequest req, HttpServletResponse res, Object retCreds)
    throws ServletException {

        try {
            authenticator.setInvocationCredentials(null);
            unsetSSO(req, res);

        } catch (Exception e) {
            throw new ServletException(e);
        }

    }


    /*
    Sets up the SSO cookie on the response header
     */
    private void setupSSO(String userid, String password,
    HttpServletRequest req, HttpServletResponse res)
    throws java.rmi.RemoteException,
    org.omg.SecurityLevel2.LoginFailed {

        SSOAuthenticator ssoAuth = new SSOAuthenticator();
        ssoAuth.login(userid, password, req, res);

    }

    /*
    log the user out by deleting the SSOCookie
     */
    private void unsetSSO(HttpServletRequest req, HttpServletResponse res)
    throws java.rmi.RemoteException {

        SSOAuthenticator ssoAuth = new SSOAuthenticator();
        ssoAuth.logout(req, res);
    }


    protected String getRefererURL(HttpServletRequest req,
    HttpServletResponse res) {

        SSOAuthenticator ssoAuth = new SSOAuthenticator();
        return ssoAuth.getRefererURL(req, res);
    }
}



